/*  Generador de codi java del CPC.
 *  Autor: Carles Royan Salvatella
 *  Ultima modificacio: 18/06/2003
 *  Dependencies: PseudocodiTokenTypes,ParamInfo.
 */
import java.io.*;
/**
 * <p>Classe que s'encarrega d'escriure en un fitxer la traduccio al Java de 
 * les instruccions del pseudocodi.
 * </p>
 * @author Carles Royan Salvatella
 * @version 1.0 - Data: 18/06/2003
 */
class CpcGenJava implements PseudocodiTokenTypes
{
	private BufferedWriter out;
	private String version,className;
	private int numIdent=0;
	private boolean isDefaultConstructor=false;
	private boolean verboseMode=false;

	public CpcGenJava()
	{
		// Constructor per defecte
	}
	/** Constructor. filename es el nom del fitxer de sortida, ver es la versio
	    del traductor i verbose especifica si esta activat el mode verbose.
	*/
	public CpcGenJava(String filename,String ver,boolean verbose)
	{
		try
		{
			version=ver;
			verboseMode=verbose;
			out=new BufferedWriter(new FileWriter(filename));
		}
		catch (IOException e)
		{
			System.err.println(e);
		}
	}
	/** Tanca el fitxer de sortida.	*/	
	public void closeFile()
	{
		try
		{
			out.close();
		}
		catch (IOException e)
		{
			System.err.println(e);
		}		
	}
	/**  Comprova si s'ha escrit un constructor per defecte. En cas negatiu l'escriu ara. */
	public void checkDefaultConstructor()
	{
		try
		{
			if (!isDefaultConstructor)
			{
				if (verboseMode)
					System.err.println("[Afegint constructor per defecte : " + className + "]");
				writeIdent();
				out.write("public " + className + "() {"); sepaInst();
				incIdent(); writeIdent();
				out.write("super();"); sepaInst();
				decIdent(); writeIdent();
				out.write("}"); sepaInst();
			}
		}
		catch (IOException e)
		{
			System.err.println(e);
		}
	}
	/**  Escriu la capzalera del fitxer traduit. */	
	private void writeFileHead()
	{
		try
		{
			writeIdent();
			out.write("/* Fitxer generat per: CPC "+version+" */");
			sepaInst(); sepaInst();
			out.write("import java.io.BufferedReader;"); sepaInst();
			out.write("import java.io.InputStreamReader;"); sepaInst();
		}
		catch (IOException e)
		{
			System.err.println(e);
		}			
	}
	/** Escriu la declaracio de la funcio principal (main) en java. */
	public void writeHeadOfMain()
	{
		try
		{
			writeIdent();
			out.write("public static void main(String[] args) { "); sepaInst();
			incIdent();
		}
		catch (IOException e)
		{
			System.err.println(e);
		}		
	}
	/** Escriu la declaracio de la classe, i declara variables
		auxiliars per poder fer E/S. */	
	public void writeHeadOfClass(String cName,String superClassName)
	{
		try
		{
			writeFileHead();
			writeIdent();
			className=cName;
			out.write("class "+className);
			if (!superClassName.equals("")) // Escriu l'herencia de la classe
				out.write(" extends " + superClassName);
			out.write(" {"); sepaInst();
			// Per a poder llegir dades amb "llegir", de l'entrada estandar
			sepaInst();	out.write("private static _________InputOutput _________inputing=new _________InputOutput();");
			sepaInst(); sepaInst();
			incIdent();
		}
		catch (IOException e)
		{
			System.err.println(e);
		}		
	}
	/** Retorna un string que representa el tipus elemental	 */
	private String getStringOfElementalType(int type)
	{
		String text;
		
		switch(type)
		{
			case NUMERO_ENTER:
				text="int";
				break;
			case NUMERO_REAL:
				// Els numeros reals els agafem per defecte double
				text="double";
				break;
			case BOOLEA:
				text="boolean";
				break;
			case LITERAL_CARACTER:
				text="char";
				break;
			default:
				text="int";
				break;
		}
		return text;		
	}
	/** Retorna un string amb l'expressio "crear" traduida al java	*/
	public String getExpressionCrear(int type,String textType)
	{
		String text;			
		
		text=textType;
		if (text=="")  // Es un tipus elemental
			text=getStringOfElementalType(type);
		text= "new " + text;
		
		return text;
	}
	/** Retorna un String amb l'expressio llegir traduida al java. Es fa us
		de la classe d'entrada i sortida */
	public String getExpressionLlegir(int type)
	{
		String text="";

		switch(type)
		{
			case NUMERO_ENTER:
				text="_________inputing.getEnter()";
				break;
			case NUMERO_REAL:
				// Els numeros reals els agafem per defecte double
				text="_________inputing.getReal()";
				break;
			case BOOLEA:
				text="_________inputing.getBoolea()";
				break;
			case LITERAL_CARACTER:
				text="_________inputing.getCaracter()";
				break;
		}
		
		return text;
	}
	/** Escriu el final d'un bloc de codi */
	public void writeEndBlock()
	{
		try
		{
			decIdent();
			writeIdent();
			out.write("}"); sepaInst();
		}
		catch (IOException e)
		{
			System.err.println(e);
		}		
	}
	/** Escriu la declaracio d'una variable. name es el Identificador, idType el tipus de ident., type el tipus
	    de variale, typeClassName es el nom de la classe, typeDecl es el modificador d'acces. */
	public void writeDeclaVar(String name,int idType,int type,int dimension,String typeClassName,int typeDecl)
	{
		try
		{
			writeIdent();
			// Escriu el modificador d'accessibilitat
			switch(typeDecl)
			{
				case ATRIBUTS:
					out.write("public ");
					break;
				case ATRIBUTS_CLASSE:
					out.write("public static ");
					break;
				case IMPLEMENTACIO:
					out.write("protected ");
					break;
				case RES:
					// res //
					break;
			}
			// Escriu el tipus
			switch(idType)
			{
				case ELEMENTAL_TYPE:		
					out.write(getStringOfElementalType(type));
					break;
				case CLASS_TYPE:
					out.write(typeClassName);
					break;
			}
			// Escriu les dimensions, si es tracta d'una taula
			for(int i=0; i < dimension; i++)
				out.write("[]");

			// Escriu el nom
			out.write(" " + name);
			writeValIni(idType,type,dimension);
			out.write(";");
			sepaInst();
		}
		catch (IOException e)
		{
			System.err.println(e);
		}			
	}
	/** Escriu el valor inicial de la variable. Totes les variables tenen un valor
	    inicial per defecte. */
	private void writeValIni(int idType,int type,int dimension)
	{
		try
		{
			if (dimension==0)
			{
				// Inicialitza el valor de la variable
				switch(idType)
				{
					case ELEMENTAL_TYPE:		
						switch (type)
						{
							case NUMERO_ENTER:
								out.write("=0");
								break;
							case NUMERO_REAL:
								out.write("=0.0");
								break;
							case BOOLEA:
								out.write("=false");
								break;
							case LITERAL_CARACTER:
								out.write("=' '");
								break;							
						}
						break;
					case CLASS_TYPE:
						out.write("=null");
						break;
				}
			}
			else
			{
				// Inicialitza el valor de la taula
				out.write("=null");
			}
		}
		catch (IOException e)
		{
			System.err.println(e);
		}				
	}
	/** Declara una constant. name es el nom, type el tipus, expressio el seu valor i typeDecl 
		el modificador d'acces. */
	public void writeDeclaConst(String name,int Type,String expressio,int typeDecl)
	{
		try
		{
			writeIdent();
			// Escriu el modificador d'accessibilitat
			switch(typeDecl)
			{
				case ATRIBUTS_CLASSE:
					out.write("public static ");
					break;
				case IMPLEMENTACIO:
					out.write("protected ");
					break;
				case RES:
					// res //
					break;
			}
			switch(Type)
			{
				case NUMERO_ENTER:
					out.write("final int " + name + "=" + expressio + ";");
					break;
				case NUMERO_REAL:
					// Els numeros reals els agafem per defecte double
					out.write("final double " + name + "=" + expressio + ";");
					break;
				case BOOLEA:
					out.write("final boolean " + name + "=" + expressio + ";");
					break;
				case LITERAL_CARACTER:
					out.write("final char " + name + "=" + expressio + ";");
					break;					
			}
			sepaInst();
		}
		catch (IOException e)
		{
			System.err.println(e);
		}			
	}
	/** Escriu la declaracio d'una accio (VOID) */
	public void writeHeadAccio(int funcType,String name,ParamInfo[] params,int numParams)
	{
		try
		{
			writeIdent();
			switch(funcType)
			{
				case FUNCIO:
					out.write("protected void " + name + "(");
					break;
				case METODE:
					out.write("public void " + name + "(");
					break;
				case METODE_CLASSE:
					out.write("protected static void " + name + "(");
					break;						
				case CONSTRUCTOR:
					if (numParams==0)  // Estableix que ja s'ha escrit un constructor per defecte
						isDefaultConstructor=true;
					out.write("protected " + className + "(");
					break;
				case DESTRUCTOR:
					out.write("protected void finalize ("); // Manera d'implementar-se destructors en java
					break;
			}
			writeFuncParams(params,numParams);
			out.write(") {"); sepaInst();
			incIdent();			
		}
		catch (IOException e)
		{
			System.err.println(e);
		}	
	}
	/** Escriu la declaracio d'una funcio que retorna un valor (retParam).	 */
	public void writeHeadFuncio(int funcType,String name,ParamInfo[] params,int numParams,ParamInfo retParam)
	{
		try
		{
			writeIdent();
			switch(funcType)
			{
				case FUNCIO:
					out.write("protected ");
					break;
				case METODE:
					out.write("public ");
					break;
				case METODE_CLASSE:
					out.write("public static ");
					break;					
			}
			if (retParam.getIdType()==ELEMENTAL_TYPE)
				out.write(getStringOfElementalType(retParam.getType()) + " ");
			else
				out.write(retParam.getClassName() + " ");
			
			out.write(name + "(");
			writeFuncParams(params,numParams);
			out.write(") {"); sepaInst();
			// Declara el parametre de retorn
			writeIdent();
			if (retParam.getIdType()==ELEMENTAL_TYPE)
				out.write(getStringOfElementalType(retParam.getType()) + " " + retParam.getName().toLowerCase() + ";");
			else // Es una classe, la declara i la inicialitza per poder-la utilitzar directament en pseudocodi
				out.write(retParam.getClassName() + " " + retParam.getName().toLowerCase() +
						  "= new " + retParam.getClassName() + "();");
			sepaInst();
			incIdent();
		}
		catch (IOException e)
		{
			System.err.println(e);
		}	
	}
	/** Escriu en el fitxer els parametres d'un metode	*/
	private void writeFuncParams(ParamInfo[] params,int numParams)
	{
		try
		{	
			if (numParams > 0)
			{
				if (params[0].getIdType()==ELEMENTAL_TYPE)
					out.write(getStringOfElementalType(params[0].getType()) + " ");
				else
					out.write(params[0].getClassName() + " ");
				out.write(params[0].getName().toLowerCase());		
			}
			for(int i=1; i < numParams; i++)
			{
				if (params[i].getIdType()==ELEMENTAL_TYPE)
					out.write("," + getStringOfElementalType(params[i].getType()) + " ");
				else
					out.write(params[i].getClassName() + " ");				
				out.write(params[i].getName().toLowerCase());
			}
		}
		catch (IOException e)
		{
			System.err.println(e);
		}	
	}
	/** Escriu la capzalera de la inst. mentre, traduida com a while. */
	public void writeInstHeadMentre(String expression)
	{
		try
		{
			writeIdent();
			out.write("while (" + expression + ") {"); sepaInst();
			incIdent();
		}
		catch (IOException e)
		{
			System.err.println(e);
		}			
	}
	/** Escriu una crida a una accio/funcio	*/
	public void writeInstCrida(String crida)
	{
		try
		{
			writeIdent();
			out.write(crida + ";"); sepaInst();
		}
		catch (IOException e)
		{
			System.err.println(e);
		}			
	}
	/** Escriu "escriure" utilitzant la classe d'entrada i sortida */
	public void writeInstEscriure(String expression)
	{
		try
		{
			writeIdent();
			out.write("_________inputing.escriure(" + expression + ");"); sepaInst();
		}
		catch (IOException e)
		{
			System.err.println(e);
		}
	}
	/** Escriu "escriureln" utilitzant la classe d'entrada i sortida */
	public void writeInstEscriureln(String expression)
	{
		try
		{
			writeIdent();
			out.write("_________inputing.escriureln(" + expression + ");"); sepaInst();
		}
		catch (IOException e)
		{
			System.err.println(e);
		}			
	}
	/** Escriu la inst. per com a FOR */
	public void writeInstPer(String indexVar,String iniExp,String endExp,String pas)
	{
		try
		{
			String comp="";
			
			if (pas.equals(""))
			{
				comp="<="; // Correspon a bucles cap endavant
				pas=indexVar + "=" + indexVar +"+1";
			}
			else
			{
				if (Integer.parseInt(pas) < 0)
					comp=">="; // Correspon a bucles cap enrera
				else
					comp="<="; // Correspon a bucles cap endavant				
				pas=indexVar + "=" + indexVar + "+(" + pas + ")";
			}
			writeIdent();
			out.write("for (" + indexVar + "=" + iniExp + ";" + indexVar + comp +
		  			 endExp + ";" + pas + ") {"); sepaInst();
			incIdent();
		}
		catch (IOException e)
		{
			System.err.println(e);
		}	
	}
	/** Escriu en el fitxer la instrucci "si" com a IF */
	public void writeInstSi(String expression)
	{
		try
		{
			writeIdent();
			out.write("if (" + expression + ")" + " {"); sepaInst();
			incIdent();
		}
		catch (IOException e)
		{
			System.err.println(e);
		}			
	}
	/** Escriu en el fitxer la instrucci altrament (ELSE)	*/
	public void writeInstAltrament()
	{
		try
		{
			writeIdent();
			out.write("else {"); sepaInst();
			incIdent();
		}
		catch (IOException e)
		{
			System.err.println(e);
		}			
	}
	/** Escriu en el fitxer la instrucci "cas" (la primera condicio) com a IF */
	public void writeInstCasPrimer(String expression)
	{
		writeInstSi(expression);
		incIdent();
	}
	/** Escriu en el fitxer la instrucci "cas" (les cond. seguents a la primera)
	  	com a ELSE IF */
	public void writeInstCasAltres(String expression)
	{
		try
		{
			writeIdent();
			out.write("else if (" + expression + ")" + " {"); sepaInst();
			incIdent();
		}
		catch (IOException e)
		{
			System.err.println(e);
		}	
	}
	/** Escriu la inst d'assignacio */
	public void writeInstAssig(String varName,String expression)
	{
		try
		{
			writeIdent();
			out.write(varName + "=" + expression + ";"); sepaInst();
		}
		catch (IOException e)
		{
			System.err.println(e);
		}			
	}
	/** Escriu la instruccio retorna (RETURN) */
	public void writeInstRetorna(String varName)
	{
		try
		{
			writeIdent();
			out.write("return " + varName + ";"); sepaInst();
		}
		catch (IOException e)
		{
			System.err.println(e);
		}			
	}
	/** <p>Escriu la instruccio destruir ID.</p>
		<p>	El que es fa es igualar a null, de tal manera que el GC  
		s'encarregara de lliberar l'espai de mem.</p>*/
	public void writeInstDestruir(String varName)
	{
		try
		{
			writeIdent();
			out.write(varName + "=null;"); sepaInst();
		}
		catch (IOException e)
		{
			System.err.println(e);
		}			
	}
	/** Escriu la instruccio res com un comentari */
	public void writeInstRes()
	{
		try
		{
			writeIdent();
			out.write("// RES //"); sepaInst();
		}
		catch (IOException e)
		{
			System.err.println(e);
		}			
	}
	/** Afegeix identacio cap a la dreta */	
	private void incIdent()
	{
		numIdent++;
	}
	/** Treu identacio (text cap a l'esquerra) */	
	private void decIdent()
	{
		numIdent--;
	}
	/** Escriu la identacio en el fitxer de sortida */	
	private void writeIdent()
	{
		try
		{
			int i;
			for(i=1;i<=numIdent;i++)
				out.write("\t");
		}
		catch (IOException e)
		{
			System.err.println(e);
		}
	}
	/** Escriu salt de linia. */	
	public void sepaInst()
	{
		try
		{ out.write("\n"); }
		catch (IOException e)
		{ System.err.println(e); }		
	}
}
