// $ANTLR 2.7.1: "cpc.g" -> "CpcParser.java"$

import java.io.*;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.ListIterator;

import antlr.TokenBuffer;
import antlr.TokenStreamException;
import antlr.TokenStreamIOException;
import antlr.ANTLRException;
import antlr.LLkParser;
import antlr.Token;
import antlr.TokenStream;
import antlr.RecognitionException;
import antlr.NoViableAltException;
import antlr.MismatchedTokenException;
import antlr.SemanticException;
import antlr.ParserSharedInputState;
import antlr.collections.impl.BitSet;
import antlr.collections.AST;
import antlr.ASTPair;
import antlr.collections.impl.ASTArray;

/**
 * <p>Classe que incorpora l'analisis sintactica del compilador. Es la classe principal que parseja
 *  el fitxer de pseudocodi. A m&eacute;s executa les comprovacions sem&aacute;ntiques i la generaci&oacute; de codi.
 * <p>Abans de parserjar un programa, es millor establir primer les seguents propietats:</p>
 * <p>
 * <ul>
 *			<li>{@link #setVersion(java.lang.String) setVersion(String)}</li>
 *			<li>{@link #setVerboseMode(boolean) setVerboseMode(boolean)}</li>
 *			<li>{@link #setInputFilename(java.lang.String) setInputFilename(String)}</li>
 * </ul>
 * </p>
 * <p>Finalment parsejar el programa cridant:</p>
 * {@link #programa(boolean) programa(boolean)}
 * </p>
 * @author Carles Royan Salvatella
 * @version 1.0 - Data: 18/06/2003
 */
public class CpcParser extends antlr.LLkParser
       implements PseudocodiTokenTypes
 {

/* ******************************** Variables de la classe ****************************** */
/** Constant per identificar quan un ID o un camp d'una classe no s'ha declarat. */
private final int ID_NOT_DECLARED=-1;
/** Constant que defineix l'extensi&oacute; per defecte dels fitxers pseudocodi d'entrada. */
private final String EXT_DEF=".pse";
/** <p>Constant que defineix un m&egrave;tode acceptat en les taules.</p><p> nomTaula.longitud() 
 *  retorna un enter que especifica la llarg&agrave;ria amb que s'ha instanciat la taula.</p> */
private final String LONGITUT_OF_TAULA="longitud";
/** La taula de s&iacute;mbols utilitzada per el traductor */
private SymbolTable symbolTable=new SymbolTable();
/** S'encarrega de generar el codi Java de sortida. */
private CpcGenJava out;
/** Nom del fitxer d'entrada que es tracta. S'utilitza quan es reporten missatges d'error. */
private String inFilename;
/** Nom del fitxer de sortida que es genera. */
private String outFile;
/** Versi&oacute; del traductor, que s'imprimeix en els fitxers de sortida generats. */
private String version;
/** Auxiliar, per saber la info de la variable en una inst. d'assignacio. */
private ParamInfo auxExpAssig=null;
/** Informaci&oacute; del traductor, per imprimir al final del tractament. */
private int numErrors=0,numAvisos=0;
/** Per saber si <b>super()</b> &eacute;s la primera instrucci&oacute; d'un constructor */
private boolean isFirstInstruction=false;
/** Especifica si el mode verbose est&agrave; activat.<p>En cas afirmatiu treu missatges
    informatius durant el proc&eacute;s de traducci&oacute; */
private boolean verboseMode=false;
/* ******************************** Fi Variables de la classe ****************************** */
/** <p>Sobreescriu la funcio de reportar errors sint&agrave;ctics.</p>
	<p>Canvia una mica el missatge desat en l'excepcio per a que sigui igual al
	que surt quan es reporten missatges sem&agrave;ntics. En acabat, incrementa
	la variable de comptatge d'errors.</p> */
public void reportError(RecognitionException ex)
{
	System.err.print(inFilename + ":");
	System.err.println(ex.toString().substring(5));
	numErrors=numErrors+1;
}
/** <p>Funcio de reportar errors sem&agrave;ntics.</p>
    <p>Imprimeix per pantalla l'error i incrementa la variable de comptatge
	d'errors.</p> */
public void reportError(PSemanticException ex)
{
	System.err.println(ex);
	numErrors=numErrors+1;
}
/** <p>Sobreescriu la funci&oacute; de reportar avisos (Warnings).</p>
    <p>Imprimeix per pantalla l'av&iacute;s i incrementa la variable de comptatge
	d'avisos.</p> */
public void reportWarning(PSemanticWarning ex)
{
	System.err.println(ex);
	numAvisos=numAvisos+1;
}
/** Retorna la funci&oacute; amb els tipus dels parametres formatejats, de tal manera
	que s&oacute;n llegibles per l'usuari. */
private String getFuncToString(String varName,ParamInfo[] parParams,int numParams)
{
	String text;
	int i;
	
	text="";
	i=0;
	while (i < numParams)
	{
		text=text + "," + getTypeName(parParams[i]);
		i++;
	}
	if (text!="")
		text=text.substring(1);
	text=varName+"(" + text + ")";
	return text;
}
/** Retorna un String que representa el nom del tipus, ja sigui elemental o una classe. */
private String getTypeName(ParamInfo parInf)
{
	String typeName="";
	
	if (parInf.getIdType()==CLASS_TYPE)
		typeName=parInf.getClassName();
	else
		typeName=getElementalTypeName(parInf.getType());
	
	return typeName;
}
/** <p>Retorna el nom del tipus elemental passat com constant entera representativa (enter, real,...).</p>
    <p>La constant est&agrave; definida en la interf&iacute;cie PseudocodiTokenTypes.</p> */
private String getElementalTypeName(int type)
{
	String typeName="";
	
	switch(type)
	{
		case NUMERO_ENTER:
			typeName="enter";
			break;
		case NUMERO_REAL:
			typeName="real";
			break;
		case BOOLEA:
			typeName="boolea";
			break;
		case LITERAL_CARACTER:
			typeName="caracter";
			break;
		case LITERAL_CADENA:  // Al existir el literal, tb es pot passar per parametre a una func.
			typeName="cadena";
			break;
	}
	return typeName;
}
/** Retorna un numero &uacute;nic representatiu pera  cada classe de pseudocodi amb una funcio hash. */
private int getHash(String className)
{
	return className.hashCode();
}
/** Declara una variable en l'ambit actual de la TS */
private void 
declararVariable(String varName,int idType,int type,int dimension,String typeClassName,int typeDecl,int linia)
{
	TNode node;
	LinkedList list;
	
	try
	{
		// A causa del CASE_SENSITIVE,s'accedeix sempre en minuscules a la taula de simbols.
		if (symbolTable.lookupNameInCurrentScope2(varName.toLowerCase())!=(LinkedList)null)  
			throw new IdAlreadyDeclaredException(varName,inFilename,linia);
		else
		{
			node=new TNode();
			node.setType(type);
			node.setIdType(idType);
			node.setDimension(dimension);
			node.setText(typeClassName);
			switch(typeDecl)
			{
				case ATRIBUTS:
					node.setAccesMode(PUBLIC);
					break;
				case ATRIBUTS_CLASSE:
					node.setAccesMode(MEMBRE_CLASSE);
					break;
				default:
					node.setAccesMode(PRIVATE);
					break;
			}
			list=new LinkedList();
			list.add(node);								
			symbolTable.add(varName.toLowerCase(),list); // A causa del CASE_SENSITIVE,es tracta sempre en MIN en la TS.
		}
		// Escriu en el fitxer desti
		out.writeDeclaVar(varName.toLowerCase(),idType,type,dimension,typeClassName,typeDecl);
	}
	catch (IdAlreadyDeclaredException sEx)
	{
		reportError(sEx);
	}
}
/** Declara una constant en l'ambit actual de la TS */
private void declararConstant(String varName,int Type,String expressio,int typeDecl,int linia)
{
	TNode node;
	LinkedList list;
	
	try
	{
		// A causa del CASE_SENSITIVE,s'accedeix sempre en minuscules a la taula de simbols.
		if (symbolTable.lookupNameInCurrentScope2(varName.toLowerCase())!=null)
			throw new IdAlreadyDeclaredException(varName,inFilename,linia);
		else
		{
			node=new TNode();
			node.setType(Type);
			node.setIdType(CONST);	// Estableix que l'identificador correspon a una constant.
			switch(typeDecl)
			{
				case ATRIBUTS_CLASSE:
					node.setAccesMode(MEMBRE_CLASSE);
					break;
				default:
					node.setAccesMode(PRIVATE);
					break;
			}
			list=new LinkedList();  // El polimorfisme fa que tinguem una llista per cada entrada.
			list.add(node);			// En aquest cas nomes tractarem el primer element.
			symbolTable.add(varName.toLowerCase(),list);  // A causa del CASE_SENSITIVE,es tracta sempre en MIN en la TS.
		}
		// Escriu en el fitxer desti
		out.writeDeclaConst(varName.toLowerCase(),Type,expressio,typeDecl);
	}
	catch (IdAlreadyDeclaredException sEx)
	{
		reportError(sEx);
	}
}
/** Declara una funcio/metode/metode de classe/constructor/destructor en la TS */
private void 
declararFuncio(String scopeName,ParamInfo[] params,int numParams,ParamInfo retParam,boolean declararAmbit,int linia,int funcType)
{
	int i=0;
	TNode node;
	LinkedList list;
	
	if (declararAmbit)
	{
		if (funcType==FUNCIO)
			// Es una funcio privada, l'inserta en l'ambit actual per poder ser cridada.
			declararFuncioAmbitActual(scopeName,params,numParams,retParam,funcType,linia);		
		else
		{
			if (funcType!=DESTRUCTOR)
				// Comprova que existeix la declaracio en la interficie
				checkIsDefinedInInterficie(scopeName,params,numParams,linia);
		}
		// Crea un nou ambit per la funcio
		symbolTable.pushScope(scopeName);
		// Estableix el tipus d'ambit
		symbolTable.setTypeOfScope(funcType);		
		while ((params[i]!=null) && (i < params.length))
		{
			node=new TNode();
			/**/
			node.setType(params[i].getType());
			node.setIdType(params[i].getIdType());
			node.setText(params[i].getClassName());
			/**/
			node.setAccesMode(PRIVATE);

			list=new LinkedList();  // El polimorfisme fa que tinguem una llista per cada entrada.
			list.add(node);
			symbolTable.add(params[i].getName().toLowerCase(),list);  // afegeix els parametres en minuscules per el CASE SENSITIVE
			i++;
		}
		if (retParam!=null)
		{
			node=new TNode();
			/**/
			node.setType(retParam.getType());
			node.setIdType(retParam.getIdType());
			node.setText(retParam.getClassName());
			/**/
			node.setAccesMode(PRIVATE);
			
			list=new LinkedList();
			list.add(node);	
			symbolTable.add(retParam.getName().toLowerCase(),list);	 // Param. de retorn
			// Realitza la traduccio
			out.writeHeadFuncio(funcType,scopeName.toLowerCase(),params,numParams,retParam);	
		}
		else
		{
			// Realitza la traduccio
			out.writeHeadAccio(funcType,scopeName.toLowerCase(),params,numParams);
		}
		// Indica que la proxima instruccio, es la primera de l'ambit local
		isFirstInstruction=true;
	}
	else
	{
		// Membres d'interficie: introdueix en l'ambit actual i permet ser cridada.
		declararFuncioAmbitActual(scopeName,params,numParams,retParam,funcType,linia);
	}
}
/** Declara una funcio/metode/metode de classe/constructor/destructor en l'ambit actual, de tal manera
	que sigui visible en l'ambit i pugui ser cridada. */
private void declararFuncioAmbitActual(String scopedName,ParamInfo[] params,int numParams,ParamInfo retParam,int funcType,int linia)
{
	TNode node;
	LinkedList list;
	boolean isDefined;
	FuncInfo fInfo;
	String scopeName;
	
	try
	{
		scopeName=scopedName.toLowerCase();
		// Comprova que el nom per la funcio sigui correcte
		if ((funcType==CONSTRUCTOR) && (!scopeName.equalsIgnoreCase(symbolTable.getParentScopeName())))
			throw new ConstructorNotValidException(inFilename,linia);
		else if ((funcType!=CONSTRUCTOR) && (scopeName.equalsIgnoreCase(symbolTable.getParentScopeName())))
			throw new FuncIdNotValidException(inFilename,linia);
		else
		{	// El nom de la funcio es correcte, fa les comprovacions de visibilitat i polimorfisme
			list=symbolTable.lookupNameInCurrentScope2(scopeName);
			if (list!=(LinkedList)null)
			{
				isDefined=isDefinedFuncInList(list,params,numParams);
			}
			else
			{
				isDefined=false;
				list=new LinkedList();  // El polimorfisme fa que tinguem una llista per cada entrada.
			}
			if (!isDefined)
			{
				node=new TNode();
				node.setParamInfo(params);
				node.setNumParams(numParams);
				if (retParam!=null)
				{
					node.setType(retParam.getType());   // Param. de retorn
					node.setIdType(FUNCIO);				// Especifica que el ID es una funcio
					if (retParam.getIdType()==CLASS_TYPE)
					{
						// El parametre de retorn es una classe, desa el seu nom
						node.setText(retParam.getClassName());
					}
				}
				else
					node.setIdType(ACCIO);		// Al no tenir param. de retorn es una accio
				// Estableix la visibilitat o el mode d'acces
				switch(funcType)
				{
					case CONSTRUCTOR:
						node.setAccesMode(PUBLIC);
						break;				
					case METODE:
						node.setAccesMode(PUBLIC);
						break;
					case METODE_CLASSE:
						node.setAccesMode(MEMBRE_CLASSE);
						break;				
					case FUNCIO:
						node.setAccesMode(PRIVATE);
						break;			
				}
				list.add(node);
				symbolTable.add(scopeName,list);
				if (funcType!=FUNCIO)
				{
					// Afegeix a la llista de membres de l'interficie
					fInfo=new FuncInfo();
					fInfo.setName(scopedName);	// Aqui l'afegeix conservant majusc, per si ha de mostrar error
					fInfo.setNode(node);
					symbolTable.listFunctions.add(fInfo);
				}
			}
			else
			{
				throw new IdAlreadyDeclaredException(getFuncToString(scopedName,params,numParams),inFilename,linia);
			}
		}
	}
	catch (PSemanticException sEx)
	{
		reportError(sEx);
	}	
}
/** Comprova si la funcio est&agrave; definida, tot comprovant la signatura de les funcions de la llista. */
private boolean isDefinedFuncInList(LinkedList list,ParamInfo[] params,int numParams)
{
	boolean eq=false;
	ParamInfo[] fParams;
	int fNumParams;
	ListIterator it;
	TNode node;
	
	it=list.listIterator(0);
	while ((it.hasNext()) && (!eq))
	{
		node=(TNode)it.next();
		if ((node.getIdType()==FUNCIO) || (node.getIdType()==ACCIO))
			eq=isEqualsParamsFunc(params,numParams,node.getParamInfo(),node.getNumParams());
	}
	return eq;
}
/** Comprova si els parametres d'una funcio son iguals als d'un altra. Retorna cert si son iguals. */
private boolean isEqualsParamsFunc(ParamInfo[] params1,int nParams1,ParamInfo[] params2,int nParams2)
{
	boolean eq=true;
	int nPar;
	
	if (nParams1==nParams2)
	{
		nPar=0;
		while ((nPar < nParams1) && eq)
		{
			// Comprova exactament el tipus, no es pot fer casting a causa de la sobrecarrega.
			if (params1[nPar].getType()!=params2[nPar].getType())
				eq=false;
			else
				nPar++;
		}
	}
	else
		eq=false;
	return eq;
}
/** Comprova que tots els membres de la interficie estiguin implementats */
private void checkAllMetodesImplemented(int linia)
{
	ListIterator it;
	FuncInfo fInfo=null;
	TNode node=null;
	
	try
	{
		it=symbolTable.listFunctions.listIterator(0);
		while (it.hasNext())
		{
			fInfo=(FuncInfo)it.next();
			node=fInfo.getNode();
			if (!node.getIsImplemented())
				throw new MetodeNotImplementedException(getFuncToString(fInfo.getName(),
							node.getParamInfo(),node.getNumParams()),inFilename,linia);
		}	
	}
	catch(PSemanticException sEx)
	{
		reportError(sEx);
	}
}
/** Comprova que el metode estigui declarat en la interficie. Si hi es, marca
	que ja s'ha implementat. Si no error, ja que els metodes han d'estar declarats a la interficie */
private void checkIsDefinedInInterficie(String scopeName,ParamInfo[] params,int numParams,int linia)
{
	ListIterator it;
	boolean eq=false;
	FuncInfo fInfo=null;
	TNode node=null;
	
	try
	{
		it=symbolTable.listFunctions.listIterator(0);
		while ((it.hasNext()) && (!eq))
		{
			fInfo=(FuncInfo)it.next();
			if (scopeName.equalsIgnoreCase(fInfo.getName()))
			{
				node=fInfo.getNode();
				eq=isEqualsParamsFunc(params,numParams,node.getParamInfo(),node.getNumParams());
			}
		}
		if (!eq)
		{	// Error: els metodes han d'estar declarats en la interficie
			throw new MetodeNotDeclInterfException(getFuncToString(scopeName,params,numParams),inFilename,linia);
		}
		else
		{
			if (node.getIsImplemented())
			{
				// El metode ja s'ha implementat anteriorment!
				throw new MetodeAlreadyImplementedException(getFuncToString(scopeName,params,numParams),inFilename,linia);
			}
			else
				node.setIsImplemented(true);
		}
	}
	catch(PSemanticException sEx)
	{
		reportError(sEx);
	}
}
/** <p>Comprova que la variable sigui adequada per una assignacio.</p>
	<p>En cas que no ho sigui, mostra el missatge en pantalla.</p> */
private boolean isStoreVariable(ParamInfo var1,int linia)
{
	boolean storeVariable=false;
	
	try
	{
		if (var1!=null)
		{
			storeVariable=!((var1.getIdType()==FUNCIO) || (var1.getIdType()==ACCIO));
			if (!storeVariable)
				throw new RequiredVariableException(var1.getText(),inFilename,linia);
			else
			{
				if (var1.getIdType()==CONST)
				{
					storeVariable=false;
					throw new CantModifyConstantException(var1.getText(),inFilename,linia);
				}
			}
		}
	}
	
	catch(PSemanticException sEx)
	{
		reportError(sEx);
	}
	finally
	{
		return storeVariable;
	}
}
/* <p>Comprova que el tipus de l'identificador sigui una funci&oacute;.</p>
	<p>En el sintactic es pot escapar una linia que sigui: "var1", ja que es pensa
	que es una crida a una funcio, pero li falta els parentesis; aqu&iacute, es
	fa la comprovaci&oacute; final</p>.
*/
private void isIdFunction(ParamInfo destInf,int linia)
{
	try
	{
		if ((destInf.getIdType()!=FUNCIO) && ((destInf.getIdType()!=ACCIO)))
			throw new NotInstructionException(destInf.getText(),inFilename,linia);
	}
	catch(PSemanticException sEx)
	{
		reportError(sEx);
	}
}
/** Comprova que l'identificador estigui declarat, i retorna la info que ens interesa de la TS */
private ParamInfo isDeclaredId(String id,int linia,LinkedList list)
{
	TNode node=null;
	String funcName;
	ParamInfo idInf=new ParamInfo();
	ParamInfo typeInf=null;
	
	try
	{
 		// A causa del CASE_SENSITIVE,es tracta sempre en MIN en la TS.
		list=symbolTable.lookupNameInCurrentScope2(id.toLowerCase());
		if (list!=null) node=(TNode)list.getFirst();	// Agafem el primer ID de la llista
		else node=null;
		if (node!=null)
		{
			// Agafa tota la info
			copyNodeToParamInfo(node,idInf,id);
			// Comprovacions d'acces a les variables
			funcName=symbolTable.currentScopeAsString();
			funcName=funcName.substring(funcName.lastIndexOf("::")+2,funcName.length());
			node=null;
			list=symbolTable.lookupNameInCurrentScope2(funcName.toLowerCase());
			if (list!=null) node=(TNode)list.getFirst();
			if (node!=null)
			{
				if (node.getAccesMode()==MEMBRE_CLASSE)
				{
					if ((idInf.getAccesMode()!=MEMBRE_CLASSE) && (idInf.getAccesMode()!=PRIVATE))
						throw new CantRefenceInstanciaException(id,inFilename,linia);
				}
			}
		}
		else
		{
			idInf.setType(ID_NOT_DECLARED);	// Estableix el tipus indicant que no esta declarat
			// Va cercant per les classes pares, fins que troba el id, o arriba
			// a una classe que no te cap classe pare.
			if ((symbolTable.getSuperClassName()!=null) && (linia > 0))
			{
				// Comprova que la definicio de la classe sigui visible en la TS
				typeInf=isDeclaredId(symbolTable.getSuperClassName(),-10,new LinkedList());
				if (typeInf.getType()!=ID_NOT_DECLARED)
				{
					typeInf.setIdType(SUPER_CLASS);	 // Marca que es una superclasse.
					idInf=isDeclaredIdOfClass(id,typeInf,linia);
					// idInf es visible, retorna un ParamInfo amb tota la info,
					// si no es visible: idInf.getType()==ID_NOT_DECLARED
				}
			}
		}
	}
	catch (PSemanticException sEx)
	{
		reportError(sEx);
	}
	return idInf;
}
/** <p>Comprova que l'identificador estigui declarat en la classe, i retorna la info de la TS.</p>
	<p>El que fa &eacute;s comprovar que el camp estigui declarat com a visible en la definici&oacute;
	de la classe</p>
*/
private ParamInfo isDeclaredIdOfClass(String id,ParamInfo classInf,int linia)
{
	TNode node=null,cNode=null;
	LinkedList list=null;
	ParamInfo idInf=new ParamInfo(),superClassInf=null;
	Hashtable symTable=null;
	String identif,className;

	try
	{
		if (classInf.getType()!=ID_NOT_DECLARED)
		{	// Si la variable pare esta declarada, fa les comprovacions
			className=classInf.getClassName();
			identif=(symbolTable.addScopeToName(className,id)).toLowerCase(); // CASE_SENSITIVE: es tracta sempre en MIN en la TS.
			list=symbolTable.lookupNameInCurrentScope2(className.toLowerCase());
			if (list!=null) cNode=(TNode)list.getFirst();
		
			if (cNode==null)
			{	// No ha trobat la classe, intenta carregar-la
				loadClassToSimbolTable(className,linia);
				// Torna a provar de cercar la definicio en la TS
				list=symbolTable.lookupNameInCurrentScope2(className.toLowerCase());
				if (list!=null) cNode=(TNode)list.getFirst();			
			}
			if (cNode!=null)
			{
				// Ha trobat la definicio de la classe, carregara la interficie.
				symTable=cNode.getClassInterface();
				list=(LinkedList)symTable.get(identif);
				if (list!=null) node=(TNode)(list.getFirst());
				else node=null;			
				if (node!=null)
				{	// Ha trobat el simbol! El tractem:
					// Indica en kina superclasse esta definit el simbol (per les funcions...)
					idInf.setSuperClassName(className);			
					// S'accedeix a un membre d'una classe, comprova que el membre
					// es visible, que sigui PUBLIC.
					if (classInf.getIdType()==CLASS_DEF)
					{   // Es el nom de la classe, nomes pots accedir a membres de la classe.
						if (node.getAccesMode()==MEMBRE_CLASSE)
						{
							// Agafa tota la info				
							copyNodeToParamInfo(node,idInf,id);						
						}
						else
						{
						
							idInf.setType(ID_NOT_DECLARED);	// Estableix el tipus indicant que no esta declarat
							throw new NotMemberClassException(id,inFilename,linia);
						}
					}
					else if (classInf.getIdType()==SUPER_CLASS)
					{
						// S'agafa el ID per herencia, per tant tot es visible.
						// Agafa tota la info
						copyNodeToParamInfo(node,idInf,id);
					}
					else
					{
						switch(node.getAccesMode())
						{
							case PUBLIC: // PUBLIC, es pot accedir
								// Agafa tota la info
								copyNodeToParamInfo(node,idInf,id);
								break;				
							case PRIVATE: // No es pot accedir
								idInf.setType(ID_NOT_DECLARED);	// Estableix el tipus indicant que no esta declarat
								throw new PrivateAccesException(id,inFilename,linia);		
							case MEMBRE_CLASSE:
								idInf.setType(ID_NOT_DECLARED);	// Estableix el tipus indicant que no esta declarat
								throw new ClassMemberAccesException(id,inFilename,linia);
							default:
								idInf.setType(ID_NOT_DECLARED);	// Estableix el tipus indicant que no esta declarat
								throw new IdNotDeclaredException(id,inFilename,linia);
						}
					}
				}
				else
				{
					// Va cercant per les classes pares, fins que troba el id, o arriba
					// a una classe que no te cap classe pare.
					if (cNode.getSuperClassName()!=null)
					{
						// Busca en l'ambit de la classe pare
						superClassInf=new ParamInfo();
						superClassInf.setClassName(cNode.getSuperClassName());
						superClassInf.setIdType(classInf.getIdType());
						superClassInf.setType(getHash(cNode.getSuperClassName()));
						idInf=isDeclaredIdOfClass(id,superClassInf,linia);
						// idInf es visible, retorna un ParamInfo amb tota la info,
						// si no es visible: idInf.getType()==ID_NOT_DECLARED
					}
					else
						idInf.setType(ID_NOT_DECLARED);	// Estableix el tipus indicant que no esta declarat
				}
			}
			else
				// No esta la definicio de la classe en la TS, ni pot carregar el fitxer
				throw new CantLoadClassFileException(className,inFilename,linia);
		}
	}
	catch (PSemanticException sEx)
	{
		reportError(sEx);
	}
	return idInf;		
}
/** Copia la informacio desada en la TS (TNode) a la informacio del parametre ParamInfo */
private void copyNodeToParamInfo(TNode node,ParamInfo idInf,String text)
{
	idInf.setType(node.getType());
	idInf.setIdType(node.getIdType());
	idInf.setDimension(node.getDimension());
	idInf.setText(text);
	idInf.setClassName(node.getText());			/************************** Modificar el text del Node per className ************/
	idInf.setAccesMode(node.getAccesMode());
}
/** Tracta l'acces a una taula, i comprova que les dimensions concordin */
private void checkAccesTable(ParamInfo factInf,ParamInfo tableInf,int linia)
{
	try
	{
		if (factInf.getDimension()>0)
		{	// Es comprova que la variable sigui efectivament una taula
			if (tableInf.getDimension() > factInf.getDimension())
				throw new BadTableDimensionException(factInf.getText(),inFilename,linia);
			// Afegeix les referencies [] de la taula al nom de variable
			factInf.appendText(tableInf.getText());
			// Actualitza les dimensions per l'element referenciat
			factInf.setDimension(factInf.getDimension()-tableInf.getDimension());
		}
		else
			throw new RequiredTableException(factInf.getText(),inFilename,linia);
	}
	catch(PSemanticException sEx)
	{
		reportError(sEx);
	}
}
/** Retorna el node que conte la informacio de la funcio */
private LinkedList getListOfFunctions(String varName,String className,int linia)
{
	LinkedList list=null;
	TNode node=null;
	String identif;
	
	if (className==null)
	{
		list=symbolTable.lookupNameInCurrentScope2(varName.toLowerCase()); // CASE_SENSITIVE: Acces a la TS en minuscules
	}
	else
	{
		if (className!="")
		{
			// Busca el nom de la funcio en la classe
			identif=(symbolTable.addScopeToName(className,varName)).toLowerCase();
			list=symbolTable.lookupNameInCurrentScope2(className.toLowerCase());
			if (list!=null) node=(TNode)list.getFirst();
			if (node!=null)
			{
				list=(LinkedList)node.getClassInterface().get(identif);
				if (list!=null) node=(TNode)(list.getFirst());
				else node=null;
			}
			else
			{
				System.err.println("No es troben els simbols de la classe:"+className);
				numErrors+=1;	
			}
		}
	}
	return list;
}
/** Comprova els parametres de la funcio. Si falla, intenta buscar-la en les classes pares, si n'hi ha. */
private ParamInfo 
checkParametersOfFunction(String varName,String className,int cridaType,LinkedList list,ParamInfo[] parParams,int numParams,int linia)
{
	ParamInfo parInf,classInf=new ParamInfo();
	String superClass=null,identif;
	TNode cNode=null,node=null;
	
	parInf=checkFuncParams(list,parParams,numParams,linia);
	if (parInf==null)
	{
		// Intenta carregar la classe pare
		if (className==null) 
		{
			superClass=symbolTable.getSuperClassName();
		}
		else
		{
			identif=(symbolTable.addScopeToName(className,varName)).toLowerCase();	// CASE_SENSITIVE: Acces a la TS en minuscules
			list=symbolTable.lookupNameInCurrentScope2(identif);
			if (list!=null) node=(TNode)list.getFirst();
			if (list==null)
			{	// No ha trobat la classe, intenta carregar-la
				loadClassToSimbolTable(className,linia);
				// Torna a provar de cercar la definicio en la TS
				list=symbolTable.lookupNameInCurrentScope2(className.toLowerCase());
				if (list!=null) node=(TNode)list.getFirst();			
			}
			if (node!=null)
			{
				superClass=node.getSuperClassName();
			}
		}
		if (superClass!=null)
		{
			// Busquem l'ambit on sigui visible el simbol (recorre les classes pares)
			classInf.setClassName(superClass);
			classInf.setIdType(SUPER_CLASS);
			classInf.setType(getHash(superClass));
			
			if (cridaType==CONSTRUCTOR)
				varName=superClass;
			
			parInf=isDeclaredIdOfClass(varName,classInf,linia);
			
			if (parInf.getType()!=ID_NOT_DECLARED)
			{
				parInf=null;
				list=getListOfFunctions(varName,superClass,linia);
				// Comprovem els parametres recursivament
				parInf=checkParametersOfFunction(varName,superClass,cridaType,list,parParams,numParams,linia);
			}
			else
				parInf=null; 	// No s'ha trobat la funcio.
		}
		else
		{
			// No ha trobat la signatura de la funcio en les superclasses, per tant dona error
			parInf=null;
		}
	}
	return parInf;	
}
/** Comprova que els parametres passats a la funcio, siguin correctes (te en compte sobrecarrega de noms) */
private ParamInfo checkFuncParams(LinkedList list,ParamInfo[] parParams,int nParParams,int linia)
{
	int i,nRePar;
	ParamInfo[] rePar;
	ParamInfo fucInfo=null,nodeInf;	
	TNode node=null;
	ListIterator it;
	boolean trobat=false;
	
	it=list.listIterator(0);
	while ((it.hasNext()) && (!trobat))
	{
		trobat=true;
		node=(TNode)it.next();
		if ((node.getIdType()==FUNCIO) || (node.getIdType()==ACCIO))
		{
			nRePar=node.getNumParams();
			rePar=node.getParamInfo();
			if (nRePar!=nParParams)
			{
				trobat=false;
			}			
			else
			{
				i=0;
				while ((i < nRePar) && (trobat))
				{
					if (checkTypes(rePar[i],parParams[i],false,false,linia)==0)
						trobat=false;
					else
						i++;
				}
			}
		}
		else
			trobat=false;
	}
	// Recull la informacio de la funcio
	if (trobat)
	{
		fucInfo=new ParamInfo();
		fucInfo.setType(node.getType());
		fucInfo.setIdType(node.getIdType());
		if (node.getText()!=null)
		{
			// El parametre de retorn es una classe
			fucInfo.setClassName(node.getText());
		}
	}
	return fucInfo;
}
/** <p>Comprova que els dos tipus siguin compatibles.</p><p>Si doAutocast es cert, llavors intenta fer un cast automatic.</p>
    <p>Si reportError es cert, en cas que siguin incompatibles mostrara un missatge d'error.</p>
*/
private int checkTypes(ParamInfo pInf1,ParamInfo pInf2,boolean doAutoCast,boolean reportError,int linia)
{
	int finalType=0;
	
	try
	{
		if (auxExpAssig!=null)
		{
			// Comprova tipus pInf1 amb pInf2 amb assignacio
			if (pInf1.getDimension()!=pInf2.getDimension())
				throw new TypeInvalidException(inFilename,linia);	// Dimensions no valides, no diu res del tipus.
			else
			{
				if ((pInf1.getType()!=pInf2.getType()) && (pInf2.getType()!=NULL))
				{
					if (doAutoCast)
					{
						finalType=tryCast(auxExpAssig,pInf1,pInf2,linia);
						if (finalType==0)
						{
							if (reportError) 
								throw new TypeInvalidException(getTypeName(pInf1),getTypeName(pInf2),inFilename,linia);
						}
					}
					else
						if (reportError)
							throw new TypeInvalidException(getTypeName(pInf1),getTypeName(pInf2),inFilename,linia);
				}
				else
					finalType=pInf1.getType();  // Son iguals
			}
		}
		else
		{
			// Comprova tipus pInf1 amb pInf2 sense assignacio
			if (pInf1.getDimension()!=pInf2.getDimension())
				throw new TypeInvalidException(inFilename,linia);	// Dimensions no valides, no diu res del tipus.
			else
			{
				if ((pInf1.getType()!=pInf2.getType()) && (pInf2.getType()!=NULL))
				{
					if (doAutoCast)
					{
						finalType=tryCast(auxExpAssig,pInf1,pInf2,linia);
						if (finalType==0)
						{
							if (reportError)
								throw new TypeInvalidException(getTypeName(pInf1),getTypeName(pInf2),inFilename,linia);
						}
					}
					else if ((pInf1.getIdType()==CLASS_TYPE) && (pInf2.getIdType()==CLASS_TYPE))
					{
						if (tryObjectCast(pInf1,pInf2))
							finalType=pInf1.getType();  // pInf1 es el tipus del parametre formal
					}
					else
						if (reportError)
							throw new TypeInvalidException(getTypeName(pInf1),getTypeName(pInf2),inFilename,linia);
				}
				else
					finalType=pInf1.getType();  // Son iguals
			}
		}
	}
	catch(PSemanticException sEx)
	{
		reportError(sEx);
	}
	return finalType;
}
/** Comprova si es pot fer un cast entre classes, tot utilitzant la herencia de classes */
private boolean tryObjectCast(ParamInfo pReq,ParamInfo pPar)
{
	boolean cast=false,continua=true;
	LinkedList list;
	TNode cNode=null;
	String superClass;

	superClass=pPar.getClassName();
	while ((superClass!=null) && (!cast))
	{
		// Carrega la info de superClass
		list=symbolTable.lookupNameInCurrentScope2(superClass.toLowerCase());
		if (list!=null) // Faltaria potser fer un intent de carregar el fitxer si no tenim al def a la TS
		{
			cNode=(TNode)list.getFirst();
		}

		superClass=cNode.getSuperClassName();  // Actualitza la classe pare
		if (superClass!=null)
		{
			// Si la classe pare es igual a la que es requereix, es fa cast
			if (superClass.equals(pReq.getClassName()))
			{
				cast=true;		// Qualsevol fill es pot fer passar per el seu pare (no a l'inreves)
			}
		}
	}
	return cast;
}
/** Intenta fer cast de tipus elementals. Si ha sigut exitos, retorna el tipus del cast, sino retorna 0. */
private int tryCast(ParamInfo destInf,ParamInfo pInf1,ParamInfo pInf2,int linia)
{
	int finalType=0, desType;
	String aux;

	try
	{
		if (auxExpAssig!=null)
		{
			if (auxExpAssig.getIdType()==ELEMENTAL_TYPE)
			{
				desType=auxExpAssig.getType();
				switch(desType)
				{
					case NUMERO_ENTER:
						// Canvi de numero real a enter
						if ((pInf1.getType()==NUMERO_REAL) && (pInf2.getType()==NUMERO_ENTER))
						{
							aux=pInf1.getText();
							pInf1.setType(NUMERO_ENTER);
							pInf1.setText("((int)(" + pInf1.getText() + "))");
							finalType=NUMERO_ENTER;
							throw new LossPrecisionWarningException(aux,inFilename,linia);
						}
						else if ((pInf1.getType()==NUMERO_ENTER) && (pInf2.getType()==NUMERO_REAL))
						{
							aux=pInf1.getText();
							pInf2.setType(NUMERO_ENTER);
							pInf2.setText("((int)(" + pInf2.getText() + "))");
							finalType=NUMERO_ENTER;
							throw new LossPrecisionWarningException(aux,inFilename,linia);
						}
						break;
					case NUMERO_REAL:
						// Canvi de numero enter a real (no cal generar warning.
						if ((pInf1.getType()==NUMERO_ENTER) && (pInf2.getType()==NUMERO_REAL))
						{
							pInf1.setType(NUMERO_REAL);
							pInf1.setText("((double)(" + pInf1.getText() + "))");
							finalType=NUMERO_REAL;
						}
						else if ((pInf1.getType()==NUMERO_ENTER) && (pInf2.getType()==NUMERO_ENTER))
						{
							pInf2.setType(NUMERO_REAL);
							pInf2.setText("((double)(" + pInf2.getText() + "))");
							finalType=NUMERO_REAL;
						}
						break;					
					default:
						// Canvi de numero real a enter
						if ((pInf1.getType()==NUMERO_REAL) && (pInf2.getType()==NUMERO_ENTER))
						{
							aux=pInf1.getText();
							pInf1.setType(NUMERO_ENTER);
							pInf1.setText("((int)(" + pInf1.getText() + "))");
							finalType=NUMERO_ENTER;
							throw new LossPrecisionWarningException(aux,inFilename,linia);
						}
						else if  ((pInf1.getType()==NUMERO_ENTER) && (pInf2.getType()==NUMERO_REAL))
						{
							aux=pInf1.getText();
							pInf2.setType(NUMERO_ENTER);
							pInf2.setText("((int)(" + pInf2.getText() + "))");
							finalType=NUMERO_ENTER;
							throw new LossPrecisionWarningException(aux,inFilename,linia);
						}
						break;			
				}
			}
		}
	}
	catch(PSemanticWarning sEx)
	{	
		reportWarning(sEx);
	}
	finally
	{
		return finalType;
	}
}
/** Tracta l'instruccio d'assignacio: Fa comprobacions semantiques i la traduccio. */
private void tractarInstAssig(ParamInfo destInf,ParamInfo sourceInf,int linia)
{
	try
	{
		if (isStoreVariable(destInf,linia))
		{
			// Comprova els tipus
			if (sourceInf.getType()!=LLEGIR)
			{
				checkTypes(destInf,sourceInf,true,true,linia);
			}
			else
			{
				if (destInf.getIdType()!=ELEMENTAL_TYPE)
					throw new CantReadVariableException(destInf.getText(),inFilename,linia);
				sourceInf.setText(out.getExpressionLlegir(destInf.getType()));
			}
		}
		out.writeInstAssig(destInf.getText(),sourceInf.getText());
	}
	catch(PSemanticException sEx)
	{
		reportError(sEx);
	}
}
/** Retorna el nom de la classe pare. Si no en te cap, retorna cadena buida: "" */
private String toStringSuperClassName()
{
	String sClass;
	
	if (symbolTable.getSuperClassName()==null)
		sClass="";
	else
		sClass=symbolTable.getSuperClassName();

	return sClass;
}
/** Tracta l'instruccio que es una crida a una accio/funcio: fa comprovacions semantiques i la corresponent traduccio */
private void tractarInstCrida(ParamInfo cridaInf,int linia)
{
	if (cridaInf!=null)
	{
		isIdFunction(cridaInf,linia);
		out.writeInstCrida(cridaInf.getText());
	}
}
/** Tracta l'instruccio per: fa comprovacions semantiques i la corresponent traduccio */
private void tractarInstPer(ParamInfo varInf,ParamInfo iniExp,ParamInfo endExp,ParamInfo pasExp,int linia)
{
	String pas="";
	
	try
	{
		if (isStoreVariable(varInf,linia))
		{
			// Comprova els tipus
			if (varInf.getType()!=NUMERO_ENTER)
				throw new RequiredEnterException("la variable del per",inFilename,linia);
			if (iniExp.getType()!=NUMERO_ENTER)
				throw new RequiredEnterException("l'expressio d'inici del per",inFilename,linia);
			if (endExp.getType()!=NUMERO_ENTER)
				throw new RequiredEnterException("l'expressio de fi del per",inFilename,linia);

			if (pasExp!=null)
			{
				if (pasExp.getType()!=NUMERO_ENTER)
					throw new RequiredEnterException("l'expressio del pas",inFilename,linia);
				pas=pasExp.getText();
			}
			out.writeInstPer(varInf.getText(),iniExp.getText(),endExp.getText(),pas);
		}
	}
	catch(PSemanticException sEx)
	{
		reportError(sEx);
	}
}
/** Tracta l'instruccio si: Fa comprovacions semantiques i la corresponent traduccio */
private void tractarInstSi(ParamInfo condInf,int linia)
{
	try
	{
		if (condInf.getType()!=BOOLEA) 
			throw new RequiredBooleaException("la condicio del si",inFilename,linia);
		out.writeInstSi(condInf.getText());
	}
	catch(PSemanticException sEx)
	{
		reportError(sEx);
	}
}
/** Tracta l'instruccio cas: Fa les comprovacions semantiques i la corresponent traduccio */
private void tractarInstCas(ParamInfo condInf,boolean firstCond,int linia)
{
	try
	{
		if (condInf.getType()!=BOOLEA) 
			throw new RequiredBooleaException("la condicio del cas",inFilename,linia);
		if (firstCond)
			out.writeInstCasPrimer(condInf.getText());
		else
			out.writeInstCasAltres(condInf.getText());
	}
	catch(PSemanticException sEx)
	{
		reportError(sEx);
	}
}
/** Tracta quan es troba la referencia a la propia classe: JO, modificant el ParamInfo */
private void tractarJo(ParamInfo factInf)
{
	factInf.setText("this");
	factInf.setIdType(CLASS_TYPE);
	factInf.setType(getHash(symbolTable.getParentScopeName()));
	factInf.setClassName(symbolTable.getParentScopeName());
}
/** Comprova que super es la primera instruccio del constructor */
private void checkSuperIsFirstStatement(int linia)
{
	try
	{
		if (symbolTable.getTypeOfScope()!=CONSTRUCTOR)
		{
			throw new CallConstSuperException(inFilename,linia);
		}
		else
		{
			if (!isFirstInstruction)
				throw new SuperFirstStatementException(inFilename,linia);
		}
	}
	catch(PSemanticException sEx)
	{
		reportError(sEx);
	}
}
/** Especifica la versio del traductor, que s'escriu en els fitxers generats. */
public void setVersion(String ver)
{
	version=ver;
}
/** Especifica si esta actiu el mode Verbose, que reporta a l'usuari molta informacio
	referent a les tasques que duu a terme el traductor */
public void setVerboseMode(boolean verbose)
{
	verboseMode=verbose;
}
/** Especifica el nom del fitxer d'entrada per imprimir-lo ens els missatges d'errors */
public void setInputFilename(String filename)
{
	inFilename=filename;
	openOutFilename();
}
/** Obre el fitxer de sortida per a poder escriure-hi. */
private void openOutFilename()
{
	outFile=genRandomName();
	out=new CpcGenJava(outFile,version,verboseMode);
}
/** Genera un nom a l'atzar per el fitxer temporal de sortida */
private String genRandomName()
{
	String fName="_";
	char car;
	int i;
	
	for(i=0;i<5;i++)
	{
		car=(char)Math.round((Math.random()*25) + 97);
		fName=fName + car;
	}
	fName=fName + ".tmp";
	return fName;
}
/** Tracta l'herencia de la classe, quan aquesta es declarada */
private void tractarHerencia(String className,int linia)
{
	if (!symbolTable.getParentScopeName().equalsIgnoreCase(className))  // CASE_SENSITIVE: no es te en compte majus/minus
		loadClassToSimbolTable(className,linia);
	else
	{
		System.err.println("Error: " + linia + ": no pots heretar de tu mateix!");
		numErrors+=numErrors+1;
	}
	
	symbolTable.setSuperClassName(className);
}
/** Retorna la TS confeccionada fins al moment */
public SymbolTable getSymbolTable()
{
	return symbolTable;
}
/** Retorna el numero d'avisos generats */
public int getNumAvisos()
{
	return numAvisos;
}
/** Retorna el nom de fitxer de sortida en el que s'escriu */
public String getOutFile()
{
	return outFile;
}
/** Tanca el fitxer de sortida en el que escrivia */
public void closeOutFile()
{
	out.closeFile();
}
/** <p>Carrega tot el que es public d'una classe (la seva interficie).</p>
	<p> Per fer-ho, intenta traduir el fitxer de classe. Un cop traduit, agafa la taula de 
	simbols generada per la traduccio de la classe externai la desa en un node. Aquest node
	es introduit en la nostra TS, tot identificant-lo com una definicio de classe (CLASS_DEF).</p>
*/
private void loadClassToSimbolTable(String classname,int linia)
{
	try
	{
		CpcLexer lexer;
		CpcParser parser;
		String filename;
		TNode node=null;
		LinkedList list=null;
		int numErrClass;
		
		// Comprova que no estigui carregada ja
		list=symbolTable.lookupNameInCurrentScope2(classname.toLowerCase()); // Accedeix en minus a la TS
		if (list!=null) node=(TNode)list.getFirst();
		if (node==null)
		{	// Carrega la interficie de la classe
			filename=classname+EXT_DEF;
			if (verboseMode)
				System.err.println("[carregant la classe " + filename + "]");
			
			// Llegeix i parseja el fitxer de la classe
			lexer=new CpcLexer(new DataInputStream(new FileInputStream(filename)));
			parser=new CpcParser(lexer);
			parser.setVersion(version);
			parser.setVerboseMode(verboseMode);
			parser.setInputFilename(filename);
			numErrClass=parser.programa(true);
			
			// L'inserta a la taula de simbols en l'ambit actual com a classe
			node=new TNode();
			node.setIdType(CLASS_DEF);
			node.setText(classname);
			node.setClassInterface(parser.getSymbolTable().getSymTable());
			node.setSuperClassName(parser.getSymbolTable().getSuperClassName());
			
			list=new LinkedList();
			list.add(node);
			symbolTable.addClassInterface(classname.toLowerCase(),list); // L'inserta en la TS en minus per el CASE_SENSITIVE
			numErrors+=numErrClass;  // Afegeix els errors trobats a la classe
			numAvisos+=parser.getNumAvisos();    // Afegeix els avisos trobats a la classe
		}
		else
		{
		
			if (node.getIdType()!=CLASS_DEF)
			{
				System.err.println("No es un tipus valid: "+ classname);
			}
		}
	}
	catch (FileNotFoundException e)
	{
		reportError(new CantLoadClassFileException(classname,inFilename,linia));
	}
	catch (Exception e)
	{
		e.printStackTrace();
		System.exit(-1);
	}		
}
private void tractarFitxerFinal()
{
	/** Tracta el fitxer final, si hi ha error esborra el fitxer temporal, altrament
		el renombra al nom final */
	String finalFile;
	
	finalFile=inFilename.substring(0,inFilename.lastIndexOf('.'))+".java";
	if (numErrors > 0)
	{
		// Esborra el fitxer
		(new File(outFile)).delete();
	}
	else
	{
		// Renombra el fitxer
		(new File(outFile)).renameTo(new File(finalFile));
		outFile=finalFile;
	}
}

protected CpcParser(TokenBuffer tokenBuf, int k) {
  super(tokenBuf,k);
  tokenNames = _tokenNames;
}

public CpcParser(TokenBuffer tokenBuf) {
  this(tokenBuf,1);
}

protected CpcParser(TokenStream lexer, int k) {
  super(lexer,k);
  tokenNames = _tokenNames;
}

public CpcParser(TokenStream lexer) {
  this(lexer,1);
}

public CpcParser(ParserSharedInputState state) {
  super(state,1);
  tokenNames = _tokenNames;
}

/** <p>Regla principal, que engloba qualsevol fitxer de pseudocodi.</p>
	<p>Aquesta regla es la que s'ha de cridar per poder parsejar un fitxer. El parametre
	boolea isNeedBeClass especifica, si es cert, que el fitxer a tractar ha de ser
	obligatoriament	una classe.
*/
	public final int  programa(
		boolean isNeedBeClass
	) throws RecognitionException, TokenStreamException {
		int numErr=-1;
		
		
				if (verboseMode)
					System.err.println("[traduint el fitxer " + inFilename + "]");
			
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case SALT_LINIA:
			case COMENTARI1:
			{
				{
				sepa_inst();
				}
				break;
			}
			case ALGORISME:
			case CLASSE:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			{
			switch ( LA(1)) {
			case ALGORISME:
			{
				algorisme();
				break;
			}
			case CLASSE:
			{
				numErr=classe();
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
				if (isNeedBeClass && numErr==-1)
					{
						System.err.println("Error: Es necessita una classe");
						numErrors+=1;
					}
			
					if (!isNeedBeClass)
					{
						if (numErrors > 0)
						{
							// Si hi ha errors, reporta el numero total
							if (numErrors==1)
								System.err.print(numErrors + " error ");
							else 
								System.err.print(numErrors + " errors ");
						}
						numErr=numErrors;
						if (numAvisos > 0)
						{
							// Si hi ha avisos, reporta el numero total
							if (numAvisos==1)
								System.err.print(numAvisos + " avis");
							else 
								System.err.print(numAvisos + " avisos");			
						}
						if ((numAvisos > 0) || (numErrors > 0))
							System.err.println();
					}
					tractarFitxerFinal();
					if ((verboseMode) && (numErrors==0))
						System.err.println("[fitxer de sortida " + outFile + " escrit correctament]");
				
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_0);
		}
		return numErr;
	}
	
/** sepa_inst: regla que especifica el separador d'instruccions */
	public final void sepa_inst() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			{
			int _cnt65=0;
			_loop65:
			do {
				switch ( LA(1)) {
				case SALT_LINIA:
				{
					match(SALT_LINIA);
					break;
				}
				case COMENTARI1:
				{
					match(COMENTARI1);
					break;
				}
				default:
				{
					if ( _cnt65>=1 ) { break _loop65; } else {throw new NoViableAltException(LT(1), getFilename());}
				}
				}
				_cnt65++;
			} while (true);
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_1);
		}
	}
	
	public final void algorisme() throws RecognitionException, TokenStreamException {
		
		Token  id = null;
		
		try {      // for error handling
			match(ALGORISME);
			id = LT(1);
			match(IDENTIFICADOR);
			
							symbolTable.pushScope(id.getText());
							symbolTable.setTypeOfScope(ALGORISME);
							out.writeHeadOfClass(id.getText(),"");
							out.writeHeadOfMain(); 
						
			sepa_inst();
			bloc_algorisme();
			match(FALGORISME);
			{
			switch ( LA(1)) {
			case SALT_LINIA:
			case COMENTARI1:
			{
				sepa_inst();
				break;
			}
			case EOF:
			{
				match(Token.EOF_TYPE);
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			
							symbolTable.popScope(); // S'elimina el primer ambit
							out.writeEndBlock();  // Final de la funcio main
							out.checkDefaultConstructor(); // Afegeix un const. per defecte
							out.writeEndBlock();  // Final de la classe principal
							closeOutFile();
						
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_0);
		}
	}
	
	public final int  classe() throws RecognitionException, TokenStreamException {
		int numErr=0;
		
		Token  id = null;
		Token  hId = null;
		
		try {      // for error handling
			match(CLASSE);
			id = LT(1);
			match(IDENTIFICADOR);
			{
			switch ( LA(1)) {
			case HERETA:
			{
				match(HERETA);
				hId = LT(1);
				match(IDENTIFICADOR);
				break;
			}
			case SALT_LINIA:
			case COMENTARI1:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			{
			sepa_inst();
			}
			
						symbolTable.pushScope(id.getText().toLowerCase());		// CASE_SENSITIVE: Acces a TS en minusc
						symbolTable.setTypeOfScope(CLASSE);
						if (hId!=null)
						{
							tractarHerencia(hId.getText(),LT(1).getLine());
							out.writeHeadOfClass(id.getText(),hId.getText());
						}
						else
							out.writeHeadOfClass(id.getText(),"");
					
			bloc_classe();
			match(FCLASSE);
			{
			switch ( LA(1)) {
			case SALT_LINIA:
			case COMENTARI1:
			{
				sepa_inst();
				break;
			}
			case EOF:
			{
				match(Token.EOF_TYPE);
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			
						symbolTable.popScope(); // S'elimina el primer ambit
						out.checkDefaultConstructor(); // Comprova que hi hagi un const. per defecte
						out.writeEndBlock();	// Final de classe
						out.closeFile();
						checkAllMetodesImplemented(LT(1).getLine());  // Comprova que tots els membres de la interficie estiguin implementats
						// Retorna el numero d'errors
						numErr=numErrors;
					
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_0);
		}
		return numErr;
	}
	
	public final void bloc_algorisme() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			bloc_instruccions();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_2);
		}
	}
	
	public final void bloc_classe() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case INTERFICIE:
			{
				bloc_interficie();
				break;
			}
			case FCLASSE:
			case IMPLEMENTACIO:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			{
			switch ( LA(1)) {
			case IMPLEMENTACIO:
			{
				bloc_implementacio();
				break;
			}
			case FCLASSE:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_3);
		}
	}
	
	public final void bloc_interficie() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			match(INTERFICIE);
			sepa_inst();
			{
			switch ( LA(1)) {
			case ATRIBUTS_CLASSE:
			{
				bloc_atribut_classe();
				break;
			}
			case FINTERFICIE:
			case ATRIBUTS:
			case METODES:
			case METODES_CLASSE:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			{
			switch ( LA(1)) {
			case ATRIBUTS:
			{
				bloc_atribut();
				break;
			}
			case FINTERFICIE:
			case METODES:
			case METODES_CLASSE:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			{
			switch ( LA(1)) {
			case METODES_CLASSE:
			{
				bloc_metode_classe();
				break;
			}
			case FINTERFICIE:
			case METODES:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			{
			switch ( LA(1)) {
			case METODES:
			{
				bloc_metode();
				break;
			}
			case FINTERFICIE:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			match(FINTERFICIE);
			sepa_inst();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_4);
		}
	}
	
	public final void bloc_implementacio() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			match(IMPLEMENTACIO);
			sepa_inst();
			{
			switch ( LA(1)) {
			case VAR:
			{
				declar_var(IMPLEMENTACIO);
				break;
			}
			case FUNCIO:
			case FIMPLEMENTACIO:
			case CONSTRUCTOR:
			case DESTRUCTOR:
			case METODE:
			case METODE_CLASSE:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			{
			_loop103:
			do {
				switch ( LA(1)) {
				case FUNCIO:
				{
					funcio();
					break;
				}
				case METODE:
				{
					metode();
					break;
				}
				case METODE_CLASSE:
				{
					metode_classe();
					break;
				}
				case CONSTRUCTOR:
				{
					constructor();
					break;
				}
				case DESTRUCTOR:
				{
					destructor();
					break;
				}
				default:
				{
					break _loop103;
				}
				}
			} while (true);
			}
			match(FIMPLEMENTACIO);
			sepa_inst();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_3);
		}
	}
	
	public final void bloc_atribut_classe() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			match(ATRIBUTS_CLASSE);
			sepa_inst();
			{
			switch ( LA(1)) {
			case VAR:
			{
				declar_var(ATRIBUTS_CLASSE);
				{
				switch ( LA(1)) {
				case CONST:
				{
					declar_const(ATRIBUTS_CLASSE);
					break;
				}
				case FATRIBUTS_CLASSE:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				break;
			}
			case CONST:
			{
				declar_const(ATRIBUTS_CLASSE);
				{
				switch ( LA(1)) {
				case VAR:
				{
					declar_var(ATRIBUTS_CLASSE);
					break;
				}
				case FATRIBUTS_CLASSE:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				break;
			}
			case FATRIBUTS_CLASSE:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			match(FATRIBUTS_CLASSE);
			sepa_inst();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_5);
		}
	}
	
	public final void bloc_atribut() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			match(ATRIBUTS);
			sepa_inst();
			{
			switch ( LA(1)) {
			case VAR:
			{
				declar_var(ATRIBUTS);
				break;
			}
			case FATRIBUTS:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			match(FATRIBUTS);
			sepa_inst();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_6);
		}
	}
	
	public final void bloc_metode_classe() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			match(METODES_CLASSE);
			sepa_inst();
			{
			_loop99:
			do {
				if ((LA(1)==METODE_CLASSE)) {
					capzalera_metode_classe(false);
				}
				else {
					break _loop99;
				}
				
			} while (true);
			}
			match(FMETODES_CLASSE);
			sepa_inst();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_7);
		}
	}
	
	public final void bloc_metode() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			match(METODES);
			sepa_inst();
			{
			_loop94:
			do {
				if ((LA(1)==CONSTRUCTOR)) {
					capzalera_constructor(false);
				}
				else {
					break _loop94;
				}
				
			} while (true);
			}
			{
			_loop96:
			do {
				if ((LA(1)==METODE)) {
					capzalera_metode(false);
				}
				else {
					break _loop96;
				}
				
			} while (true);
			}
			match(FMETODES);
			sepa_inst();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_8);
		}
	}
	
	public final void declar_atributs() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case ATRIBUTS:
			{
				{
				bloc_atribut();
				}
				{
				switch ( LA(1)) {
				case ATRIBUTS_CLASSE:
				{
					bloc_atribut_classe();
					break;
				}
				case EOF:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				break;
			}
			case ATRIBUTS_CLASSE:
			{
				{
				bloc_atribut_classe();
				}
				{
				switch ( LA(1)) {
				case ATRIBUTS:
				{
					bloc_atribut();
					break;
				}
				case EOF:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_0);
		}
	}
	
	public final void declar_var(
		int typeDecl
	) throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			match(VAR);
			{
			sepa_inst();
			}
			{
			_loop150:
			do {
				if ((LA(1)==IDENTIFICADOR)) {
					llista_var(typeDecl);
				}
				else {
					break _loop150;
				}
				
			} while (true);
			}
			match(FVAR);
			{
			sepa_inst();
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_9);
		}
	}
	
	public final void declar_const(
		int typeDecl
	) throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			match(CONST);
			{
			sepa_inst();
			}
			{
			_loop159:
			do {
				if ((LA(1)==IDENTIFICADOR)) {
					llista_const(typeDecl);
				}
				else {
					break _loop159;
				}
				
			} while (true);
			}
			match(FCONST);
			{
			sepa_inst();
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_10);
		}
	}
	
	public final void capzalera_constructor(
		boolean declararAmbit
	) throws RecognitionException, TokenStreamException {
		
		Token  id = null;
		ParamInfo []params=new ParamInfo[128]; int Type=0,numParams=0;
		
		try {      // for error handling
			match(CONSTRUCTOR);
			id = LT(1);
			match(IDENTIFICADOR);
			match(PAREN_ESQ);
			numParams=parametres_formals(params,0);
			match(PAREN_DRET);
			{
			sepa_inst();
			}
			
							// Declara el constructor. Resta un al numero de linia, ja que ha parsejat sepa_inst
							declararFuncio(id.getText(),params,numParams,null,declararAmbit,LT(1).getLine()-1,CONSTRUCTOR);
						
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_11);
		}
	}
	
	public final void capzalera_metode(
		boolean declararAmbit
	) throws RecognitionException, TokenStreamException {
		
		Token  id = null;
		ParamInfo []params=new ParamInfo[128]; ParamInfo retParam=null; int Type=0,numParams=0;
		
		try {      // for error handling
			match(METODE);
			id = LT(1);
			match(IDENTIFICADOR);
			match(PAREN_ESQ);
			numParams=parametres_formals(params,0);
			match(PAREN_DRET);
			{
			switch ( LA(1)) {
			case RETORNA:
			{
				match(RETORNA);
				retParam=parametres_retorn();
				break;
			}
			case SALT_LINIA:
			case COMENTARI1:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			{
			sepa_inst();
			}
			
							// Declara el metode. Resta un al numero de linia, ja que ha parsejat sepa_inst
							declararFuncio(id.getText(),params,numParams,retParam,declararAmbit,LT(1).getLine()-1,METODE);
						
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_12);
		}
	}
	
	public final void capzalera_metode_classe(
		boolean declararAmbit
	) throws RecognitionException, TokenStreamException {
		
		Token  id = null;
		ParamInfo []params=new ParamInfo[128]; ParamInfo retParam=null; int Type=0,numParams=0;
		
		try {      // for error handling
			match(METODE_CLASSE);
			id = LT(1);
			match(IDENTIFICADOR);
			match(PAREN_ESQ);
			numParams=parametres_formals(params,0);
			match(PAREN_DRET);
			{
			switch ( LA(1)) {
			case RETORNA:
			{
				match(RETORNA);
				retParam=parametres_retorn();
				break;
			}
			case SALT_LINIA:
			case COMENTARI1:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			{
			sepa_inst();
			}
			
							// Declara el metode de classe. Resta un al numero de linia, ja que ha parsejat sepa_inst
							declararFuncio(id.getText(),params,numParams,retParam,declararAmbit,LT(1).getLine()-1,METODE_CLASSE);
						
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_13);
		}
	}
	
	public final void funcio() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			capzalera_funcio(true);
			bloc_instruccions();
			match(FFUNCIO);
			{
			sepa_inst();
			}
			
					 	// Quan es fa match amb aquesta regla, ja s'ha acabat de "parsejar"
						// la funcio, i per tant es pot eliminar l'ambit.
						symbolTable.popScope();
						out.writeEndBlock();			
					
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_14);
		}
	}
	
	public final void metode() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			capzalera_metode(true);
			bloc_instruccions();
			match(FMETODE);
			{
			sepa_inst();
			}
			
					 	// Quan es fa match amb aquesta regla, ja s'ha acabat de "parsejar"
						// el metode, i per tant es pot eliminar l'ambit.
						symbolTable.popScope();
						out.writeEndBlock();
					
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_14);
		}
	}
	
	public final void metode_classe() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			capzalera_metode_classe(true);
			bloc_instruccions();
			match(FMETODE_CLASSE);
			{
			sepa_inst();
			}
			
					 	// Quan es fa match amb aquesta regla, ja s'ha acabat de "parsejar"
						// el metode, i per tant es pot eliminar l'ambit.
						symbolTable.popScope();
						out.writeEndBlock();			
					
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_14);
		}
	}
	
	public final void constructor() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			capzalera_constructor(true);
			bloc_instruccions();
			match(FCONSTRUCTOR);
			{
			sepa_inst();
			}
			
					 	// Quan es fa match amb aquesta regla, ja s'ha acabat de "parsejar"
						// el constructor, i per tant es pot eliminar l'ambit.
						symbolTable.popScope();
						out.writeEndBlock();			
					
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_14);
		}
	}
	
	public final void destructor() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			capzalera_destructor(true);
			bloc_instruccions();
			match(FDESTRUCTOR);
			{
			sepa_inst();
			}
			
					 	// Quan es fa match amb aquesta regla, ja s'ha acabat de "parsejar"
						// el destructor, i per tant es pot eliminar l'ambit.
						symbolTable.popScope();
						out.writeEndBlock();			
					
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_14);
		}
	}
	
	public final int  parametres_formals(
		ParamInfo[] params,int index
	) throws RecognitionException, TokenStreamException {
		int numParametres=0;;
		
		ParamInfo aux=null;
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case IDENTIFICADOR:
			{
				aux=parametre();
				{
				switch ( LA(1)) {
				case COMA:
				{
					{
					match(COMA);
					numParametres=parametres_formals(params,index+1);
					}
					break;
				}
				case PAREN_DRET:
				{
					numParametres=index+1;
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				break;
			}
			case PAREN_DRET:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
				params[index]=aux;	
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_15);
		}
		return numParametres;
	}
	
	public final ParamInfo  parametres_retorn() throws RecognitionException, TokenStreamException {
		ParamInfo pInfo=null;
		
		Token  id = null;
		
		try {      // for error handling
			id = LT(1);
			match(IDENTIFICADOR);
			match(DOSPUNTS);
			pInfo=tipus();
				/* ABANS, la regla era: parametres_formals */
					pInfo.setName(id.getText());
				
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_16);
		}
		return pInfo;
	}
	
	public final void bloc_instruccions() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case VAR:
			{
				declar_var(RES);
				{
				switch ( LA(1)) {
				case CONST:
				{
					declar_const(RES);
					break;
				}
				case FALGORISME:
				case DESTRUIR:
				case MENTRE:
				case SI:
				case PER:
				case CAS:
				case ESCRIURE:
				case ESCRIURELN:
				case RETORNA:
				case FFUNCIO:
				case RES:
				case JO:
				case SUPER:
				case FCONSTRUCTOR:
				case FDESTRUCTOR:
				case FMETODE:
				case FMETODE_CLASSE:
				case IDENTIFICADOR:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				instruccions();
				break;
			}
			case CONST:
			{
				declar_const(RES);
				{
				switch ( LA(1)) {
				case VAR:
				{
					declar_var(RES);
					break;
				}
				case FALGORISME:
				case DESTRUIR:
				case MENTRE:
				case SI:
				case PER:
				case CAS:
				case ESCRIURE:
				case ESCRIURELN:
				case RETORNA:
				case FFUNCIO:
				case RES:
				case JO:
				case SUPER:
				case FCONSTRUCTOR:
				case FDESTRUCTOR:
				case FMETODE:
				case FMETODE_CLASSE:
				case IDENTIFICADOR:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				instruccions();
				break;
			}
			case FALGORISME:
			case DESTRUIR:
			case MENTRE:
			case SI:
			case PER:
			case CAS:
			case ESCRIURE:
			case ESCRIURELN:
			case RETORNA:
			case FFUNCIO:
			case RES:
			case JO:
			case SUPER:
			case FCONSTRUCTOR:
			case FDESTRUCTOR:
			case FMETODE:
			case FMETODE_CLASSE:
			case IDENTIFICADOR:
			{
				instruccions();
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_17);
		}
	}
	
	public final void capzalera_funcio(
		boolean declararAmbit
	) throws RecognitionException, TokenStreamException {
		
		Token  id = null;
		ParamInfo []params=new ParamInfo[128]; ParamInfo retParam=null; int Type=0,numParams=0;
		
		try {      // for error handling
			match(FUNCIO);
			id = LT(1);
			match(IDENTIFICADOR);
			match(PAREN_ESQ);
			numParams=parametres_formals(params,0);
			match(PAREN_DRET);
			{
			switch ( LA(1)) {
			case RETORNA:
			{
				match(RETORNA);
				retParam=parametres_retorn();
				break;
			}
			case SALT_LINIA:
			case COMENTARI1:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			{
			sepa_inst();
			}
			
							// Declara la funcio. Resta un al numero de linia, ja que ha parsejat sepa_inst
							declararFuncio(id.getText(),params,numParams,retParam,declararAmbit,LT(1).getLine()-1,FUNCIO);
						
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_18);
		}
	}
	
	public final void capzalera_destructor(
		boolean declararAmbit
	) throws RecognitionException, TokenStreamException {
		
		Token  id = null;
		ParamInfo []params=new ParamInfo[1]; params[0]=null;
		
		try {      // for error handling
			match(DESTRUCTOR);
			id = LT(1);
			match(IDENTIFICADOR);
			match(PAREN_ESQ);
			match(PAREN_DRET);
			{
			sepa_inst();
			}
			
							// Declara el destructor. Resta un al numero de linia, ja que ha parsejat sepa_inst
							declararFuncio(id.getText(),params,0,null,declararAmbit,LT(1).getLine()-1,DESTRUCTOR);
						
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_19);
		}
	}
	
	public final ParamInfo  parametre() throws RecognitionException, TokenStreamException {
		ParamInfo pInfo=null;
		
		Token  id = null;
		
		try {      // for error handling
			id = LT(1);
			match(IDENTIFICADOR);
			match(DOSPUNTS);
			pInfo=tipus();
			
					pInfo.setName(id.getText());
				
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_20);
		}
		return pInfo;
	}
	
	public final ParamInfo  tipus() throws RecognitionException, TokenStreamException {
		ParamInfo typeInf=null;
		
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case ENTER:
			case REAL:
			case CARACTER:
			case BOOLEA:
			case IDENTIFICADOR:
			{
				typeInf=tipus_simple();
				break;
			}
			case TAULA:
			{
				typeInf=tipus_compost();
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_21);
		}
		return typeInf;
	}
	
	public final void instruccions() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			{
			_loop215:
			do {
				if ((_tokenSet_22.member(LA(1)))) {
					instruccio_ratlla();
					{
					sepa_inst();
					}
				}
				else {
					break _loop215;
				}
				
			} while (true);
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_23);
		}
	}
	
	public final ParamInfo  tipus_simple() throws RecognitionException, TokenStreamException {
		ParamInfo typeInf=new ParamInfo();
		
		Token  id = null;
		int Type=0;
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case IDENTIFICADOR:
			{
				id = LT(1);
				match(IDENTIFICADOR);
				
							// Si no es tracta de la propia classe que es parseja, es carrega
							// la interficie de la classe en l'ambit actual de la TS
							if (!symbolTable.getParentScopeName().equalsIgnoreCase(id.getText()))
								loadClassToSimbolTable(id.getText(),LT(1).getLine());	
						
				break;
			}
			case ENTER:
			case REAL:
			case CARACTER:
			case BOOLEA:
			{
				Type=tipus_elemental();
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			
						if (Type==0)
						{
							// No es tipus basic, es una classe
							typeInf.setIdType(CLASS_TYPE);
							typeInf.setType(getHash(id.getText()));
							typeInf.setClassName(id.getText());
						}
						else
						{
							// Tipus basic
							typeInf.setIdType(ELEMENTAL_TYPE);
							typeInf.setType(Type);
						}
					
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_21);
		}
		return typeInf;
	}
	
	public final ParamInfo  tipus_compost() throws RecognitionException, TokenStreamException {
		ParamInfo type=null;
		
		ParamInfo typeInf=null; int dim=0;
		
		try {      // for error handling
			match(TAULA);
			{
			int _cnt145=0;
			_loop145:
			do {
				if ((LA(1)==CORXET_ESQ)) {
					dim=corxets(dim+1);
				}
				else {
					if ( _cnt145>=1 ) { break _loop145; } else {throw new NoViableAltException(LT(1), getFilename());}
				}
				
				_cnt145++;
			} while (true);
			}
			match(DE);
			typeInf=tipus_simple();
			
					type=new ParamInfo();
					type.setType(typeInf.getType());
					type.setIdType(typeInf.getIdType());
					type.setClassName(typeInf.getClassName());
					type.setDimension(dim);
				
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_21);
		}
		return type;
	}
	
	public final int  tipus_elemental() throws RecognitionException, TokenStreamException {
		int Type=0;
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case ENTER:
			{
				match(ENTER);
				Type=NUMERO_ENTER;
				break;
			}
			case REAL:
			{
				match(REAL);
				Type=NUMERO_REAL;
				break;
			}
			case CARACTER:
			{
				match(CARACTER);
				Type=LITERAL_CARACTER;
				break;
			}
			case BOOLEA:
			{
				match(BOOLEA);
				Type=BOOLEA;
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_21);
		}
		return Type;
	}
	
	public final int  corxets(
		int dim
	) throws RecognitionException, TokenStreamException {
		int dimension=0;
		
		
		try {      // for error handling
			match(CORXET_ESQ);
			match(CORXET_DRET);
			dimension=dim;
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_24);
		}
		return dimension;
	}
	
	public final void llista_var(
		int typeDecl
	) throws RecognitionException, TokenStreamException {
		
		String []ids=new String[128]; int Type=0; ParamInfo typeInf=null; /** PER culpa del sepa_inst, no marca be la linia ***/
		
		try {      // for error handling
			llista_identif(ids,0);
			match(DOSPUNTS);
			typeInf=tipus();
			{
			sepa_inst();
			}
			
					// ANALISIS SEMANTIC: Declara les variables.
					int i=0;
					Type=typeInf.getType();
					while ((ids[i]!=null) && (i < ids.length))
					{
						declararVariable(ids[i],typeInf.getIdType(),Type,typeInf.getDimension(),typeInf.getClassName(),typeDecl,LT(1).getLine());
						i++;
					}
				
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_25);
		}
	}
	
	public final void llista_identif(
		String[] declName,int index
	) throws RecognitionException, TokenStreamException {
		
		Token  id = null;
		declName[index] = "";
		
		try {      // for error handling
			id = LT(1);
			match(IDENTIFICADOR);
			{
			switch ( LA(1)) {
			case COMA:
			{
				match(COMA);
				llista_identif(declName,index+1);
				break;
			}
			case DOSPUNTS:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			declName[index]=id.getText();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_26);
		}
	}
	
	public final void llista_const(
		int typeDecl
	) throws RecognitionException, TokenStreamException {
		
		Token  id = null;
		ParamInfo expInf=null,constInf=null;
		
		try {      // for error handling
			id = LT(1);
			match(IDENTIFICADOR);
			match(DOSPUNTS);
			constInf=tipus();
			match(OP_IGUAL);
			expInf=expressio();
			{
			sepa_inst();
			}
			
					// Comprova que sigui correcta la declaracio de la constant
					try
					{
						if (constInf.getIdType()==ELEMENTAL_TYPE)
						{
							checkTypes(constInf,expInf,false,true,LT(1).getLine());	// No permet fer Cast automatic
							declararConstant(id.getText(),constInf.getType(),expInf.getText(),typeDecl,LT(1).getLine());
						}
						else
							throw new NeedElementalTypeException(id.getText(),inFilename,LT(1).getLine());
					}
					catch(PSemanticException sEx)
					{
						reportError(sEx);
					}
				
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_27);
		}
	}
	
	public final ParamInfo  expressio() throws RecognitionException, TokenStreamException {
		ParamInfo factInf=new ParamInfo();
		
		int Type=0,Type1=0,Type2=0; String op=""; ParamInfo factInf1=null,factInf2=null;
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case CERT:
			case FALS:
			case NULL:
			case JO:
			case SUPER:
			case OP_NO:
			case NUMERO_ENTER:
			case NUMERO_REAL:
			case IDENTIFICADOR:
			case LITERAL_CARACTER:
			case LITERAL_CADENA:
			case PAREN_ESQ:
			case OP_SUMA:
			case OP_RESTA:
			{
				factInf1=expressio_simple();
				{
				switch ( LA(1)) {
				case OP_I:
				case OP_O:
				case OP_IGUAL:
				case OP_DIFERENT:
				case OP_MPETIT:
				case OP_MPETIT_IGUAL:
				case OP_MGRAN_IGUAL:
				case OP_MGRAN:
				{
					op=op_relacional();
					factInf2=expressio_simple();
					break;
				}
				case FER:
				case FMENTRE:
				case LLAVORS:
				case ALTRAMENT:
				case FSI:
				case FINS:
				case PAS:
				case FPER:
				case PAREN_DRET:
				case CORXET_DRET:
				case COMA:
				case PUNTCOMA:
				case SALT_LINIA:
				case OP_CAS:
				case COMENTARI1:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				break;
			}
			case CREAR:
			{
				factInf=inst_crear();
				break;
			}
			case LLEGIR:
			{
				factInf=inst_llegir();
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			
				  		  // ANALISIS SEMANTIC: comprovar de tipus
						  if (factInf1!=null)
						  {
						  	  // Parseja expressio
							  Type1=factInf1.getType();
							  if (factInf2!=null) Type2=factInf2.getType();
							  if (Type2==0) 
								Type=Type1;
							  else
							  {
								if (factInf2!=null)
									Type=checkTypes(factInf1,factInf2,true,true,LT(1).getLine());
								Type=BOOLEA;				
							  }
							  if (factInf2!=null) factInf.setText(factInf1.getText() + op + factInf2.getText());
							  else factInf.setText(factInf1.getText());
							  factInf.setDimension(factInf1.getDimension());
							  factInf.setType(Type);
							  factInf.setIdType(factInf1.getIdType());
							  factInf.setClassName(factInf1.getClassName());
						  }
						
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_28);
		}
		return factInf;
	}
	
	public final ParamInfo  variable() throws RecognitionException, TokenStreamException {
		ParamInfo factInf=null;
		
		String id="",className="";
		
		try {      // for error handling
			switch ( LA(1)) {
			case JO:
			{
				match(JO);
				{
				switch ( LA(1)) {
				case PUNT:
				{
					match(PUNT);
					factInf=post_variable(null);
					break;
				}
				case DIV:
				case MOD:
				case FER:
				case FMENTRE:
				case LLAVORS:
				case ALTRAMENT:
				case FSI:
				case FINS:
				case PAS:
				case FPER:
				case OP_I:
				case OP_O:
				case PAREN_DRET:
				case CORXET_DRET:
				case COMA:
				case PUNTCOMA:
				case SALT_LINIA:
				case OP_SUMA:
				case OP_RESTA:
				case OP_MULT:
				case OP_DIV:
				case OP_IGUAL:
				case OP_DIFERENT:
				case OP_MPETIT:
				case OP_MPETIT_IGUAL:
				case OP_MGRAN_IGUAL:
				case OP_MGRAN:
				case OP_ASSIGNACIO:
				case OP_CAS:
				case COMENTARI1:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				if (factInf==null)
					  {
					  	factInf=new ParamInfo();
						tractarJo(factInf);
					  }
					  else
						factInf.setText("this." + factInf.getText());
					
				break;
			}
			case SUPER:
			{
				match(SUPER);
				className=toStringSuperClassName();
				{
				switch ( LA(1)) {
				case PUNT:
				{
					match(PUNT);
					factInf=post_variable(null);
					factInf.setText("." + factInf.getText());
					break;
				}
				case PAREN_ESQ:
				{
					factInf=crida_funcio(className,className,CONSTRUCTOR);
					checkSuperIsFirstStatement(LT(1).getLine()); factInf.setIdType(ACCIO);
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				factInf.setText("super" + factInf.getText());
				break;
			}
			case IDENTIFICADOR:
			{
				factInf=post_variable(null);
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_29);
		}
		return factInf;
	}
	
	public final ParamInfo  post_variable(
		ParamInfo typeInf
	) throws RecognitionException, TokenStreamException {
		ParamInfo factInf=new ParamInfo();
		
		Token  id = null;
		LinkedList list=new LinkedList(); ParamInfo factInf1=null,factInf2=null,tableInf=null; String className=null; int tCrida;
		
		try {      // for error handling
			{
			id = LT(1);
			match(IDENTIFICADOR);
			
						tCrida=RES;
						if (typeInf!=null)
						{
							if (typeInf.getDimension()==0)
							{
								if (!symbolTable.getParentScopeName().equalsIgnoreCase(typeInf.getClassName()))
								{	// Si no es tracta de la classe que estem parsejant
									factInf=isDeclaredIdOfClass(id.getText(),typeInf,LT(1).getLine());
									className=typeInf.getClassName();
								}
								else  // Es tracta de la classe q estem parsejant, per tant ha de ser visible en ambit actual
									factInf=isDeclaredId(id.getText(),LT(1).getLine(),list);
								factInf.setText(factInf.getText().toLowerCase());  // Passa el text parsejat a minuscules
							}
							else
							{
								factInf.setType(ID_NOT_DECLARED);  // Per a una taula no hi ha membres declarats
								tCrida=TAULA;
							}
						}
						else
						{
							factInf=isDeclaredId(id.getText(),LT(1).getLine(),list);
							if (factInf.getIdType()!=CLASS_DEF)
								factInf.setText(factInf.getText().toLowerCase());  // Passa el text parsejat a minuscules si no es nom d'una classe
						}
						if ((factInf.getType()!=ID_NOT_DECLARED) && (factInf.getSuperClassName()!=null))
							// Si el ID esta declarat, i definit en una altre superclasse, actualitza el nom de la classe.
							className=factInf.getSuperClassName();
			
			{
			switch ( LA(1)) {
			case PUNT:
			{
				match(PUNT);
				factInf1=post_variable(factInf);
				break;
			}
			case CORXET_ESQ:
			{
				{
				tableInf=index_taula(id.getText());
				}
				checkAccesTable(factInf,tableInf,LT(1).getLine());
				{
				switch ( LA(1)) {
				case PUNT:
				{
					match(PUNT);
					factInf1=post_variable(factInf);
					break;
				}
				case DIV:
				case MOD:
				case FER:
				case FMENTRE:
				case LLAVORS:
				case ALTRAMENT:
				case FSI:
				case FINS:
				case PAS:
				case FPER:
				case OP_I:
				case OP_O:
				case PAREN_DRET:
				case CORXET_DRET:
				case COMA:
				case PUNTCOMA:
				case SALT_LINIA:
				case OP_SUMA:
				case OP_RESTA:
				case OP_MULT:
				case OP_DIV:
				case OP_IGUAL:
				case OP_DIFERENT:
				case OP_MPETIT:
				case OP_MPETIT_IGUAL:
				case OP_MGRAN_IGUAL:
				case OP_MGRAN:
				case OP_ASSIGNACIO:
				case OP_CAS:
				case COMENTARI1:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				break;
			}
			case PAREN_ESQ:
			{
				factInf2=crida_funcio(id.getText(),className,tCrida);
				
											if (factInf2!=null)
											{
												// Tracta una crida a una funcio
												if (factInf2.getType()!=TAULA)
												{
													factInf.setIdType(factInf2.getIdType());
													factInf.setType(factInf2.getType());
													factInf.setClassName(factInf2.getClassName());
													factInf.appendText(factInf2.getText());
												}
												else
												{
													// Membre LONGITUT d'una taula
													factInf.setIdType(factInf2.getIdType());
													factInf.setType(NUMERO_ENTER);
													factInf.setText("length");
												}
											}
									
				{
				switch ( LA(1)) {
				case PUNT:
				{
					match(PUNT);
					factInf1=post_variable(factInf);
					break;
				}
				case DIV:
				case MOD:
				case FER:
				case FMENTRE:
				case LLAVORS:
				case ALTRAMENT:
				case FSI:
				case FINS:
				case PAS:
				case FPER:
				case OP_I:
				case OP_O:
				case PAREN_DRET:
				case CORXET_DRET:
				case COMA:
				case PUNTCOMA:
				case SALT_LINIA:
				case OP_SUMA:
				case OP_RESTA:
				case OP_MULT:
				case OP_DIV:
				case OP_IGUAL:
				case OP_DIFERENT:
				case OP_MPETIT:
				case OP_MPETIT_IGUAL:
				case OP_MGRAN_IGUAL:
				case OP_MGRAN:
				case OP_ASSIGNACIO:
				case OP_CAS:
				case COMENTARI1:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				break;
			}
			case DIV:
			case MOD:
			case FER:
			case FMENTRE:
			case LLAVORS:
			case ALTRAMENT:
			case FSI:
			case FINS:
			case PAS:
			case FPER:
			case OP_I:
			case OP_O:
			case PAREN_DRET:
			case CORXET_DRET:
			case COMA:
			case PUNTCOMA:
			case SALT_LINIA:
			case OP_SUMA:
			case OP_RESTA:
			case OP_MULT:
			case OP_DIV:
			case OP_IGUAL:
			case OP_DIFERENT:
			case OP_MPETIT:
			case OP_MPETIT_IGUAL:
			case OP_MGRAN_IGUAL:
			case OP_MGRAN:
			case OP_ASSIGNACIO:
			case OP_CAS:
			case COMENTARI1:
			{
				
									if (((factInf.getIdType()==FUNCIO) || (factInf.getIdType()==ACCIO)) && (factInf2==null) && (factInf1==null))
									{
										// Actualiza per a que doni error de inst. no valida, ja que no ha
										// parsejat per crida (si hagues parsejat crida, factInf1==null)
										factInf.setIdType(CLASS_TYPE);
									}		  
						
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			}
				
					 	try
						{
							// Reporta missatge d'error si no esta declarat i no es una crida a una func.
							if ((factInf.getType()==ID_NOT_DECLARED) && (factInf2==null))
							{	// Actualitzem el text parsejat
								factInf.appendText(id.getText());
								throw new IdNotDeclaredException(id.getText(),inFilename,LT(1).getLine());
							}
							if (factInf1!=null)
							{
								// Tracta un camp o atribut d'una classe
								if (factInf1.getType()!=ID_NOT_DECLARED) // Si esta declarat, actualitzem el tipus
								{
									factInf.setType(factInf1.getType());
									factInf.setIdType(factInf1.getIdType());
									factInf.setDimension(factInf1.getDimension());
									factInf.setClassName(factInf1.getClassName());
									factInf.appendText("." + factInf1.getText());
								}
								else
									// Nomes actualitzem el text parsejat
									factInf.appendText("." + factInf1.getText());					
								
							}
						}
						catch(PSemanticException sEx)
						{
							reportError(sEx);
						}
					
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_29);
		}
		return factInf;
	}
	
	public final ParamInfo  crida_funcio(
		String varName,String className,int cridaType
	) throws RecognitionException, TokenStreamException {
		ParamInfo funcInfo=new ParamInfo();
		
		
						int numParams=0;
						LinkedList list=null;
						ParamInfo[] parParams=new ParamInfo[30];
						ParamInfo parInf=new ParamInfo();
						String text="";
		
						funcInfo.setText("(");	
					
		
		try {      // for error handling
			match(PAREN_ESQ);
			numParams=parametres_actuals(parParams,0,parInf);
			match(PAREN_DRET);
			
							try
							{
								// Afegeix els parametres actuals parsejats
								text=parInf.getText();
								if (text!="")
									text=text.substring(0,text.lastIndexOf(','));
								funcInfo.appendText(text);
								funcInfo.appendText(")");
								parInf=null; // marca per el GC				
								list=getListOfFunctions(varName,className,LT(1).getLine());
								// Comprovacions de tipus
								if (list!=null)
								{
									// El ID de la funcio es visible en aquest ambit, comprovem els parametres
									parInf=checkParametersOfFunction(varName,className,cridaType,list,parParams,numParams,LT(1).getLine());
									if (parInf!=null)
									{
										funcInfo.setType(parInf.getType());
										funcInfo.setIdType(parInf.getIdType());
										funcInfo.setClassName(parInf.getClassName());
									}
									else
									{
										// Identificador declarat, pero no coincideix la signatura de la funcio
										if ((cridaType==CONSTRUCTOR) && (numParams==0))
										{
											// El constructor per defecte sempre existeix, encara q no estigui declarat, d'altra manera: error.
										}
										else
										{
											funcInfo.setIdType(ACCIO);  // Per que no peti i digui que no es una instruccio valida
											throw new MetodeNotExistException(getFuncToString(varName,parParams,numParams),
																			  inFilename,LT(1).getLine());
										}
									}
								}
								else // Identificador no declarat
								{
									
									if ((cridaType==CONSTRUCTOR) && (numParams==0))
									{
										// El constructor per defecte sempre existeix, encara q no estigui declarat, d'altra manera: error.
									}
									else if (cridaType==TAULA)
									{
										if ((varName.equals(LONGITUT_OF_TAULA))  && (numParams==0))
										{
											// Acceptem el membre de taula q dona la LONGITUT de la taula
											funcInfo.setIdType(FUNCIO);
											funcInfo.setType(TAULA);
										}
										else
										{
											funcInfo.setIdType(ACCIO);  // Per que no peti i digui que no es una instruccio valida
											throw new MetodeNotExistException(getFuncToString(varName,parParams,numParams),
																			  inFilename,LT(1).getLine());
										}							
									}
									else
									{
										funcInfo.setIdType(ACCIO);  // Per que no peti i digui que no es una instruccio valida
										throw new MetodeNotExistException(getFuncToString(varName,parParams,numParams),
																		  inFilename,LT(1).getLine());							
									}
								}
							}
							catch(PSemanticException sEx)
							{
								reportError(sEx);
							}
						
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_30);
		}
		return funcInfo;
	}
	
	public final ParamInfo  index_taula(
		String varName
	) throws RecognitionException, TokenStreamException {
		ParamInfo indexInf=new ParamInfo();
		
		int dim=0;
		
		try {      // for error handling
			{
			int _cnt179=0;
			_loop179:
			do {
				if ((LA(1)==CORXET_ESQ)) {
					acces_taula(varName,indexInf);
				}
				else {
					if ( _cnt179>=1 ) { break _loop179; } else {throw new NoViableAltException(LT(1), getFilename());}
				}
				
				_cnt179++;
			} while (true);
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_30);
		}
		return indexInf;
	}
	
	public final int  parametres_actuals(
		ParamInfo []parParams,int index,ParamInfo textInf
	) throws RecognitionException, TokenStreamException {
		int numParametres=0;;
		
		ParamInfo parInf=null;
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case CERT:
			case FALS:
			case CREAR:
			case NULL:
			case LLEGIR:
			case JO:
			case SUPER:
			case OP_NO:
			case NUMERO_ENTER:
			case NUMERO_REAL:
			case IDENTIFICADOR:
			case LITERAL_CARACTER:
			case LITERAL_CADENA:
			case PAREN_ESQ:
			case OP_SUMA:
			case OP_RESTA:
			{
				parInf=parametre_actual();
				{
				switch ( LA(1)) {
				case COMA:
				{
					match(COMA);
					numParametres=parametres_actuals(parParams,index+1,textInf);
					break;
				}
				case PAREN_DRET:
				{
					numParametres=index+1;
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				break;
			}
			case PAREN_DRET:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			
					// Comprovacio dels tipus dels parametres
					if (parParams!=null)
					{
							if (parInf!=null)
							{
								parParams[index]=parInf;
								textInf.setText(parInf.getText() + "," + textInf.getText());
							}
					}
				
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_15);
		}
		return numParametres;
	}
	
	public final ParamInfo  parametre_actual() throws RecognitionException, TokenStreamException {
		ParamInfo paramInf=null;
		
		
		try {      // for error handling
			paramInf=expressio();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_20);
		}
		return paramInf;
	}
	
	public final void acces_taula(
		String varName,ParamInfo indexInf
	) throws RecognitionException, TokenStreamException {
		
		ParamInfo expInf=null;
		
		try {      // for error handling
			{
			match(CORXET_ESQ);
			expInf=expressio();
			match(CORXET_DRET);
			}
			
					// Comprova que l'expressio per indexar la taula sigui de tipus entera
					try
					{
						indexInf.incDimension();
						indexInf.appendText("[" + expInf.getText() + "]");
						if (expInf.getType()!=NUMERO_ENTER)
							throw new RequiredEnterForIndexException(varName,inFilename,LT(1).getLine());
					}
					catch(PSemanticException sEx)
					{
						reportError(sEx);
					}
				
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_31);
		}
	}
	
	public final String  op_suma_resta() throws RecognitionException, TokenStreamException {
		String val=null;
		
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case OP_SUMA:
			{
				match(OP_SUMA);
				val="+";
				break;
			}
			case OP_RESTA:
			{
				match(OP_RESTA);
				val="-";
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_32);
		}
		return val;
	}
	
	public final String  op_mult_div() throws RecognitionException, TokenStreamException {
		String val=null;
		
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case OP_MULT:
			{
				match(OP_MULT);
				val="*";
				break;
			}
			case OP_DIV:
			{
				match(OP_DIV);
				val="/";
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_33);
		}
		return val;
	}
	
	public final String  op_mod() throws RecognitionException, TokenStreamException {
		String val=null;
		
		
		try {      // for error handling
			{
			match(MOD);
			val="%";
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_33);
		}
		return val;
	}
	
	public final String  op_div() throws RecognitionException, TokenStreamException {
		String val=null;
		
		
		try {      // for error handling
			{
			match(DIV);
			val="/";
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_33);
		}
		return val;
	}
	
	public final String  op_relacional() throws RecognitionException, TokenStreamException {
		String val=null;
		
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case OP_IGUAL:
			{
				match(OP_IGUAL);
				val="==";
				break;
			}
			case OP_DIFERENT:
			{
				match(OP_DIFERENT);
				val="!=";
				break;
			}
			case OP_MPETIT:
			{
				match(OP_MPETIT);
				val="<";
				break;
			}
			case OP_MPETIT_IGUAL:
			{
				match(OP_MPETIT_IGUAL);
				val="<=";
				break;
			}
			case OP_MGRAN:
			{
				match(OP_MGRAN);
				val=">";
				break;
			}
			case OP_MGRAN_IGUAL:
			{
				match(OP_MGRAN_IGUAL);
				val=">=";
				break;
			}
			case OP_I:
			{
				match(OP_I);
				val="&&";
				break;
			}
			case OP_O:
			{
				match(OP_O);
				val="||";
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_32);
		}
		return val;
	}
	
	public final ParamInfo  factor() throws RecognitionException, TokenStreamException {
		ParamInfo factInf=new ParamInfo();
		
		Token  id1 = null;
		Token  id2 = null;
		Token  id3 = null;
		Token  id4 = null;
		String text=null; int type=0,dim=0; ParamInfo factInf1=null;
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case NUMERO_ENTER:
			{
				id1 = LT(1);
				match(NUMERO_ENTER);
				type=NUMERO_ENTER; text=id1.getText();		
				break;
			}
			case NUMERO_REAL:
			{
				id2 = LT(1);
				match(NUMERO_REAL);
				type=NUMERO_REAL; text=id2.getText();  		
				break;
			}
			case LITERAL_CARACTER:
			{
				id3 = LT(1);
				match(LITERAL_CARACTER);
				type=LITERAL_CARACTER; text=id3.getText();	
				break;
			}
			case LITERAL_CADENA:
			{
				id4 = LT(1);
				match(LITERAL_CADENA);
				type=LITERAL_CADENA; text=id4.getText();		
				break;
			}
			case CERT:
			case FALS:
			{
				text=val_boolea();
				type=BOOLEA; 									
				break;
			}
			case JO:
			case SUPER:
			case IDENTIFICADOR:
			{
				factInf1=variable();
				type=factInf1.getType(); text=factInf1.getText(); dim=factInf1.getDimension();
				break;
			}
			case PAREN_ESQ:
			{
				match(PAREN_ESQ);
				factInf1=expressio();
				match(PAREN_DRET);
				type=factInf1.getType(); text=factInf1.getText(); dim=factInf1.getDimension();
				break;
			}
			case OP_NO:
			{
				match(OP_NO);
				factInf1=factor();
				type=factInf1.getType(); text="(!" + factInf1.getText() + ")"; 
																														dim=factInf1.getDimension();
				break;
			}
			case NULL:
			{
				match(NULL);
				type=NULL; dim=0; text="null";
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			
						factInf.setType(type);
						factInf.setText(text);
						factInf.setDimension(dim);
						if (factInf1!=null)
						{
							factInf.setIdType(factInf1.getIdType());
							factInf.setClassName(factInf1.getClassName());
						}
					
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_34);
		}
		return factInf;
	}
	
	public final String  val_boolea() throws RecognitionException, TokenStreamException {
		String val=null;
		
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case CERT:
			{
				match(CERT);
				val="true";
				break;
			}
			case FALS:
			{
				match(FALS);
				val="false";
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_34);
		}
		return val;
	}
	
	public final ParamInfo  operant() throws RecognitionException, TokenStreamException {
		ParamInfo factInf=new ParamInfo();
		
		int Type=0,Type1=0,Type2=0; ParamInfo factInf1=null,factInf2=null; String op="",op2="";
		
		try {      // for error handling
			factInf1=factor();
			{
			switch ( LA(1)) {
			case OP_MULT:
			case OP_DIV:
			{
				{
				op=op_mult_div();
				factInf2=operant();
				}
				break;
			}
			case MOD:
			{
				{
				op=op_mod();
				factInf2=operant();
				}
				break;
			}
			case DIV:
			{
				{
				op2=op_div();
				factInf2=operant();
				}
				break;
			}
			case FER:
			case FMENTRE:
			case LLAVORS:
			case ALTRAMENT:
			case FSI:
			case FINS:
			case PAS:
			case FPER:
			case OP_I:
			case OP_O:
			case PAREN_DRET:
			case CORXET_DRET:
			case COMA:
			case PUNTCOMA:
			case SALT_LINIA:
			case OP_SUMA:
			case OP_RESTA:
			case OP_IGUAL:
			case OP_DIFERENT:
			case OP_MPETIT:
			case OP_MPETIT_IGUAL:
			case OP_MGRAN_IGUAL:
			case OP_MGRAN:
			case OP_CAS:
			case COMENTARI1:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			
				  // ANALISIS SEMANTIC: comprovar de tipus
				  try
				  {
					  Type1=factInf1.getType();
					  if (factInf2!=null) Type2=factInf2.getType();
					  if (Type2==0) 
						Type=Type1;
					  else
					  {
						if (op2!="")
						{
							// Tractem DIV (divisio entera): comprova que els dos factors siguin enters
							if ((Type1!=NUMERO_ENTER) || (Type2!=NUMERO_ENTER))
								throw new RequiredEnterException("els operadors del div",inFilename,LT(1).getLine());
							op=op2;
						}
						if (op.equals("%"))
						{
							// Si es MOD, comprova que els dos factors siguin enters
							if ((Type1!=NUMERO_ENTER) || (Type2!=NUMERO_ENTER))
								throw new RequiredEnterException("els operadors del mod",inFilename,LT(1).getLine());
						}
						if (factInf2!=null)
							Type=checkTypes(factInf1,factInf2,true,true,LT(1).getLine());		
					  }
					  // Text parsejat
					  factInf.setType(Type);
					  factInf.setIdType(factInf1.getIdType());
					  factInf.setClassName(factInf1.getClassName());
					  factInf.setDimension(factInf1.getDimension());  
					  if (factInf2!=null) factInf.setText(factInf1.getText() + op + factInf2.getText());
					  else factInf.setText(factInf1.getText());
				   }
				   catch (PSemanticException sEx)
				   {
				   	  reportError(sEx);
				   }
				
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_35);
		}
		return factInf;
	}
	
	public final ParamInfo  expressio_simple() throws RecognitionException, TokenStreamException {
		ParamInfo factInf=new ParamInfo();
		
		int Type=0,Type1=0,Type2=0; ParamInfo factInf1=null,factInf2=null; String op="",signe="";
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case OP_SUMA:
			case OP_RESTA:
			{
				signe=op_suma_resta();
				break;
			}
			case CERT:
			case FALS:
			case NULL:
			case JO:
			case SUPER:
			case OP_NO:
			case NUMERO_ENTER:
			case NUMERO_REAL:
			case IDENTIFICADOR:
			case LITERAL_CARACTER:
			case LITERAL_CADENA:
			case PAREN_ESQ:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			factInf1=operant();
			{
			switch ( LA(1)) {
			case OP_SUMA:
			case OP_RESTA:
			{
				op=op_suma_resta();
				factInf2=expressio_simple();
				break;
			}
			case FER:
			case FMENTRE:
			case LLAVORS:
			case ALTRAMENT:
			case FSI:
			case FINS:
			case PAS:
			case FPER:
			case OP_I:
			case OP_O:
			case PAREN_DRET:
			case CORXET_DRET:
			case COMA:
			case PUNTCOMA:
			case SALT_LINIA:
			case OP_IGUAL:
			case OP_DIFERENT:
			case OP_MPETIT:
			case OP_MPETIT_IGUAL:
			case OP_MGRAN_IGUAL:
			case OP_MGRAN:
			case OP_CAS:
			case COMENTARI1:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			
				  // ANALISIS SEMANTIC: comprovar de tipus
				  Type1=factInf1.getType();
				  if (factInf2!=null) Type2=factInf2.getType();
			if (Type2==0) 
					Type=Type1;
				  else
				  {
					if (factInf2!=null)
						Type=checkTypes(factInf1,factInf2,true,true,LT(1).getLine());
				  }
				  factInf.setType(Type);
				  factInf.setIdType(factInf1.getIdType());
				  factInf.setClassName(factInf1.getClassName());
				  factInf.setDimension(factInf1.getDimension());
				  if (factInf2!=null) factInf.setText(signe + factInf1.getText() + op + factInf2.getText());
				  else factInf.setText(signe + factInf1.getText());	  
				
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_36);
		}
		return factInf;
	}
	
	public final ParamInfo  inst_crear() throws RecognitionException, TokenStreamException {
		ParamInfo crearInf=new ParamInfo();
		
		ParamInfo typeInf=null,tableInf=null,crida=null; String className=null;
		
		try {      // for error handling
			{
			match(CREAR);
			typeInf=tipus();
			
											if (!symbolTable.getParentScopeName().equalsIgnoreCase(typeInf.getClassName()))
												// Si no es tracta de la classe que estem parsejant
												className=typeInf.getClassName();
										
			{
			switch ( LA(1)) {
			case CORXET_ESQ:
			{
				{
				tableInf=index_taula("crear");
				}
				break;
			}
			case PAREN_ESQ:
			{
				crida=crida_funcio(typeInf.getClassName(),className,CONSTRUCTOR);
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			}
			
							if (typeInf!=null)
							{
								crearInf.setType(typeInf.getType());
								crearInf.setIdType(CLASS_TYPE);
								crearInf.setClassName(typeInf.getClassName());
								crearInf.setDimension(0);
								crearInf.setText(out.getExpressionCrear(typeInf.getType(),typeInf.getClassName()));
							}
							if (tableInf!=null)
							{
								crearInf.setDimension(tableInf.getDimension());
								crearInf.setText(crearInf.getText() +  tableInf.getText());
							}
							if (crida!=null)
								crearInf.setText(crearInf.getText() + crida.getText());
						
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_28);
		}
		return crearInf;
	}
	
	public final ParamInfo  inst_llegir() throws RecognitionException, TokenStreamException {
		ParamInfo llegirInf=new ParamInfo();
		
		
		try {      // for error handling
			match(LLEGIR);
			match(PAREN_ESQ);
			match(PAREN_DRET);
			
					llegirInf.setType(LLEGIR);
					llegirInf.setDimension(0);
				
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_28);
		}
		return llegirInf;
	}
	
	public final void instruccio_ratlla() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			instruccio_simple();
			{
			switch ( LA(1)) {
			case PUNTCOMA:
			{
				match(PUNTCOMA);
				instruccio_ratlla();
				break;
			}
			case FMENTRE:
			case ALTRAMENT:
			case FSI:
			case FPER:
			case SALT_LINIA:
			case COMENTARI1:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_37);
		}
	}
	
	public final void instruccio_simple() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case JO:
			case SUPER:
			case IDENTIFICADOR:
			{
				inst_assignacio_crida_funcio();
				break;
			}
			case SI:
			{
				inst_si_llavors();
				break;
			}
			case CAS:
			{
				inst_cas();
				break;
			}
			case MENTRE:
			{
				inst_mentre();
				break;
			}
			case PER:
			{
				inst_per();
				break;
			}
			case RES:
			{
				inst_res();
				break;
			}
			case ESCRIURE:
			{
				inst_escriure();
				break;
			}
			case ESCRIURELN:
			{
				inst_escriureln();
				break;
			}
			case DESTRUIR:
			{
				inst_destruir();
				break;
			}
			case RETORNA:
			{
				inst_retorna();
				/* Dintre algoritme no, dintre de l'ambit una propietat per saber kin tipus es */
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			isFirstInstruction=false; /* S'ha parsejat una inst., ja no es la primera */
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_38);
		}
	}
	
	public final void inst_assignacio_crida_funcio() throws RecognitionException, TokenStreamException {
		
		ParamInfo sourceInf=null,destInf=null;
		
		try {      // for error handling
			destInf=variable();
			auxExpAssig=destInf;
			{
			switch ( LA(1)) {
			case OP_ASSIGNACIO:
			{
				match(OP_ASSIGNACIO);
				sourceInf=expressio();
				tractarInstAssig(destInf,sourceInf,LT(1).getLine());
				break;
			}
			case FMENTRE:
			case ALTRAMENT:
			case FSI:
			case FPER:
			case PUNTCOMA:
			case SALT_LINIA:
			case COMENTARI1:
			{
				/* En aquest cas es una crida */ tractarInstCrida(destInf,LT(1).getLine());
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			auxExpAssig=null;
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_38);
		}
	}
	
	public final void inst_si_llavors() throws RecognitionException, TokenStreamException {
		
		ParamInfo condInf=null;
		
		try {      // for error handling
			match(SI);
			condInf=expressio();
			match(LLAVORS);
			tractarInstSi(condInf,LT(1).getLine());
			{
			switch ( LA(1)) {
			case DESTRUIR:
			case MENTRE:
			case SI:
			case PER:
			case CAS:
			case ESCRIURE:
			case ESCRIURELN:
			case RETORNA:
			case RES:
			case JO:
			case SUPER:
			case IDENTIFICADOR:
			{
				instruccio_ratlla();
				{
				switch ( LA(1)) {
				case ALTRAMENT:
				{
					match(ALTRAMENT);
					out.writeEndBlock(); out.writeInstAltrament();
					instruccio_ratlla();
					break;
				}
				case FSI:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				break;
			}
			case SALT_LINIA:
			case COMENTARI1:
			{
				{
				sepa_inst();
				}
				instruccions();
				{
				switch ( LA(1)) {
				case ALTRAMENT:
				{
					match(ALTRAMENT);
					out.writeEndBlock(); out.writeInstAltrament();
					{
					sepa_inst();
					}
					instruccions();
					break;
				}
				case FSI:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			match(FSI);
			
						 		out.writeEndBlock();
							
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_38);
		}
	}
	
	public final void inst_cas() throws RecognitionException, TokenStreamException {
		
		boolean firstCond=true;
		
		try {      // for error handling
			match(CAS);
			{
			sepa_inst();
			}
			{
			int _cnt238=0;
			_loop238:
			do {
				if ((LA(1)==CORXET_ESQ)) {
					firstCond=cond_cas(firstCond);
				}
				else {
					if ( _cnt238>=1 ) { break _loop238; } else {throw new NoViableAltException(LT(1), getFilename());}
				}
				
				_cnt238++;
			} while (true);
			}
			match(FCAS);
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_38);
		}
	}
	
	public final void inst_mentre() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			capzalera_mentre();
			{
			switch ( LA(1)) {
			case DESTRUIR:
			case MENTRE:
			case SI:
			case PER:
			case CAS:
			case ESCRIURE:
			case ESCRIURELN:
			case RETORNA:
			case RES:
			case JO:
			case SUPER:
			case IDENTIFICADOR:
			{
				instruccio_ratlla();
				break;
			}
			case SALT_LINIA:
			case COMENTARI1:
			{
				{
				sepa_inst();
				}
				instruccions();
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			match(FMENTRE);
			
						 	out.writeEndBlock();
						
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_38);
		}
	}
	
	public final void inst_per() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			capzalera_per();
			{
			switch ( LA(1)) {
			case DESTRUIR:
			case MENTRE:
			case SI:
			case PER:
			case CAS:
			case ESCRIURE:
			case ESCRIURELN:
			case RETORNA:
			case RES:
			case JO:
			case SUPER:
			case IDENTIFICADOR:
			{
				instruccio_ratlla();
				break;
			}
			case SALT_LINIA:
			case COMENTARI1:
			{
				{
				sepa_inst();
				}
				instruccions();
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			match(FPER);
			
						 	out.writeEndBlock();
						
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_38);
		}
	}
	
	public final void inst_res() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			match(RES);
			out.writeInstRes();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_38);
		}
	}
	
	public final void inst_escriure() throws RecognitionException, TokenStreamException {
		
		ParamInfo expInf=null;
		
		try {      // for error handling
			match(ESCRIURE);
			match(PAREN_ESQ);
			{
			switch ( LA(1)) {
			case CERT:
			case FALS:
			case CREAR:
			case NULL:
			case LLEGIR:
			case JO:
			case SUPER:
			case OP_NO:
			case NUMERO_ENTER:
			case NUMERO_REAL:
			case IDENTIFICADOR:
			case LITERAL_CARACTER:
			case LITERAL_CADENA:
			case PAREN_ESQ:
			case OP_SUMA:
			case OP_RESTA:
			{
				expInf=expressio();
				break;
			}
			case PAREN_DRET:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			match(PAREN_DRET);
			
						if (expInf==null)
							out.writeInstEscriure("");
						else
							out.writeInstEscriure(expInf.getText());
					
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_38);
		}
	}
	
	public final void inst_escriureln() throws RecognitionException, TokenStreamException {
		
		ParamInfo expInf=null;
		
		try {      // for error handling
			match(ESCRIURELN);
			match(PAREN_ESQ);
			{
			switch ( LA(1)) {
			case CERT:
			case FALS:
			case CREAR:
			case NULL:
			case LLEGIR:
			case JO:
			case SUPER:
			case OP_NO:
			case NUMERO_ENTER:
			case NUMERO_REAL:
			case IDENTIFICADOR:
			case LITERAL_CARACTER:
			case LITERAL_CADENA:
			case PAREN_ESQ:
			case OP_SUMA:
			case OP_RESTA:
			{
				expInf=expressio();
				break;
			}
			case PAREN_DRET:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			match(PAREN_DRET);
			
						if (expInf==null)
							out.writeInstEscriureln("");
						else
							out.writeInstEscriureln(expInf.getText());		
					
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_38);
		}
	}
	
	public final void inst_destruir() throws RecognitionException, TokenStreamException {
		
		Token  id = null;
		
		try {      // for error handling
			match(DESTRUIR);
			id = LT(1);
			match(IDENTIFICADOR);
			out.writeInstDestruir(id.getText());
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_38);
		}
	}
	
	public final void inst_retorna() throws RecognitionException, TokenStreamException {
		
		Token  id = null;
		
		try {      // for error handling
			match(RETORNA);
			id = LT(1);
			match(IDENTIFICADOR);
			out.writeInstRetorna(id.getText());
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_38);
		}
	}
	
	public final boolean  cond_cas(
		boolean firstCond
	) throws RecognitionException, TokenStreamException {
		boolean fCond=false;;
		
		ParamInfo condInf=null;
		
		try {      // for error handling
			{
			match(CORXET_ESQ);
			match(CORXET_DRET);
			condInf=expressio();
			match(OP_CAS);
			tractarInstCas(condInf,firstCond,LT(1).getLine());
			{
			switch ( LA(1)) {
			case SALT_LINIA:
			case COMENTARI1:
			{
				sepa_inst();
				break;
			}
			case DESTRUIR:
			case MENTRE:
			case SI:
			case PER:
			case CAS:
			case FCAS:
			case ESCRIURE:
			case ESCRIURELN:
			case RETORNA:
			case RES:
			case JO:
			case SUPER:
			case IDENTIFICADOR:
			case CORXET_ESQ:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			instruccions();
			}
			
						out.writeEndBlock();
					
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_39);
		}
		return fCond;
	}
	
	public final void capzalera_mentre() throws RecognitionException, TokenStreamException {
		
		ParamInfo condInf=null;
		
		try {      // for error handling
			match(MENTRE);
			condInf=expressio();
			match(FER);
			
							try
							{
								if (condInf.getType()!=BOOLEA) 
									throw new RequiredBooleaException("la condicio del mentre",inFilename,LT(1).getLine());
							  	out.writeInstHeadMentre(condInf.getText());
							}
							catch(PSemanticException sEx)
							{
								reportError(sEx);
							}
						
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_40);
		}
	}
	
	public final void capzalera_per() throws RecognitionException, TokenStreamException {
		
		ParamInfo factInf=null,iniExp=null,endExp=null,pasExp=null;
		
		try {      // for error handling
			match(PER);
			{
			factInf=variable();
			match(OP_ASSIGNACIO);
			iniExp=expressio();
			}
			match(FINS);
			endExp=expressio();
			{
			switch ( LA(1)) {
			case PAS:
			{
				match(PAS);
				pasExp=expressio();
				break;
			}
			case FER:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			match(FER);
			
					tractarInstPer(factInf,iniExp,endExp,pasExp,LT(1).getLine());
				
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_40);
		}
	}
	
	
	public static final String[] _tokenNames = {
		"<0>",
		"EOF",
		"<2>",
		"NULL_TREE_LOOKAHEAD",
		"\"algorisme\"",
		"\"falgorisme\"",
		"\"const\"",
		"\"fconst\"",
		"\"var\"",
		"\"fvar\"",
		"\"taula\"",
		"\"de\"",
		"\"enter\"",
		"\"real\"",
		"\"caracter\"",
		"\"boolea\"",
		"\"cert\"",
		"\"fals\"",
		"\"div\"",
		"\"mod\"",
		"\"crear\"",
		"\"destruir\"",
		"\"null\"",
		"\"mentre\"",
		"\"fer\"",
		"\"fmentre\"",
		"\"si\"",
		"\"llavors\"",
		"\"altrament\"",
		"\"fsi\"",
		"\"per\"",
		"\"fins\"",
		"\"pas\"",
		"\"fper\"",
		"\"cas\"",
		"\"fcas\"",
		"\"escriure\"",
		"\"escriureln\"",
		"\"llegir\"",
		"\"funcio\"",
		"\"retorna\"",
		"\"ffuncio\"",
		"\"res\"",
		"\"classe\"",
		"\"fclasse\"",
		"\"hereta\"",
		"\"jo\"",
		"\"super\"",
		"\"interficie\"",
		"\"finterficie\"",
		"\"implementacio\"",
		"\"fimplementacio\"",
		"\"constructor\"",
		"\"fconstructor\"",
		"\"destructor\"",
		"\"fdestructor\"",
		"\"atributs\"",
		"\"fatributs\"",
		"\"atributs_classe\"",
		"\"fatributs_classe\"",
		"\"metodes\"",
		"\"fmetodes\"",
		"\"metodes_classe\"",
		"\"fmetodes_classe\"",
		"\"metode\"",
		"\"fmetode\"",
		"\"metode_classe\"",
		"\"fmetode_classe\"",
		"\"i\"",
		"\"o\"",
		"\"no\"",
		"NUMERO_ENTER",
		"NUMERO_REAL",
		"ELEMENTAL_TYPE",
		"CLASS_TYPE",
		"CLASS_DEF",
		"ACCIO",
		"PUBLIC",
		"PRIVATE",
		"MEMBRE_CLASSE",
		"SUPER_CLASS",
		"IDENTIFICADOR",
		"NUMERO",
		"EXPONENT",
		"LITERAL_CARACTER",
		"LITERAL_CADENA",
		"'('",
		"')'",
		"'['",
		"']'",
		"','",
		"';'",
		"'.'",
		"':'",
		"SALT_LINIA",
		"OP_SUMA",
		"OP_RESTA",
		"OP_MULT",
		"OP_DIV",
		"OP_IGUAL",
		"OP_DIFERENT",
		"OP_MPETIT",
		"OP_MPETIT_IGUAL",
		"OP_MGRAN_IGUAL",
		"OP_MGRAN",
		"'<-'",
		"'->'",
		"ESPAIS_BLANC",
		"COMENTARI1",
		"COMENTARI2"
	};
	
	private static final long _tokenSet_0_data_[] = { 2L, 0L };
	public static final BitSet _tokenSet_0 = new BitSet(_tokenSet_0_data_);
	private static final long _tokenSet_1_data_[] = { -35465849732110L, 16908303L, 0L, 0L };
	public static final BitSet _tokenSet_1 = new BitSet(_tokenSet_1_data_);
	private static final long _tokenSet_2_data_[] = { 32L, 0L };
	public static final BitSet _tokenSet_2 = new BitSet(_tokenSet_2_data_);
	private static final long _tokenSet_3_data_[] = { 17592186044416L, 0L };
	public static final BitSet _tokenSet_3 = new BitSet(_tokenSet_3_data_);
	private static final long _tokenSet_4_data_[] = { 1143492092887040L, 0L };
	public static final BitSet _tokenSet_4 = new BitSet(_tokenSet_4_data_);
	private static final long _tokenSet_5_data_[] = { 5837228067025584130L, 0L };
	public static final BitSet _tokenSet_5 = new BitSet(_tokenSet_5_data_);
	private static final long _tokenSet_6_data_[] = { 6053400849139367938L, 0L };
	public static final BitSet _tokenSet_6 = new BitSet(_tokenSet_6_data_);
	private static final long _tokenSet_7_data_[] = { 1153484454560268288L, 0L };
	public static final BitSet _tokenSet_7 = new BitSet(_tokenSet_7_data_);
	private static final long _tokenSet_8_data_[] = { 562949953421312L, 0L };
	public static final BitSet _tokenSet_8 = new BitSet(_tokenSet_8_data_);
	private static final long _tokenSet_9_data_[] = { 790601311662899296L, 131087L, 0L, 0L };
	public static final BitSet _tokenSet_9 = new BitSet(_tokenSet_9_data_);
	private static final long _tokenSet_10_data_[] = { 621715775880692000L, 131082L, 0L, 0L };
	public static final BitSet _tokenSet_10 = new BitSet(_tokenSet_10_data_);
	private static final long _tokenSet_11_data_[] = { 2319570636376113472L, 131073L, 0L, 0L };
	public static final BitSet _tokenSet_11 = new BitSet(_tokenSet_11_data_);
	private static final long _tokenSet_12_data_[] = { 2306059837494001984L, 131075L, 0L, 0L };
	public static final BitSet _tokenSet_12 = new BitSet(_tokenSet_12_data_);
	private static final long _tokenSet_13_data_[] = { -9223155208574467776L, 131084L, 0L, 0L };
	public static final BitSet _tokenSet_13 = new BitSet(_tokenSet_13_data_);
	private static final long _tokenSet_14_data_[] = { 24770347706351616L, 5L, 0L, 0L };
	public static final BitSet _tokenSet_14 = new BitSet(_tokenSet_14_data_);
	private static final long _tokenSet_15_data_[] = { 0L, 8388608L, 0L, 0L };
	public static final BitSet _tokenSet_15 = new BitSet(_tokenSet_15_data_);
	private static final long _tokenSet_16_data_[] = { 0L, 17593259786240L, 0L, 0L };
	public static final BitSet _tokenSet_16 = new BitSet(_tokenSet_16_data_);
	private static final long _tokenSet_17_data_[] = { 45038195296960544L, 10L, 0L, 0L };
	public static final BitSet _tokenSet_17 = new BitSet(_tokenSet_17_data_);
	private static final long _tokenSet_18_data_[] = { 219027303563584L, 131072L, 0L, 0L };
	public static final BitSet _tokenSet_18 = new BitSet(_tokenSet_18_data_);
	private static final long _tokenSet_19_data_[] = { 36245625299272000L, 131072L, 0L, 0L };
	public static final BitSet _tokenSet_19 = new BitSet(_tokenSet_19_data_);
	private static final long _tokenSet_20_data_[] = { 0L, 75497472L, 0L, 0L };
	public static final BitSet _tokenSet_20 = new BitSet(_tokenSet_20_data_);
	private static final long _tokenSet_21_data_[] = { 0L, 17627715993600L, 0L, 0L };
	public static final BitSet _tokenSet_21 = new BitSet(_tokenSet_21_data_);
	private static final long _tokenSet_22_data_[] = { 216828280307712L, 131072L, 0L, 0L };
	public static final BitSet _tokenSet_22 = new BitSet(_tokenSet_22_data_);
	private static final long _tokenSet_23_data_[] = { 45038239085494304L, 16777226L, 0L, 0L };
	public static final BitSet _tokenSet_23 = new BitSet(_tokenSet_23_data_);
	private static final long _tokenSet_24_data_[] = { 2048L, 16777216L, 0L, 0L };
	public static final BitSet _tokenSet_24 = new BitSet(_tokenSet_24_data_);
	private static final long _tokenSet_25_data_[] = { 512L, 131072L, 0L, 0L };
	public static final BitSet _tokenSet_25 = new BitSet(_tokenSet_25_data_);
	private static final long _tokenSet_26_data_[] = { 0L, 536870912L, 0L, 0L };
	public static final BitSet _tokenSet_26 = new BitSet(_tokenSet_26_data_);
	private static final long _tokenSet_27_data_[] = { 128L, 131072L, 0L, 0L };
	public static final BitSet _tokenSet_27 = new BitSet(_tokenSet_27_data_);
	private static final long _tokenSet_28_data_[] = { 16022241280L, 21991549566976L, 0L, 0L };
	public static final BitSet _tokenSet_28 = new BitSet(_tokenSet_28_data_);
	private static final long _tokenSet_29_data_[] = { 16023027712L, 26387448594480L, 0L, 0L };
	public static final BitSet _tokenSet_29 = new BitSet(_tokenSet_29_data_);
	private static final long _tokenSet_30_data_[] = { 16023027712L, 26387717029936L, 0L, 0L };
	public static final BitSet _tokenSet_30 = new BitSet(_tokenSet_30_data_);
	private static final long _tokenSet_31_data_[] = { 16023027712L, 26387733807152L, 0L, 0L };
	public static final BitSet _tokenSet_31 = new BitSet(_tokenSet_31_data_);
	private static final long _tokenSet_32_data_[] = { 211106236923904L, 6449922496L, 0L, 0L };
	public static final BitSet _tokenSet_32 = new BitSet(_tokenSet_32_data_);
	private static final long _tokenSet_33_data_[] = { 211106236923904L, 7471552L, 0L, 0L };
	public static final BitSet _tokenSet_33 = new BitSet(_tokenSet_33_data_);
	private static final long _tokenSet_34_data_[] = { 16023027712L, 24188425338928L, 0L, 0L };
	public static final BitSet _tokenSet_34 = new BitSet(_tokenSet_34_data_);
	private static final long _tokenSet_35_data_[] = { 16022241280L, 24162655535152L, 0L, 0L };
	public static final BitSet _tokenSet_35 = new BitSet(_tokenSet_35_data_);
	private static final long _tokenSet_36_data_[] = { 16022241280L, 24156213084208L, 0L, 0L };
	public static final BitSet _tokenSet_36 = new BitSet(_tokenSet_36_data_);
	private static final long _tokenSet_37_data_[] = { 9428795392L, 17593259786240L, 0L, 0L };
	public static final BitSet _tokenSet_37 = new BitSet(_tokenSet_37_data_);
	private static final long _tokenSet_38_data_[] = { 9428795392L, 17593394003968L, 0L, 0L };
	public static final BitSet _tokenSet_38 = new BitSet(_tokenSet_38_data_);
	private static final long _tokenSet_39_data_[] = { 34359738368L, 16777216L, 0L, 0L };
	public static final BitSet _tokenSet_39 = new BitSet(_tokenSet_39_data_);
	private static final long _tokenSet_40_data_[] = { 216828280307712L, 17593259917312L, 0L, 0L };
	public static final BitSet _tokenSet_40 = new BitSet(_tokenSet_40_data_);
	
	}
