/*  Taula de simbols.
 *  Autor: Carles Royan Salvatella
 *  Ultima modificacio: 18/06/2003
 */

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.LinkedList;
/**
 * <p>Classe que implementa la taula de simbols.</p>
 * <p>En cada entrada hi ha una llista de TNode, per implementar
 * la sobrecarrega de funcions.</p>
 * <p>En un TNode hi ha desada tota la informacio del simbol tractat</p>
 * <p>Els ambits s'implementen representant el nom de l'entrada en la taula
 * com un String amb el seguent format: ambit1::subambit1::subsubambit1:: ... ::identificador</p>
 * <p>Buscar entre els ambits consisteix nomes en tractar aquest String. Aixo permet
 * desar-ho tot en una unica taula hash</p>
 * <p>A mes, independentment de les entrades a la TS, hi ha una llista amb els metodes publics
 * declarats en la classe que es tracta, per a saber si han estat o no implementats.</p>
 * @author Carles Royan Salvatella
 * @version 1.0 - Data: 18/06/2003
 */
class SymbolTable {

	/** Emmagatzema la pila d'ambits */
	private Vector scopeStack;

	/** Taula hash en la qual estan definits tots els simbols. */
	private Hashtable symTable;
  
  	/** Nom de la superclasse de la classe que estem tractant. */
	private String superClass;
	/** Tipus de l'ambit actual: classe,metode, constructor... */
	private int typeOfScope;
	/** Llista enllazada dels metodes publics declarats en la classe. */
	public LinkedList listFunctions=null;
	
	/** Constructor per defecte */
  	public SymbolTable()  {
    	scopeStack = new Vector(10);
    	symTable = new Hashtable(533);
		superClass=null;			// Afegit per CRS
		listFunctions=new LinkedList();
	}

	/** Retorna el nom del primer ambit. */
	public String getParentScopeName()
	{
    	String scopedName;
		scopedName=scopeStack.elements().nextElement().toString();
    	return scopedName;
  	}
	/** Retorna la taula hash de simbols */
	public Hashtable getSymTable()
  	{
		return symTable;
  	}
	/** Especifica el nom de la superClasse de la classe que s'esta tractant */
	public void setSuperClassName(String className)
  	{
  		superClass=className;
  	}
	/** Retorna el nom de la superClase de la classe que s'esta tractant */
	public String getSuperClassName()
  	{
  		return superClass;
  	}
	/** Especifica el tipus d'ambit actual */
	public void setTypeOfScope(int val)
  	{
  		typeOfScope=val;
  	}
	/** Retorna el tipus d'ambit actual */
	public int getTypeOfScope()
  	{
  		return typeOfScope;
  	}
	/** Afegeix la definicio de la classe en el primer ambit, per a
	    que sigui accesible des dels altres subambits (metodes,funcions,...)  */
	public LinkedList addClassInterface(String name,LinkedList list)
	{
    	return (LinkedList)symTable.put(addScopeToName(getParentScopeName(),name),list);
  	}

	/** Empila un nou ambit a la pila d'ambits */
	public void pushScope(String s)
	{
    	scopeStack.addElement(s);
  	}

	/** Desempila l'ultim ambit de la pila d'ambits */
	public void popScope()
	{
    	int size = scopeStack.size();
    	if(size > 0)
      	scopeStack.removeElementAt(size - 1);
  	}

	/** Retorna l'ambit actual com un String */
	public String currentScopeAsString()
	{
		StringBuffer buf = new StringBuffer(100);
		boolean first = true;
		Enumeration e = scopeStack.elements();
		while(e.hasMoreElements())
		{
			if(first) 
				first = false;
			else
				buf.append("::");
			buf.append(e.nextElement().toString());
		}
		return buf.toString();
	}

	/** Afegeix l'ambit actual al nom d'un simbol */
	public String addCurrentScopeToName(String name)
	{
    	String currScope = currentScopeAsString();
    	return addScopeToName(currScope, name);
  	}

	/** Afegeix l'ambit al nom d'un simbol */
	public String addScopeToName(String scope, String name)
	{
    	if(scope == null || scope.length() > 0)
      		return scope + "::" + name;
    	else
      	return name;
  	}

	/** Elimina un ambit de la pila d'ambits passada i retorna el resultat */
	public String removeOneLevelScope(String scopeName)
	{
    	int index = scopeName.lastIndexOf("::");
    	if (index > 0)
		{
      		return scopeName.substring(0,index);
    	}
    	if (scopeName.length() > 0)
		{
        	return "";
    	}
    	return null;
 	}
  
	/** Afegeix un note a la taula de simbols, amb una llista associada
	    per permetre el polimorfisme en funcions. */
	public LinkedList add(String name,LinkedList list) 
	{
		return (LinkedList)symTable.put(addCurrentScopeToName(name),list);
	}

  /** Busca un simbol en la taula, afegint a davant tot l'ambit.
      Si no el troba, desempila l'ambit i torna a buscar.
	  En cas que finalment no el trobi, no esta declarat i retorna nul.
    */
	public LinkedList lookupNameInCurrentScope2(String name) 
	{
    	String scope = currentScopeAsString();
    	String scopedName;
    	LinkedList list = null;

    	while (list == null && scope != null)
		{
			scopedName = addScopeToName(scope, name);
			list = (LinkedList)symTable.get(scopedName);
			scope = removeOneLevelScope(scope);
    	}
    	return list;
	}

}
