/*  Analisis lexic/sintactic del compilador.
 *  Autor: Carles Royan Salvatella
 *  Ultima modificacio: 18/06/2003
 *  Dependencies: SymbolTable, ParamInfo, CpcGenJava.
 */
header {
import java.io.*;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.ListIterator;
}
options {
}

/**
 * <p>Classe que incorpora l'analisis l&egravexica del compilador. La funci&oacute principal &eacute
 * analitzar el fitxer d'entrada i proveir de tokens a la classe CpcParser, que es comunica amb el constructor:
 * {@link CpcParser#CpcParser(antlr.TokenStream) CpcParser(antlr.TokenStream)}.
 * </p>
 * @author Carles Royan Salvatella
 * @version 1.0 - Data: 18/06/2003
 */
class CpcLexer extends Lexer;
options {
	k=2;
	exportVocab=Pseudocodi; 		// Nom del fitxer d'exportacio de la gramatica
	charVocabulary = '\3'..'\377'; 	// Especifica quins caracter pot trobar el Lexer.
	caseSensitive = false;			// No distingeix majuscules i minuscules
	caseSensitiveLiterals = false;	// No distingeix majuscules i minuscules quan compara la taula literals.
}

//----------------------------------------------------------------------------
// PARAULES RESERVADES
//----------------------------------------------------------------------------
tokens
{
ALGORISME		=	"algorisme"			;	
FALGORISME		= 	"falgorisme"		;
CONST			=	"const"				;
FCONST			=	"fconst"			;
VAR				= 	"var"				;
FVAR			= 	"fvar"				;
TAULA			=	"taula"				;
DE				=	"de"				;
ENTER			=	"enter"				;
REAL			=	"real"				;
CARACTER		=	"caracter"			;
BOOLEA			=	"boolea"			;
CERT			=	"cert"				;
FALS			=	"fals"				;
DIV				=	"div"				;
MOD				=	"mod"				;
CREAR			=	"crear"				;
DESTRUIR		=	"destruir"			;
NULL			=	"null"				;
MENTRE			=	"mentre"			;
FER				=	"fer"				;
FMENTRE			=	"fmentre"			;
SI				=	"si"				;
LLAVORS			=	"llavors"			;
ALTRAMENT		=	"altrament"			;
FSI				=	"fsi"				;
PER				=	"per"				;
FINS			=	"fins"				;
PAS				=	"pas"				;
FPER			=	"fper"				;
CAS				=	"cas"				;
FCAS			=	"fcas"				;
ESCRIURE		=	"escriure"			;
ESCRIURELN		=	"escriureln"		;
LLEGIR			=	"llegir"			;
FUNCIO			=	"funcio"			;
RETORNA			=	"retorna"			;
FFUNCIO			=	"ffuncio"			;
RES				=   "res"				;
CLASSE			=	"classe"			;
FCLASSE			=	"fclasse"			;
HERETA			=	"hereta"			;
JO				=	"jo"				;
SUPER			=	"super"				;
INTERFICIE		=	"interficie"		;
FINTERFICIE		=	"finterficie"		;
IMPLEMENTACIO	=	"implementacio"		;
FIMPLEMENTACIO	=	"fimplementacio"	;
CONSTRUCTOR		=	"constructor"		;
FCONSTRUCTOR	=	"fconstructor"		;
DESTRUCTOR		=	"destructor"		;
FDESTRUCTOR		=	"fdestructor"		;
ATRIBUTS		=	"atributs"			;
FATRIBUTS		=	"fatributs"			;
ATRIBUTS_CLASSE	=	"atributs_classe"	;
FATRIBUTS_CLASSE=	"fatributs_classe"	;
METODES			=	"metodes"			;
FMETODES		=	"fmetodes"			;
METODES_CLASSE	=	"metodes_classe"	;
FMETODES_CLASSE	=	"fmetodes_classe"	;
METODE			=	"metode"			;
FMETODE			=	"fmetode"			;
METODE_CLASSE 	=	"metode_classe"		;
FMETODE_CLASSE	=	"fmetode_classe"	;
OP_I			= 	"i"					;
OP_O			=	"o"					;
OP_NO			=	"no"					;
NUMERO_ENTER; NUMERO_REAL; ELEMENTAL_TYPE; CLASS_TYPE; CLASS_DEF; ACCIO;
PUBLIC; PRIVATE; MEMBRE_CLASSE; SUPER_CLASS;
}
//----------------------------------------------------------------------------
// IDENTIFICADOR
//----------------------------------------------------------------------------
/** Regla que detecta un identificador. El text parsejat que es guarda en el token
    mant&eacute; les maj/minusc. */
IDENTIFICADOR
options {
	testLiterals=true;
}
		:
		  ('a'..'z' | '_')
		  ('a'..'z' | '_' | '0'..'9')*
		;
//----------------------------------------------------------------------------
// LITERALS
//----------------------------------------------------------------------------
/** Regla que engloba tant els n&uacute;meros enters com els decimals */
NUMERO:
	('0'..'9')+	( (PUNT ('0'..'9')+) { _ttype=NUMERO_REAL; } | /*res*/ { _ttype=NUMERO_ENTER; } ) (EXPONENT)?
	;
/** Subregla per els exponents dels n&uacute;meros */	
protected
EXPONENT
	:	('e') ('+'|'-')? ('0'..'9')+
	;
/** Regla per a detectar els literals de tipus car&agrave;cter */
LITERAL_CARACTER:
	'\'' ( ('\\')? (~'\'') )? '\''
	;
/** Regla per a detectar els literals de tipus cadena */	
LITERAL_CADENA:
	'\"' (~'\"')* '\"'
	;

//----------------------------------------------------------------------------
// SEPARADORS
//----------------------------------------------------------------------------
PAREN_ESQ	options { paraphrase="'('";	}: '('	;
PAREN_DRET	options { paraphrase="')'";	}: ')'	;
CORXET_ESQ	options { paraphrase="'['";	}: '['	;
CORXET_DRET	options { paraphrase="']'";	}: ']'	;
COMA		options { paraphrase="','";	}: ','	;
PUNTCOMA	options { paraphrase="';'";	}:	';'	;
PUNT		options { paraphrase="'.'";	}: 	'.'	;
DOSPUNTS	options { paraphrase="':'"; }:	':'	;
SALT_LINIA: ( '\n'	{newline();}
			| 	'\r')
	;		
//----------------------------------------------------------------------------
// OPERADORS
//----------------------------------------------------------------------------
OP_SUMA			options { 					 }	: '+'	;
OP_RESTA		options { 					 }	: '-'	;
OP_MULT			options { 					 }	: '*'	;
OP_DIV			options { 					 }	: '/'	;
OP_IGUAL		options { 					 }	: '='   ;
OP_DIFERENT		options { 					 }	: "<>"  ;
OP_MPETIT      	options { 					 }	: '<'   ;
OP_MPETIT_IGUAL	options { 					 }	: "<="  ;
OP_MGRAN_IGUAL	options { 					 }	: ">="  ;
OP_MGRAN		options { 					 }	: '>'   ;
OP_ASSIGNACIO	options { paraphrase="'<-'"; }	: "<-"	;
OP_CAS			options { paraphrase="'->'"; }	: "->"	;

//----------------------------------------------------------------------------
// CARACTERS A IGNORAR (Espais en blanc, tabulacions, comentaris,...)
//----------------------------------------------------------------------------
/** Espais en blanc i tabuladors: s&oacute; ignorats per l'an&agrave;lisi l&egrave;xica. */
ESPAIS_BLANC: (	' '
			|	'\t')
			{ _ttype = Token.SKIP; }
	;
/** Comentari d'una linia: aquest excepcionalment no s'ignora */
COMENTARI1:
		"//" (~('\n'|'\r'))* ('\n'|'\r'('\n')?)
		{ newline(); }
	;
/** Comentari d'una o mes linies: es ignorat per l'an&agrave;lisi l&egrave;xica. */
COMENTARI2:
		"/*"
		(	/*  El comentari pot acabar amb '\r' i '\n', i a mes,
				es pot arribar a trobar '\r' seguit de '\n', aixo
				provoca que la propia gramatica ja sigui ambigua, per
				tant desactivem el warning.
			 */
			options {
				generateAmbigWarnings=false;
			}
		:
			{ LA(2)!='/' }? '*'
		|	'\r' '\n'		{newline();}
		|	'\r'			{newline();}
		|	'\n'			{newline();}
		|	~('*'|'\n'|'\r')
		)*
		"*/"
		{ _ttype = Token.SKIP; }
	;

/**
 * <p>Classe que incorpora l'analisis sintactica del compilador. Es la classe principal que parseja
 *  el fitxer de pseudocodi. A m&eacute;s executa les comprovacions sem&aacute;ntiques i la generaci&oacute; de codi.
 * <p>Abans de parserjar un programa, es millor establir primer les seguents propietats:</p>
 * <p>
 * <ul>
 *			<li>{@link #setVersion(java.lang.String) setVersion(String)}</li>
 *			<li>{@link #setVerboseMode(boolean) setVerboseMode(boolean)}</li>
 *			<li>{@link #setInputFilename(java.lang.String) setInputFilename(String)}</li>
 * </ul>
 * </p>
 * <p>Finalment parsejar el programa cridant:</p>
 * {@link #programa(boolean) programa(boolean)}
 * </p>
 * @author Carles Royan Salvatella
 * @version 1.0 - Data: 18/06/2003
 */
class CpcParser extends Parser;
options {
	buildAST = false;
}
{
/* ******************************** Variables de la classe ****************************** */
/** Constant per identificar quan un ID o un camp d'una classe no s'ha declarat. */
private final int ID_NOT_DECLARED=-1;
/** Constant que defineix l'extensi&oacute; per defecte dels fitxers pseudocodi d'entrada. */
private final String EXT_DEF=".pse";
/** <p>Constant que defineix un m&egrave;tode acceptat en les taules.</p><p> nomTaula.longitud() 
 *  retorna un enter que especifica la llarg&agrave;ria amb que s'ha instanciat la taula.</p> */
private final String LONGITUT_OF_TAULA="longitud";
/** La taula de s&iacute;mbols utilitzada per el traductor */
private SymbolTable symbolTable=new SymbolTable();
/** S'encarrega de generar el codi Java de sortida. */
private CpcGenJava out;
/** Nom del fitxer d'entrada que es tracta. S'utilitza quan es reporten missatges d'error. */
private String inFilename;
/** Nom del fitxer de sortida que es genera. */
private String outFile;
/** Versi&oacute; del traductor, que s'imprimeix en els fitxers de sortida generats. */
private String version;
/** Auxiliar, per saber la info de la variable en una inst. d'assignacio. */
private ParamInfo auxExpAssig=null;
/** Informaci&oacute; del traductor, per imprimir al final del tractament. */
private int numErrors=0,numAvisos=0;
/** Per saber si <b>super()</b> &eacute;s la primera instrucci&oacute; d'un constructor */
private boolean isFirstInstruction=false;
/** Especifica si el mode verbose est&agrave; activat.<p>En cas afirmatiu treu missatges
    informatius durant el proc&eacute;s de traducci&oacute; */
private boolean verboseMode=false;
/* ******************************** Fi Variables de la classe ****************************** */
/** <p>Sobreescriu la funcio de reportar errors sint&agrave;ctics.</p>
	<p>Canvia una mica el missatge desat en l'excepcio per a que sigui igual al
	que surt quan es reporten missatges sem&agrave;ntics. En acabat, incrementa
	la variable de comptatge d'errors.</p> */
public void reportError(RecognitionException ex)
{
	System.err.print(inFilename + ":");
	System.err.println(ex.toString().substring(5));
	numErrors=numErrors+1;
}
/** <p>Funcio de reportar errors sem&agrave;ntics.</p>
    <p>Imprimeix per pantalla l'error i incrementa la variable de comptatge
	d'errors.</p> */
public void reportError(PSemanticException ex)
{
	System.err.println(ex);
	numErrors=numErrors+1;
}
/** <p>Sobreescriu la funci&oacute; de reportar avisos (Warnings).</p>
    <p>Imprimeix per pantalla l'av&iacute;s i incrementa la variable de comptatge
	d'avisos.</p> */
public void reportWarning(PSemanticWarning ex)
{
	System.err.println(ex);
	numAvisos=numAvisos+1;
}
/** Retorna la funci&oacute; amb els tipus dels parametres formatejats, de tal manera
	que s&oacute;n llegibles per l'usuari. */
private String getFuncToString(String varName,ParamInfo[] parParams,int numParams)
{
	String text;
	int i;
	
	text="";
	i=0;
	while (i < numParams)
	{
		text=text + "," + getTypeName(parParams[i]);
		i++;
	}
	if (text!="")
		text=text.substring(1);
	text=varName+"(" + text + ")";
	return text;
}
/** Retorna un String que representa el nom del tipus, ja sigui elemental o una classe. */
private String getTypeName(ParamInfo parInf)
{
	String typeName="";
	
	if (parInf.getIdType()==CLASS_TYPE)
		typeName=parInf.getClassName();
	else
		typeName=getElementalTypeName(parInf.getType());
	
	return typeName;
}
/** <p>Retorna el nom del tipus elemental passat com constant entera representativa (enter, real,...).</p>
    <p>La constant est&agrave; definida en la interf&iacute;cie PseudocodiTokenTypes.</p> */
private String getElementalTypeName(int type)
{
	String typeName="";
	
	switch(type)
	{
		case NUMERO_ENTER:
			typeName="enter";
			break;
		case NUMERO_REAL:
			typeName="real";
			break;
		case BOOLEA:
			typeName="boolea";
			break;
		case LITERAL_CARACTER:
			typeName="caracter";
			break;
		case LITERAL_CADENA:  // Al existir el literal, tb es pot passar per parametre a una func.
			typeName="cadena";
			break;
	}
	return typeName;
}
/** Retorna un numero &uacute;nic representatiu pera  cada classe de pseudocodi amb una funcio hash. */
private int getHash(String className)
{
	return className.hashCode();
}
/** Declara una variable en l'ambit actual de la TS */
private void 
declararVariable(String varName,int idType,int type,int dimension,String typeClassName,int typeDecl,int linia)
{
	TNode node;
	LinkedList list;
	
	try
	{
		// A causa del CASE_SENSITIVE,s'accedeix sempre en minuscules a la taula de simbols.
		if (symbolTable.lookupNameInCurrentScope2(varName.toLowerCase())!=(LinkedList)null)  
			throw new IdAlreadyDeclaredException(varName,inFilename,linia);
		else
		{
			node=new TNode();
			node.setType(type);
			node.setIdType(idType);
			node.setDimension(dimension);
			node.setText(typeClassName);
			switch(typeDecl)
			{
				case ATRIBUTS:
					node.setAccesMode(PUBLIC);
					break;
				case ATRIBUTS_CLASSE:
					node.setAccesMode(MEMBRE_CLASSE);
					break;
				default:
					node.setAccesMode(PRIVATE);
					break;
			}
			list=new LinkedList();
			list.add(node);								
			symbolTable.add(varName.toLowerCase(),list); // A causa del CASE_SENSITIVE,es tracta sempre en MIN en la TS.
		}
		// Escriu en el fitxer desti
		out.writeDeclaVar(varName.toLowerCase(),idType,type,dimension,typeClassName,typeDecl);
	}
	catch (IdAlreadyDeclaredException sEx)
	{
		reportError(sEx);
	}
}
/** Declara una constant en l'ambit actual de la TS */
private void declararConstant(String varName,int Type,String expressio,int typeDecl,int linia)
{
	TNode node;
	LinkedList list;
	
	try
	{
		// A causa del CASE_SENSITIVE,s'accedeix sempre en minuscules a la taula de simbols.
		if (symbolTable.lookupNameInCurrentScope2(varName.toLowerCase())!=null)
			throw new IdAlreadyDeclaredException(varName,inFilename,linia);
		else
		{
			node=new TNode();
			node.setType(Type);
			node.setIdType(CONST);	// Estableix que l'identificador correspon a una constant.
			switch(typeDecl)
			{
				case ATRIBUTS_CLASSE:
					node.setAccesMode(MEMBRE_CLASSE);
					break;
				default:
					node.setAccesMode(PRIVATE);
					break;
			}
			list=new LinkedList();  // El polimorfisme fa que tinguem una llista per cada entrada.
			list.add(node);			// En aquest cas nomes tractarem el primer element.
			symbolTable.add(varName.toLowerCase(),list);  // A causa del CASE_SENSITIVE,es tracta sempre en MIN en la TS.
		}
		// Escriu en el fitxer desti
		out.writeDeclaConst(varName.toLowerCase(),Type,expressio,typeDecl);
	}
	catch (IdAlreadyDeclaredException sEx)
	{
		reportError(sEx);
	}
}
/** Declara una funcio/metode/metode de classe/constructor/destructor en la TS */
private void 
declararFuncio(String scopeName,ParamInfo[] params,int numParams,ParamInfo retParam,boolean declararAmbit,int linia,int funcType)
{
	int i=0;
	TNode node;
	LinkedList list;
	
	if (declararAmbit)
	{
		if (funcType==FUNCIO)
			// Es una funcio privada, l'inserta en l'ambit actual per poder ser cridada.
			declararFuncioAmbitActual(scopeName,params,numParams,retParam,funcType,linia);		
		else
		{
			if (funcType!=DESTRUCTOR)
				// Comprova que existeix la declaracio en la interficie
				checkIsDefinedInInterficie(scopeName,params,numParams,linia);
		}
		// Crea un nou ambit per la funcio
		symbolTable.pushScope(scopeName);
		// Estableix el tipus d'ambit
		symbolTable.setTypeOfScope(funcType);		
		while ((params[i]!=null) && (i < params.length))
		{
			node=new TNode();
			/**/
			node.setType(params[i].getType());
			node.setIdType(params[i].getIdType());
			node.setText(params[i].getClassName());
			/**/
			node.setAccesMode(PRIVATE);

			list=new LinkedList();  // El polimorfisme fa que tinguem una llista per cada entrada.
			list.add(node);
			symbolTable.add(params[i].getName().toLowerCase(),list);  // afegeix els parametres en minuscules per el CASE SENSITIVE
			i++;
		}
		if (retParam!=null)
		{
			node=new TNode();
			/**/
			node.setType(retParam.getType());
			node.setIdType(retParam.getIdType());
			node.setText(retParam.getClassName());
			/**/
			node.setAccesMode(PRIVATE);
			
			list=new LinkedList();
			list.add(node);	
			symbolTable.add(retParam.getName().toLowerCase(),list);	 // Param. de retorn
			// Realitza la traduccio
			out.writeHeadFuncio(funcType,scopeName.toLowerCase(),params,numParams,retParam);	
		}
		else
		{
			// Realitza la traduccio
			out.writeHeadAccio(funcType,scopeName.toLowerCase(),params,numParams);
		}
		// Indica que la proxima instruccio, es la primera de l'ambit local
		isFirstInstruction=true;
	}
	else
	{
		// Membres d'interficie: introdueix en l'ambit actual i permet ser cridada.
		declararFuncioAmbitActual(scopeName,params,numParams,retParam,funcType,linia);
	}
}
/** Declara una funcio/metode/metode de classe/constructor/destructor en l'ambit actual, de tal manera
	que sigui visible en l'ambit i pugui ser cridada. */
private void declararFuncioAmbitActual(String scopedName,ParamInfo[] params,int numParams,ParamInfo retParam,int funcType,int linia)
{
	TNode node;
	LinkedList list;
	boolean isDefined;
	FuncInfo fInfo;
	String scopeName;
	
	try
	{
		scopeName=scopedName.toLowerCase();
		// Comprova que el nom per la funcio sigui correcte
		if ((funcType==CONSTRUCTOR) && (!scopeName.equalsIgnoreCase(symbolTable.getParentScopeName())))
			throw new ConstructorNotValidException(inFilename,linia);
		else if ((funcType!=CONSTRUCTOR) && (scopeName.equalsIgnoreCase(symbolTable.getParentScopeName())))
			throw new FuncIdNotValidException(inFilename,linia);
		else
		{	// El nom de la funcio es correcte, fa les comprovacions de visibilitat i polimorfisme
			list=symbolTable.lookupNameInCurrentScope2(scopeName);
			if (list!=(LinkedList)null)
			{
				isDefined=isDefinedFuncInList(list,params,numParams);
			}
			else
			{
				isDefined=false;
				list=new LinkedList();  // El polimorfisme fa que tinguem una llista per cada entrada.
			}
			if (!isDefined)
			{
				node=new TNode();
				node.setParamInfo(params);
				node.setNumParams(numParams);
				if (retParam!=null)
				{
					node.setType(retParam.getType());   // Param. de retorn
					node.setIdType(FUNCIO);				// Especifica que el ID es una funcio
					if (retParam.getIdType()==CLASS_TYPE)
					{
						// El parametre de retorn es una classe, desa el seu nom
						node.setText(retParam.getClassName());
					}
				}
				else
					node.setIdType(ACCIO);		// Al no tenir param. de retorn es una accio
				// Estableix la visibilitat o el mode d'acces
				switch(funcType)
				{
					case CONSTRUCTOR:
						node.setAccesMode(PUBLIC);
						break;				
					case METODE:
						node.setAccesMode(PUBLIC);
						break;
					case METODE_CLASSE:
						node.setAccesMode(MEMBRE_CLASSE);
						break;				
					case FUNCIO:
						node.setAccesMode(PRIVATE);
						break;			
				}
				list.add(node);
				symbolTable.add(scopeName,list);
				if (funcType!=FUNCIO)
				{
					// Afegeix a la llista de membres de l'interficie
					fInfo=new FuncInfo();
					fInfo.setName(scopedName);	// Aqui l'afegeix conservant majusc, per si ha de mostrar error
					fInfo.setNode(node);
					symbolTable.listFunctions.add(fInfo);
				}
			}
			else
			{
				throw new IdAlreadyDeclaredException(getFuncToString(scopedName,params,numParams),inFilename,linia);
			}
		}
	}
	catch (PSemanticException sEx)
	{
		reportError(sEx);
	}	
}
/** Comprova si la funcio est&agrave; definida, tot comprovant la signatura de les funcions de la llista. */
private boolean isDefinedFuncInList(LinkedList list,ParamInfo[] params,int numParams)
{
	boolean eq=false;
	ParamInfo[] fParams;
	int fNumParams;
	ListIterator it;
	TNode node;
	
	it=list.listIterator(0);
	while ((it.hasNext()) && (!eq))
	{
		node=(TNode)it.next();
		if ((node.getIdType()==FUNCIO) || (node.getIdType()==ACCIO))
			eq=isEqualsParamsFunc(params,numParams,node.getParamInfo(),node.getNumParams());
	}
	return eq;
}
/** Comprova si els parametres d'una funcio son iguals als d'un altra. Retorna cert si son iguals. */
private boolean isEqualsParamsFunc(ParamInfo[] params1,int nParams1,ParamInfo[] params2,int nParams2)
{
	boolean eq=true;
	int nPar;
	
	if (nParams1==nParams2)
	{
		nPar=0;
		while ((nPar < nParams1) && eq)
		{
			// Comprova exactament el tipus, no es pot fer casting a causa de la sobrecarrega.
			if (params1[nPar].getType()!=params2[nPar].getType())
				eq=false;
			else
				nPar++;
		}
	}
	else
		eq=false;
	return eq;
}
/** Comprova que tots els membres de la interficie estiguin implementats */
private void checkAllMetodesImplemented(int linia)
{
	ListIterator it;
	FuncInfo fInfo=null;
	TNode node=null;
	
	try
	{
		it=symbolTable.listFunctions.listIterator(0);
		while (it.hasNext())
		{
			fInfo=(FuncInfo)it.next();
			node=fInfo.getNode();
			if (!node.getIsImplemented())
				throw new MetodeNotImplementedException(getFuncToString(fInfo.getName(),
							node.getParamInfo(),node.getNumParams()),inFilename,linia);
		}	
	}
	catch(PSemanticException sEx)
	{
		reportError(sEx);
	}
}
/** Comprova que el metode estigui declarat en la interficie. Si hi es, marca
	que ja s'ha implementat. Si no error, ja que els metodes han d'estar declarats a la interficie */
private void checkIsDefinedInInterficie(String scopeName,ParamInfo[] params,int numParams,int linia)
{
	ListIterator it;
	boolean eq=false;
	FuncInfo fInfo=null;
	TNode node=null;
	
	try
	{
		it=symbolTable.listFunctions.listIterator(0);
		while ((it.hasNext()) && (!eq))
		{
			fInfo=(FuncInfo)it.next();
			if (scopeName.equalsIgnoreCase(fInfo.getName()))
			{
				node=fInfo.getNode();
				eq=isEqualsParamsFunc(params,numParams,node.getParamInfo(),node.getNumParams());
			}
		}
		if (!eq)
		{	// Error: els metodes han d'estar declarats en la interficie
			throw new MetodeNotDeclInterfException(getFuncToString(scopeName,params,numParams),inFilename,linia);
		}
		else
		{
			if (node.getIsImplemented())
			{
				// El metode ja s'ha implementat anteriorment!
				throw new MetodeAlreadyImplementedException(getFuncToString(scopeName,params,numParams),inFilename,linia);
			}
			else
				node.setIsImplemented(true);
		}
	}
	catch(PSemanticException sEx)
	{
		reportError(sEx);
	}
}
/** <p>Comprova que la variable sigui adequada per una assignacio.</p>
	<p>En cas que no ho sigui, mostra el missatge en pantalla.</p> */
private boolean isStoreVariable(ParamInfo var1,int linia)
{
	boolean storeVariable=false;
	
	try
	{
		if (var1!=null)
		{
			storeVariable=!((var1.getIdType()==FUNCIO) || (var1.getIdType()==ACCIO));
			if (!storeVariable)
				throw new RequiredVariableException(var1.getText(),inFilename,linia);
			else
			{
				if (var1.getIdType()==CONST)
				{
					storeVariable=false;
					throw new CantModifyConstantException(var1.getText(),inFilename,linia);
				}
			}
		}
	}
	
	catch(PSemanticException sEx)
	{
		reportError(sEx);
	}
	finally
	{
		return storeVariable;
	}
}
/* <p>Comprova que el tipus de l'identificador sigui una funci&oacute;.</p>
	<p>En el sintactic es pot escapar una linia que sigui: "var1", ja que es pensa
	que es una crida a una funcio, pero li falta els parentesis; aqu&iacute, es
	fa la comprovaci&oacute; final</p>.
*/
private void isIdFunction(ParamInfo destInf,int linia)
{
	try
	{
		if ((destInf.getIdType()!=FUNCIO) && ((destInf.getIdType()!=ACCIO)))
			throw new NotInstructionException(destInf.getText(),inFilename,linia);
	}
	catch(PSemanticException sEx)
	{
		reportError(sEx);
	}
}
/** Comprova que l'identificador estigui declarat, i retorna la info que ens interesa de la TS */
private ParamInfo isDeclaredId(String id,int linia,LinkedList list)
{
	TNode node=null;
	String funcName;
	ParamInfo idInf=new ParamInfo();
	ParamInfo typeInf=null;
	
	try
	{
 		// A causa del CASE_SENSITIVE,es tracta sempre en MIN en la TS.
		list=symbolTable.lookupNameInCurrentScope2(id.toLowerCase());
		if (list!=null) node=(TNode)list.getFirst();	// Agafem el primer ID de la llista
		else node=null;
		if (node!=null)
		{
			// Agafa tota la info
			copyNodeToParamInfo(node,idInf,id);
			// Comprovacions d'acces a les variables
			funcName=symbolTable.currentScopeAsString();
			funcName=funcName.substring(funcName.lastIndexOf("::")+2,funcName.length());
			node=null;
			list=symbolTable.lookupNameInCurrentScope2(funcName.toLowerCase());
			if (list!=null) node=(TNode)list.getFirst();
			if (node!=null)
			{
				if (node.getAccesMode()==MEMBRE_CLASSE)
				{
					if ((idInf.getAccesMode()!=MEMBRE_CLASSE) && (idInf.getAccesMode()!=PRIVATE))
						throw new CantRefenceInstanciaException(id,inFilename,linia);
				}
			}
		}
		else
		{
			idInf.setType(ID_NOT_DECLARED);	// Estableix el tipus indicant que no esta declarat
			// Va cercant per les classes pares, fins que troba el id, o arriba
			// a una classe que no te cap classe pare.
			if ((symbolTable.getSuperClassName()!=null) && (linia > 0))
			{
				// Comprova que la definicio de la classe sigui visible en la TS
				typeInf=isDeclaredId(symbolTable.getSuperClassName(),-10,new LinkedList());
				if (typeInf.getType()!=ID_NOT_DECLARED)
				{
					typeInf.setIdType(SUPER_CLASS);	 // Marca que es una superclasse.
					idInf=isDeclaredIdOfClass(id,typeInf,linia);
					// idInf es visible, retorna un ParamInfo amb tota la info,
					// si no es visible: idInf.getType()==ID_NOT_DECLARED
				}
			}
		}
	}
	catch (PSemanticException sEx)
	{
		reportError(sEx);
	}
	return idInf;
}
/** <p>Comprova que l'identificador estigui declarat en la classe, i retorna la info de la TS.</p>
	<p>El que fa &eacute;s comprovar que el camp estigui declarat com a visible en la definici&oacute;
	de la classe</p>
*/
private ParamInfo isDeclaredIdOfClass(String id,ParamInfo classInf,int linia)
{
	TNode node=null,cNode=null;
	LinkedList list=null;
	ParamInfo idInf=new ParamInfo(),superClassInf=null;
	Hashtable symTable=null;
	String identif,className;

	try
	{
		if (classInf.getType()!=ID_NOT_DECLARED)
		{	// Si la variable pare esta declarada, fa les comprovacions
			className=classInf.getClassName();
			identif=(symbolTable.addScopeToName(className,id)).toLowerCase(); // CASE_SENSITIVE: es tracta sempre en MIN en la TS.
			list=symbolTable.lookupNameInCurrentScope2(className.toLowerCase());
			if (list!=null) cNode=(TNode)list.getFirst();
		
			if (cNode==null)
			{	// No ha trobat la classe, intenta carregar-la
				loadClassToSimbolTable(className,linia);
				// Torna a provar de cercar la definicio en la TS
				list=symbolTable.lookupNameInCurrentScope2(className.toLowerCase());
				if (list!=null) cNode=(TNode)list.getFirst();			
			}
			if (cNode!=null)
			{
				// Ha trobat la definicio de la classe, carregara la interficie.
				symTable=cNode.getClassInterface();
				list=(LinkedList)symTable.get(identif);
				if (list!=null) node=(TNode)(list.getFirst());
				else node=null;			
				if (node!=null)
				{	// Ha trobat el simbol! El tractem:
					// Indica en kina superclasse esta definit el simbol (per les funcions...)
					idInf.setSuperClassName(className);			
					// S'accedeix a un membre d'una classe, comprova que el membre
					// es visible, que sigui PUBLIC.
					if (classInf.getIdType()==CLASS_DEF)
					{   // Es el nom de la classe, nomes pots accedir a membres de la classe.
						if (node.getAccesMode()==MEMBRE_CLASSE)
						{
							// Agafa tota la info				
							copyNodeToParamInfo(node,idInf,id);						
						}
						else
						{
						
							idInf.setType(ID_NOT_DECLARED);	// Estableix el tipus indicant que no esta declarat
							throw new NotMemberClassException(id,inFilename,linia);
						}
					}
					else if (classInf.getIdType()==SUPER_CLASS)
					{
						// S'agafa el ID per herencia, per tant tot es visible.
						// Agafa tota la info
						copyNodeToParamInfo(node,idInf,id);
					}
					else
					{
						switch(node.getAccesMode())
						{
							case PUBLIC: // PUBLIC, es pot accedir
								// Agafa tota la info
								copyNodeToParamInfo(node,idInf,id);
								break;				
							case PRIVATE: // No es pot accedir
								idInf.setType(ID_NOT_DECLARED);	// Estableix el tipus indicant que no esta declarat
								throw new PrivateAccesException(id,inFilename,linia);		
							case MEMBRE_CLASSE:
								idInf.setType(ID_NOT_DECLARED);	// Estableix el tipus indicant que no esta declarat
								throw new ClassMemberAccesException(id,inFilename,linia);
							default:
								idInf.setType(ID_NOT_DECLARED);	// Estableix el tipus indicant que no esta declarat
								throw new IdNotDeclaredException(id,inFilename,linia);
						}
					}
				}
				else
				{
					// Va cercant per les classes pares, fins que troba el id, o arriba
					// a una classe que no te cap classe pare.
					if (cNode.getSuperClassName()!=null)
					{
						// Busca en l'ambit de la classe pare
						superClassInf=new ParamInfo();
						superClassInf.setClassName(cNode.getSuperClassName());
						superClassInf.setIdType(classInf.getIdType());
						superClassInf.setType(getHash(cNode.getSuperClassName()));
						idInf=isDeclaredIdOfClass(id,superClassInf,linia);
						// idInf es visible, retorna un ParamInfo amb tota la info,
						// si no es visible: idInf.getType()==ID_NOT_DECLARED
					}
					else
						idInf.setType(ID_NOT_DECLARED);	// Estableix el tipus indicant que no esta declarat
				}
			}
			else
				// No esta la definicio de la classe en la TS, ni pot carregar el fitxer
				throw new CantLoadClassFileException(className,inFilename,linia);
		}
	}
	catch (PSemanticException sEx)
	{
		reportError(sEx);
	}
	return idInf;		
}
/** Copia la informacio desada en la TS (TNode) a la informacio del parametre ParamInfo */
private void copyNodeToParamInfo(TNode node,ParamInfo idInf,String text)
{
	idInf.setType(node.getType());
	idInf.setIdType(node.getIdType());
	idInf.setDimension(node.getDimension());
	idInf.setText(text);
	idInf.setClassName(node.getText());			/************************** Modificar el text del Node per className ************/
	idInf.setAccesMode(node.getAccesMode());
}
/** Tracta l'acces a una taula, i comprova que les dimensions concordin */
private void checkAccesTable(ParamInfo factInf,ParamInfo tableInf,int linia)
{
	try
	{
		if (factInf.getDimension()>0)
		{	// Es comprova que la variable sigui efectivament una taula
			if (tableInf.getDimension() > factInf.getDimension())
				throw new BadTableDimensionException(factInf.getText(),inFilename,linia);
			// Afegeix les referencies [] de la taula al nom de variable
			factInf.appendText(tableInf.getText());
			// Actualitza les dimensions per l'element referenciat
			factInf.setDimension(factInf.getDimension()-tableInf.getDimension());
		}
		else
			throw new RequiredTableException(factInf.getText(),inFilename,linia);
	}
	catch(PSemanticException sEx)
	{
		reportError(sEx);
	}
}
/** Retorna el node que conte la informacio de la funcio */
private LinkedList getListOfFunctions(String varName,String className,int linia)
{
	LinkedList list=null;
	TNode node=null;
	String identif;
	
	if (className==null)
	{
		list=symbolTable.lookupNameInCurrentScope2(varName.toLowerCase()); // CASE_SENSITIVE: Acces a la TS en minuscules
	}
	else
	{
		if (className!="")
		{
			// Busca el nom de la funcio en la classe
			identif=(symbolTable.addScopeToName(className,varName)).toLowerCase();
			list=symbolTable.lookupNameInCurrentScope2(className.toLowerCase());
			if (list!=null) node=(TNode)list.getFirst();
			if (node!=null)
			{
				list=(LinkedList)node.getClassInterface().get(identif);
				if (list!=null) node=(TNode)(list.getFirst());
				else node=null;
			}
			else
			{
				System.err.println("No es troben els simbols de la classe:"+className);
				numErrors+=1;	
			}
		}
	}
	return list;
}
/** Comprova els parametres de la funcio. Si falla, intenta buscar-la en les classes pares, si n'hi ha. */
private ParamInfo 
checkParametersOfFunction(String varName,String className,int cridaType,LinkedList list,ParamInfo[] parParams,int numParams,int linia)
{
	ParamInfo parInf,classInf=new ParamInfo();
	String superClass=null,identif;
	TNode cNode=null,node=null;
	
	parInf=checkFuncParams(list,parParams,numParams,linia);
	if (parInf==null)
	{
		// Intenta carregar la classe pare
		if (className==null) 
		{
			superClass=symbolTable.getSuperClassName();
		}
		else
		{
			identif=(symbolTable.addScopeToName(className,varName)).toLowerCase();	// CASE_SENSITIVE: Acces a la TS en minuscules
			list=symbolTable.lookupNameInCurrentScope2(identif);
			if (list!=null) node=(TNode)list.getFirst();
			if (list==null)
			{	// No ha trobat la classe, intenta carregar-la
				loadClassToSimbolTable(className,linia);
				// Torna a provar de cercar la definicio en la TS
				list=symbolTable.lookupNameInCurrentScope2(className.toLowerCase());
				if (list!=null) node=(TNode)list.getFirst();			
			}
			if (node!=null)
			{
				superClass=node.getSuperClassName();
			}
		}
		if (superClass!=null)
		{
			// Busquem l'ambit on sigui visible el simbol (recorre les classes pares)
			classInf.setClassName(superClass);
			classInf.setIdType(SUPER_CLASS);
			classInf.setType(getHash(superClass));
			
			if (cridaType==CONSTRUCTOR)
				varName=superClass;
			
			parInf=isDeclaredIdOfClass(varName,classInf,linia);
			
			if (parInf.getType()!=ID_NOT_DECLARED)
			{
				parInf=null;
				list=getListOfFunctions(varName,superClass,linia);
				// Comprovem els parametres recursivament
				parInf=checkParametersOfFunction(varName,superClass,cridaType,list,parParams,numParams,linia);
			}
			else
				parInf=null; 	// No s'ha trobat la funcio.
		}
		else
		{
			// No ha trobat la signatura de la funcio en les superclasses, per tant dona error
			parInf=null;
		}
	}
	return parInf;	
}
/** Comprova que els parametres passats a la funcio, siguin correctes (te en compte sobrecarrega de noms) */
private ParamInfo checkFuncParams(LinkedList list,ParamInfo[] parParams,int nParParams,int linia)
{
	int i,nRePar;
	ParamInfo[] rePar;
	ParamInfo fucInfo=null,nodeInf;	
	TNode node=null;
	ListIterator it;
	boolean trobat=false;
	
	it=list.listIterator(0);
	while ((it.hasNext()) && (!trobat))
	{
		trobat=true;
		node=(TNode)it.next();
		if ((node.getIdType()==FUNCIO) || (node.getIdType()==ACCIO))
		{
			nRePar=node.getNumParams();
			rePar=node.getParamInfo();
			if (nRePar!=nParParams)
			{
				trobat=false;
			}			
			else
			{
				i=0;
				while ((i < nRePar) && (trobat))
				{
					if (checkTypes(rePar[i],parParams[i],false,false,linia)==0)
						trobat=false;
					else
						i++;
				}
			}
		}
		else
			trobat=false;
	}
	// Recull la informacio de la funcio
	if (trobat)
	{
		fucInfo=new ParamInfo();
		fucInfo.setType(node.getType());
		fucInfo.setIdType(node.getIdType());
		if (node.getText()!=null)
		{
			// El parametre de retorn es una classe
			fucInfo.setClassName(node.getText());
		}
	}
	return fucInfo;
}
/** <p>Comprova que els dos tipus siguin compatibles.</p><p>Si doAutocast es cert, llavors intenta fer un cast automatic.</p>
    <p>Si reportError es cert, en cas que siguin incompatibles mostrara un missatge d'error.</p>
*/
private int checkTypes(ParamInfo pInf1,ParamInfo pInf2,boolean doAutoCast,boolean reportError,int linia)
{
	int finalType=0;
	
	try
	{
		if (auxExpAssig!=null)
		{
			// Comprova tipus pInf1 amb pInf2 amb assignacio
			if (pInf1.getDimension()!=pInf2.getDimension())
				throw new TypeInvalidException(inFilename,linia);	// Dimensions no valides, no diu res del tipus.
			else
			{
				if ((pInf1.getType()!=pInf2.getType()) && (pInf2.getType()!=NULL))
				{
					if (doAutoCast)
					{
						finalType=tryCast(auxExpAssig,pInf1,pInf2,linia);
						if (finalType==0)
						{
							if (reportError) 
								throw new TypeInvalidException(getTypeName(pInf1),getTypeName(pInf2),inFilename,linia);
						}
					}
					else
						if (reportError)
							throw new TypeInvalidException(getTypeName(pInf1),getTypeName(pInf2),inFilename,linia);
				}
				else
					finalType=pInf1.getType();  // Son iguals
			}
		}
		else
		{
			// Comprova tipus pInf1 amb pInf2 sense assignacio
			if (pInf1.getDimension()!=pInf2.getDimension())
				throw new TypeInvalidException(inFilename,linia);	// Dimensions no valides, no diu res del tipus.
			else
			{
				if ((pInf1.getType()!=pInf2.getType()) && (pInf2.getType()!=NULL))
				{
					if (doAutoCast)
					{
						finalType=tryCast(auxExpAssig,pInf1,pInf2,linia);
						if (finalType==0)
						{
							if (reportError)
								throw new TypeInvalidException(getTypeName(pInf1),getTypeName(pInf2),inFilename,linia);
						}
					}
					else if ((pInf1.getIdType()==CLASS_TYPE) && (pInf2.getIdType()==CLASS_TYPE))
					{
						if (tryObjectCast(pInf1,pInf2))
							finalType=pInf1.getType();  // pInf1 es el tipus del parametre formal
					}
					else
						if (reportError)
							throw new TypeInvalidException(getTypeName(pInf1),getTypeName(pInf2),inFilename,linia);
				}
				else
					finalType=pInf1.getType();  // Son iguals
			}
		}
	}
	catch(PSemanticException sEx)
	{
		reportError(sEx);
	}
	return finalType;
}
/** Comprova si es pot fer un cast entre classes, tot utilitzant la herencia de classes */
private boolean tryObjectCast(ParamInfo pReq,ParamInfo pPar)
{
	boolean cast=false,continua=true;
	LinkedList list;
	TNode cNode=null;
	String superClass;

	superClass=pPar.getClassName();
	while ((superClass!=null) && (!cast))
	{
		// Carrega la info de superClass
		list=symbolTable.lookupNameInCurrentScope2(superClass.toLowerCase());
		if (list!=null) // Faltaria potser fer un intent de carregar el fitxer si no tenim al def a la TS
		{
			cNode=(TNode)list.getFirst();
		}

		superClass=cNode.getSuperClassName();  // Actualitza la classe pare
		if (superClass!=null)
		{
			// Si la classe pare es igual a la que es requereix, es fa cast
			if (superClass.equals(pReq.getClassName()))
			{
				cast=true;		// Qualsevol fill es pot fer passar per el seu pare (no a l'inreves)
			}
		}
	}
	return cast;
}
/** Intenta fer cast de tipus elementals. Si ha sigut exitos, retorna el tipus del cast, sino retorna 0. */
private int tryCast(ParamInfo destInf,ParamInfo pInf1,ParamInfo pInf2,int linia)
{
	int finalType=0, desType;
	String aux;

	try
	{
		if (auxExpAssig!=null)
		{
			if (auxExpAssig.getIdType()==ELEMENTAL_TYPE)
			{
				desType=auxExpAssig.getType();
				switch(desType)
				{
					case NUMERO_ENTER:
						// Canvi de numero real a enter
						if ((pInf1.getType()==NUMERO_REAL) && (pInf2.getType()==NUMERO_ENTER))
						{
							aux=pInf1.getText();
							pInf1.setType(NUMERO_ENTER);
							pInf1.setText("((int)(" + pInf1.getText() + "))");
							finalType=NUMERO_ENTER;
							throw new LossPrecisionWarningException(aux,inFilename,linia);
						}
						else if ((pInf1.getType()==NUMERO_ENTER) && (pInf2.getType()==NUMERO_REAL))
						{
							aux=pInf1.getText();
							pInf2.setType(NUMERO_ENTER);
							pInf2.setText("((int)(" + pInf2.getText() + "))");
							finalType=NUMERO_ENTER;
							throw new LossPrecisionWarningException(aux,inFilename,linia);
						}
						break;
					case NUMERO_REAL:
						// Canvi de numero enter a real (no cal generar warning.
						if ((pInf1.getType()==NUMERO_ENTER) && (pInf2.getType()==NUMERO_REAL))
						{
							pInf1.setType(NUMERO_REAL);
							pInf1.setText("((double)(" + pInf1.getText() + "))");
							finalType=NUMERO_REAL;
						}
						else if ((pInf1.getType()==NUMERO_ENTER) && (pInf2.getType()==NUMERO_ENTER))
						{
							pInf2.setType(NUMERO_REAL);
							pInf2.setText("((double)(" + pInf2.getText() + "))");
							finalType=NUMERO_REAL;
						}
						break;					
					default:
						// Canvi de numero real a enter
						if ((pInf1.getType()==NUMERO_REAL) && (pInf2.getType()==NUMERO_ENTER))
						{
							aux=pInf1.getText();
							pInf1.setType(NUMERO_ENTER);
							pInf1.setText("((int)(" + pInf1.getText() + "))");
							finalType=NUMERO_ENTER;
							throw new LossPrecisionWarningException(aux,inFilename,linia);
						}
						else if  ((pInf1.getType()==NUMERO_ENTER) && (pInf2.getType()==NUMERO_REAL))
						{
							aux=pInf1.getText();
							pInf2.setType(NUMERO_ENTER);
							pInf2.setText("((int)(" + pInf2.getText() + "))");
							finalType=NUMERO_ENTER;
							throw new LossPrecisionWarningException(aux,inFilename,linia);
						}
						break;			
				}
			}
		}
	}
	catch(PSemanticWarning sEx)
	{	
		reportWarning(sEx);
	}
	finally
	{
		return finalType;
	}
}
/** Tracta l'instruccio d'assignacio: Fa comprobacions semantiques i la traduccio. */
private void tractarInstAssig(ParamInfo destInf,ParamInfo sourceInf,int linia)
{
	try
	{
		if (isStoreVariable(destInf,linia))
		{
			// Comprova els tipus
			if (sourceInf.getType()!=LLEGIR)
			{
				checkTypes(destInf,sourceInf,true,true,linia);
			}
			else
			{
				if (destInf.getIdType()!=ELEMENTAL_TYPE)
					throw new CantReadVariableException(destInf.getText(),inFilename,linia);
				sourceInf.setText(out.getExpressionLlegir(destInf.getType()));
			}
		}
		out.writeInstAssig(destInf.getText(),sourceInf.getText());
	}
	catch(PSemanticException sEx)
	{
		reportError(sEx);
	}
}
/** Retorna el nom de la classe pare. Si no en te cap, retorna cadena buida: "" */
private String toStringSuperClassName()
{
	String sClass;
	
	if (symbolTable.getSuperClassName()==null)
		sClass="";
	else
		sClass=symbolTable.getSuperClassName();

	return sClass;
}
/** Tracta l'instruccio que es una crida a una accio/funcio: fa comprovacions semantiques i la corresponent traduccio */
private void tractarInstCrida(ParamInfo cridaInf,int linia)
{
	if (cridaInf!=null)
	{
		isIdFunction(cridaInf,linia);
		out.writeInstCrida(cridaInf.getText());
	}
}
/** Tracta l'instruccio per: fa comprovacions semantiques i la corresponent traduccio */
private void tractarInstPer(ParamInfo varInf,ParamInfo iniExp,ParamInfo endExp,ParamInfo pasExp,int linia)
{
	String pas="";
	
	try
	{
		if (isStoreVariable(varInf,linia))
		{
			// Comprova els tipus
			if (varInf.getType()!=NUMERO_ENTER)
				throw new RequiredEnterException("la variable del per",inFilename,linia);
			if (iniExp.getType()!=NUMERO_ENTER)
				throw new RequiredEnterException("l'expressio d'inici del per",inFilename,linia);
			if (endExp.getType()!=NUMERO_ENTER)
				throw new RequiredEnterException("l'expressio de fi del per",inFilename,linia);

			if (pasExp!=null)
			{
				if (pasExp.getType()!=NUMERO_ENTER)
					throw new RequiredEnterException("l'expressio del pas",inFilename,linia);
				pas=pasExp.getText();
			}
			out.writeInstPer(varInf.getText(),iniExp.getText(),endExp.getText(),pas);
		}
	}
	catch(PSemanticException sEx)
	{
		reportError(sEx);
	}
}
/** Tracta l'instruccio si: Fa comprovacions semantiques i la corresponent traduccio */
private void tractarInstSi(ParamInfo condInf,int linia)
{
	try
	{
		if (condInf.getType()!=BOOLEA) 
			throw new RequiredBooleaException("la condicio del si",inFilename,linia);
		out.writeInstSi(condInf.getText());
	}
	catch(PSemanticException sEx)
	{
		reportError(sEx);
	}
}
/** Tracta l'instruccio cas: Fa les comprovacions semantiques i la corresponent traduccio */
private void tractarInstCas(ParamInfo condInf,boolean firstCond,int linia)
{
	try
	{
		if (condInf.getType()!=BOOLEA) 
			throw new RequiredBooleaException("la condicio del cas",inFilename,linia);
		if (firstCond)
			out.writeInstCasPrimer(condInf.getText());
		else
			out.writeInstCasAltres(condInf.getText());
	}
	catch(PSemanticException sEx)
	{
		reportError(sEx);
	}
}
/** Tracta quan es troba la referencia a la propia classe: JO, modificant el ParamInfo */
private void tractarJo(ParamInfo factInf)
{
	factInf.setText("this");
	factInf.setIdType(CLASS_TYPE);
	factInf.setType(getHash(symbolTable.getParentScopeName()));
	factInf.setClassName(symbolTable.getParentScopeName());
}
/** Comprova que super es la primera instruccio del constructor */
private void checkSuperIsFirstStatement(int linia)
{
	try
	{
		if (symbolTable.getTypeOfScope()!=CONSTRUCTOR)
		{
			throw new CallConstSuperException(inFilename,linia);
		}
		else
		{
			if (!isFirstInstruction)
				throw new SuperFirstStatementException(inFilename,linia);
		}
	}
	catch(PSemanticException sEx)
	{
		reportError(sEx);
	}
}
/** Especifica la versio del traductor, que s'escriu en els fitxers generats. */
public void setVersion(String ver)
{
	version=ver;
}
/** Especifica si esta actiu el mode Verbose, que reporta a l'usuari molta informacio
	referent a les tasques que duu a terme el traductor */
public void setVerboseMode(boolean verbose)
{
	verboseMode=verbose;
}
/** Especifica el nom del fitxer d'entrada per imprimir-lo ens els missatges d'errors */
public void setInputFilename(String filename)
{
	inFilename=filename;
	openOutFilename();
}
/** Obre el fitxer de sortida per a poder escriure-hi. */
private void openOutFilename()
{
	outFile=genRandomName();
	out=new CpcGenJava(outFile,version,verboseMode);
}
/** Genera un nom a l'atzar per el fitxer temporal de sortida */
private String genRandomName()
{
	String fName="_";
	char car;
	int i;
	
	for(i=0;i<5;i++)
	{
		car=(char)Math.round((Math.random()*25) + 97);
		fName=fName + car;
	}
	fName=fName + ".tmp";
	return fName;
}
/** Tracta l'herencia de la classe, quan aquesta es declarada */
private void tractarHerencia(String className,int linia)
{
	if (!symbolTable.getParentScopeName().equalsIgnoreCase(className))  // CASE_SENSITIVE: no es te en compte majus/minus
		loadClassToSimbolTable(className,linia);
	else
	{
		System.err.println("Error: " + linia + ": no pots heretar de tu mateix!");
		numErrors+=numErrors+1;
	}
	
	symbolTable.setSuperClassName(className);
}
/** Retorna la TS confeccionada fins al moment */
public SymbolTable getSymbolTable()
{
	return symbolTable;
}
/** Retorna el numero d'avisos generats */
public int getNumAvisos()
{
	return numAvisos;
}
/** Retorna el nom de fitxer de sortida en el que s'escriu */
public String getOutFile()
{
	return outFile;
}
/** Tanca el fitxer de sortida en el que escrivia */
public void closeOutFile()
{
	out.closeFile();
}
/** <p>Carrega tot el que es public d'una classe (la seva interficie).</p>
	<p> Per fer-ho, intenta traduir el fitxer de classe. Un cop traduit, agafa la taula de 
	simbols generada per la traduccio de la classe externai la desa en un node. Aquest node
	es introduit en la nostra TS, tot identificant-lo com una definicio de classe (CLASS_DEF).</p>
*/
private void loadClassToSimbolTable(String classname,int linia)
{
	try
	{
		CpcLexer lexer;
		CpcParser parser;
		String filename;
		TNode node=null;
		LinkedList list=null;
		int numErrClass;
		
		// Comprova que no estigui carregada ja
		list=symbolTable.lookupNameInCurrentScope2(classname.toLowerCase()); // Accedeix en minus a la TS
		if (list!=null) node=(TNode)list.getFirst();
		if (node==null)
		{	// Carrega la interficie de la classe
			filename=classname+EXT_DEF;
			if (verboseMode)
				System.err.println("[carregant la classe " + filename + "]");
			
			// Llegeix i parseja el fitxer de la classe
			lexer=new CpcLexer(new DataInputStream(new FileInputStream(filename)));
			parser=new CpcParser(lexer);
			parser.setVersion(version);
			parser.setVerboseMode(verboseMode);
			parser.setInputFilename(filename);
			numErrClass=parser.programa(true);
			
			// L'inserta a la taula de simbols en l'ambit actual com a classe
			node=new TNode();
			node.setIdType(CLASS_DEF);
			node.setText(classname);
			node.setClassInterface(parser.getSymbolTable().getSymTable());
			node.setSuperClassName(parser.getSymbolTable().getSuperClassName());
			
			list=new LinkedList();
			list.add(node);
			symbolTable.addClassInterface(classname.toLowerCase(),list); // L'inserta en la TS en minus per el CASE_SENSITIVE
			numErrors+=numErrClass;  // Afegeix els errors trobats a la classe
			numAvisos+=parser.getNumAvisos();    // Afegeix els avisos trobats a la classe
		}
		else
		{
		
			if (node.getIdType()!=CLASS_DEF)
			{
				System.err.println("No es un tipus valid: "+ classname);
			}
		}
	}
	catch (FileNotFoundException e)
	{
		reportError(new CantLoadClassFileException(classname,inFilename,linia));
	}
	catch (Exception e)
	{
		e.printStackTrace();
		System.exit(-1);
	}		
}
private void tractarFitxerFinal()
{
	/** Tracta el fitxer final, si hi ha error esborra el fitxer temporal, altrament
		el renombra al nom final */
	String finalFile;
	
	finalFile=inFilename.substring(0,inFilename.lastIndexOf('.'))+".java";
	if (numErrors > 0)
	{
		// Esborra el fitxer
		(new File(outFile)).delete();
	}
	else
	{
		// Renombra el fitxer
		(new File(outFile)).renameTo(new File(finalFile));
		outFile=finalFile;
	}
}
}


// isNeedBeClass: Necessariament el programa ha de ser una classe (es crida
// des d'un altre programa per resoldre els simbols d'una classe.
/** <p>Regla principal, que engloba qualsevol fitxer de pseudocodi.</p>
	<p>Aquesta regla es la que s'ha de cridar per poder parsejar un fitxer. El parametre
	boolea isNeedBeClass especifica, si es cert, que el fitxer a tractar ha de ser
	obligatoriament	una classe.
*/
programa[boolean isNeedBeClass] returns[int numErr=-1]
	{
		if (verboseMode)
			System.err.println("[traduint el fitxer " + inFilename + "]");
	}
	: ((sepa_inst) | /*res*/) (algorisme | numErr=classe)
	{ 	if (isNeedBeClass && numErr==-1)
		{
			System.err.println("Error: Es necessita una classe");
			numErrors+=1;
		}

		if (!isNeedBeClass)
		{
			if (numErrors > 0)
			{
				// Si hi ha errors, reporta el numero total
				if (numErrors==1)
					System.err.print(numErrors + " error ");
				else 
					System.err.print(numErrors + " errors ");
			}
			numErr=numErrors;
			if (numAvisos > 0)
			{
				// Si hi ha avisos, reporta el numero total
				if (numAvisos==1)
					System.err.print(numAvisos + " avis");
				else 
					System.err.print(numAvisos + " avisos");			
			}
			if ((numAvisos > 0) || (numErrors > 0))
				System.err.println();
		}
		tractarFitxerFinal();
		if ((verboseMode) && (numErrors==0))
			System.err.println("[fitxer de sortida " + outFile + " escrit correctament]");
	}
	;
	
/** sepa_inst: regla que especifica el separador d'instruccions */
sepa_inst: ( SALT_LINIA | COMENTARI1)+
	;

algorisme: 	ALGORISME id:IDENTIFICADOR
		  	{
				symbolTable.pushScope(id.getText());
				symbolTable.setTypeOfScope(ALGORISME);
				out.writeHeadOfClass(id.getText(),"");
				out.writeHeadOfMain(); 
			}
		  	sepa_inst bloc_algorisme FALGORISME (sepa_inst | EOF)
			{
				symbolTable.popScope(); // S'elimina el primer ambit
				out.writeEndBlock();  // Final de la funcio main
				out.checkDefaultConstructor(); // Afegeix un const. per defecte
				out.writeEndBlock();  // Final de la classe principal
				closeOutFile();
			}
	;

classe returns[int numErr=0]
		: CLASSE id:IDENTIFICADOR (HERETA hId:IDENTIFICADOR)? (sepa_inst)
		{ 
			symbolTable.pushScope(id.getText().toLowerCase());		// CASE_SENSITIVE: Acces a TS en minusc
			symbolTable.setTypeOfScope(CLASSE);
			if (hId!=null)
			{
				tractarHerencia(hId.getText(),LT(1).getLine());
				out.writeHeadOfClass(id.getText(),hId.getText());
			}
			else
				out.writeHeadOfClass(id.getText(),"");
		}
		bloc_classe
		FCLASSE (sepa_inst | EOF)
		{
			symbolTable.popScope(); // S'elimina el primer ambit
			out.checkDefaultConstructor(); // Comprova que hi hagi un const. per defecte
			out.writeEndBlock();	// Final de classe
			out.closeFile();
			checkAllMetodesImplemented(LT(1).getLine());  // Comprova que tots els membres de la interficie estiguin implementats
			// Retorna el numero d'errors
			numErr=numErrors;
		}
	;
bloc_classe: (bloc_interficie)? (bloc_implementacio)?
	;
bloc_interficie: INTERFICIE sepa_inst
				 (bloc_atribut_classe)? (bloc_atribut)?
				 (bloc_metode_classe)? (bloc_metode)?
				 FINTERFICIE sepa_inst
	;
declar_atributs: 	(
			  		  (bloc_atribut) (bloc_atribut_classe)?
					| (bloc_atribut_classe) (bloc_atribut)?
					)
	;
bloc_atribut:		ATRIBUTS sepa_inst
					(declar_var[ATRIBUTS])?
					FATRIBUTS sepa_inst
	;
bloc_atribut_classe:	ATRIBUTS_CLASSE sepa_inst
						( declar_var[ATRIBUTS_CLASSE] (declar_const[ATRIBUTS_CLASSE] | /*res*/)
						| declar_const[ATRIBUTS_CLASSE] (declar_var[ATRIBUTS_CLASSE] | /*res*/) )?
						FATRIBUTS_CLASSE sepa_inst
	;
bloc_metode:	METODES sepa_inst
				(capzalera_constructor[false])*
				(capzalera_metode[false])*
				FMETODES sepa_inst
	;
bloc_metode_classe: METODES_CLASSE sepa_inst
					(capzalera_metode_classe[false])*
					FMETODES_CLASSE sepa_inst
	;
bloc_implementacio: IMPLEMENTACIO sepa_inst
					(declar_var[IMPLEMENTACIO])?
					(funcio | metode | metode_classe | constructor | destructor)*
					FIMPLEMENTACIO sepa_inst
	;
capzalera_metode[boolean declararAmbit]
				{ ParamInfo []params=new ParamInfo[128]; ParamInfo retParam=null; int Type=0,numParams=0; }
				: METODE id:IDENTIFICADOR 
				  PAREN_ESQ numParams=parametres_formals[params,0] PAREN_DRET
				  (RETORNA retParam=parametres_retorn | /* res */)
				  (sepa_inst)
			{
				// Declara el metode. Resta un al numero de linia, ja que ha parsejat sepa_inst
				declararFuncio(id.getText(),params,numParams,retParam,declararAmbit,LT(1).getLine()-1,METODE);
			}
	;
metode:  capzalera_metode[true] 
		 bloc_instruccions FMETODE (sepa_inst)
		 { 
		 	// Quan es fa match amb aquesta regla, ja s'ha acabat de "parsejar"
			// el metode, i per tant es pot eliminar l'ambit.
			symbolTable.popScope();
			out.writeEndBlock();
		 }
	;
capzalera_metode_classe[boolean declararAmbit]
				{ ParamInfo []params=new ParamInfo[128]; ParamInfo retParam=null; int Type=0,numParams=0; }
				:  	METODE_CLASSE id:IDENTIFICADOR 
					PAREN_ESQ numParams=parametres_formals[params,0] PAREN_DRET
					(RETORNA retParam=parametres_retorn | /* res */)
					(sepa_inst)
			{
				// Declara el metode de classe. Resta un al numero de linia, ja que ha parsejat sepa_inst
				declararFuncio(id.getText(),params,numParams,retParam,declararAmbit,LT(1).getLine()-1,METODE_CLASSE);
			}					
	;
metode_classe:  capzalera_metode_classe[true]
				bloc_instruccions FMETODE_CLASSE (sepa_inst)
		 { 
		 	// Quan es fa match amb aquesta regla, ja s'ha acabat de "parsejar"
			// el metode, i per tant es pot eliminar l'ambit.
			symbolTable.popScope();
			out.writeEndBlock();			
		 }
	;
capzalera_funcio[boolean declararAmbit]
				{ ParamInfo []params=new ParamInfo[128]; ParamInfo retParam=null; int Type=0,numParams=0; }
				:   FUNCIO id:IDENTIFICADOR 
					PAREN_ESQ numParams=parametres_formals[params,0]  PAREN_DRET
					(RETORNA retParam=parametres_retorn | /* res */)
					(sepa_inst)
			{
				// Declara la funcio. Resta un al numero de linia, ja que ha parsejat sepa_inst
				declararFuncio(id.getText(),params,numParams,retParam,declararAmbit,LT(1).getLine()-1,FUNCIO);
			}				
	;	
funcio:  capzalera_funcio[true]
		 bloc_instruccions FFUNCIO (sepa_inst)
		 { 
		 	// Quan es fa match amb aquesta regla, ja s'ha acabat de "parsejar"
			// la funcio, i per tant es pot eliminar l'ambit.
			symbolTable.popScope();
			out.writeEndBlock();			
		 }
	;
capzalera_constructor[boolean declararAmbit]
				{ ParamInfo []params=new ParamInfo[128]; int Type=0,numParams=0; }
				:	CONSTRUCTOR id:IDENTIFICADOR 
					PAREN_ESQ numParams=parametres_formals[params,0] PAREN_DRET
					(sepa_inst)
			{
				// Declara el constructor. Resta un al numero de linia, ja que ha parsejat sepa_inst
				declararFuncio(id.getText(),params,numParams,null,declararAmbit,LT(1).getLine()-1,CONSTRUCTOR);
			}					
	;
constructor:    capzalera_constructor[true]
		 		bloc_instruccions FCONSTRUCTOR (sepa_inst)
		 { 
		 	// Quan es fa match amb aquesta regla, ja s'ha acabat de "parsejar"
			// el constructor, i per tant es pot eliminar l'ambit.
			symbolTable.popScope();
			out.writeEndBlock();			
		 }
	;
capzalera_destructor[boolean declararAmbit]
			{ ParamInfo []params=new ParamInfo[1]; params[0]=null; }
			:  DESTRUCTOR id:IDENTIFICADOR 
			   PAREN_ESQ PAREN_DRET (sepa_inst)
			{
				// Declara el destructor. Resta un al numero de linia, ja que ha parsejat sepa_inst
				declararFuncio(id.getText(),params,0,null,declararAmbit,LT(1).getLine()-1,DESTRUCTOR);
			}
	;
destructor:    	capzalera_destructor[true]
		 		bloc_instruccions FDESTRUCTOR (sepa_inst)
		 { 
		 	// Quan es fa match amb aquesta regla, ja s'ha acabat de "parsejar"
			// el destructor, i per tant es pot eliminar l'ambit.
			symbolTable.popScope();
			out.writeEndBlock();			
		 }
	;

parametres_formals[ParamInfo[] params,int index] returns[int numParametres=0;]
	{ ParamInfo aux=null; }
	: (aux=parametre ((COMA numParametres=parametres_formals[params,index+1]) | /*res*/ { numParametres=index+1; }))?
	{ 	params[index]=aux;	}
	;
parametres_retorn returns [ParamInfo pInfo=null]
	: id:IDENTIFICADOR DOSPUNTS pInfo=tipus 
	{	/* ABANS, la regla era: parametres_formals */
		pInfo.setName(id.getText());
	}
	;	
parametre returns [ParamInfo pInfo=null]
	: id:IDENTIFICADOR DOSPUNTS pInfo=tipus
	{ 
		pInfo.setName(id.getText());
	}
	;	
	
/* ***************** REGLES COMUNES: VARIABLES,DECLARACIONS,INSTRUCCIONS, etc.***************** */
bloc_algorisme: bloc_instruccions
	;
bloc_instruccions:  ( declar_var[RES] (declar_const[RES] | ) instruccions 
					| declar_const[RES] (declar_var[RES] | ) instruccions
					| instruccions
					)
	;

tipus returns [ParamInfo typeInf=null]
	:	(typeInf=tipus_simple | typeInf=tipus_compost)
	;
tipus_simple returns [ParamInfo typeInf=new ParamInfo()]
		{ int Type=0; }
		:(  id:IDENTIFICADOR
		{
			// Si no es tracta de la propia classe que es parseja, es carrega
			// la interficie de la classe en l'ambit actual de la TS
			if (!symbolTable.getParentScopeName().equalsIgnoreCase(id.getText()))
				loadClassToSimbolTable(id.getText(),LT(1).getLine());	
		}
		| Type=tipus_elemental
		)
		{
			if (Type==0)
			{
				// No es tipus basic, es una classe
				typeInf.setIdType(CLASS_TYPE);
				typeInf.setType(getHash(id.getText()));
				typeInf.setClassName(id.getText());
			}
			else
			{
				// Tipus basic
				typeInf.setIdType(ELEMENTAL_TYPE);
				typeInf.setType(Type);
			}
		}
	;
tipus_elemental returns [int Type=0]
	 :   ENTER 		{ Type=NUMERO_ENTER; }
	   | REAL 		{ Type=NUMERO_REAL; }
	   | CARACTER	{ Type=LITERAL_CARACTER; } 
	   | BOOLEA		{ Type=BOOLEA; }
	;
tipus_compost returns [ParamInfo type=null]
	{ ParamInfo typeInf=null; int dim=0; }
	:  TAULA (dim=corxets[dim+1])+ DE typeInf=tipus_simple
	{ 
		type=new ParamInfo();
		type.setType(typeInf.getType());
		type.setIdType(typeInf.getIdType());
		type.setClassName(typeInf.getClassName());
		type.setDimension(dim);
	}
	;
corxets[int dim] returns[int dimension=0]
	: CORXET_ESQ CORXET_DRET
	{ dimension=dim; }
	;

declar_var[int typeDecl]
	: VAR (sepa_inst) (llista_var[typeDecl])* FVAR (sepa_inst)
	;
llista_var[int typeDecl] 
	{ String []ids=new String[128]; int Type=0; ParamInfo typeInf=null; /** PER culpa del sepa_inst, no marca be la linia ***/ }
	: llista_identif[ids,0] DOSPUNTS typeInf=tipus (sepa_inst)
	{
		// ANALISIS SEMANTIC: Declara les variables.
		int i=0;
		Type=typeInf.getType();
		while ((ids[i]!=null) && (i < ids.length))
		{
			declararVariable(ids[i],typeInf.getIdType(),Type,typeInf.getDimension(),typeInf.getClassName(),typeDecl,LT(1).getLine());
			i++;
		}
	}
	;
llista_identif[String[] declName,int index] { declName[index] = ""; }
	: id:IDENTIFICADOR (COMA llista_identif[declName,index+1] | )
	{ declName[index]=id.getText(); }
	;

declar_const[int typeDecl]
	: CONST (sepa_inst) (llista_const[typeDecl])* FCONST (sepa_inst)
	;
llista_const[int typeDecl]
	{ ParamInfo expInf=null,constInf=null; }
	: id:IDENTIFICADOR DOSPUNTS constInf=tipus OP_IGUAL expInf=expressio (sepa_inst)
	{
		// Comprova que sigui correcta la declaracio de la constant
		try
		{
			if (constInf.getIdType()==ELEMENTAL_TYPE)
			{
				checkTypes(constInf,expInf,false,true,LT(1).getLine());	// No permet fer Cast automatic
				declararConstant(id.getText(),constInf.getType(),expInf.getText(),typeDecl,LT(1).getLine());
			}
			else
				throw new NeedElementalTypeException(id.getText(),inFilename,LT(1).getLine());
		}
		catch(PSemanticException sEx)
		{
			reportError(sEx);
		}
	}
	;

// Variable: identificador i pot anar seguin de punt i 
// identificador que seria un atribut/mebre public del objecte
variable returns[ParamInfo factInf=null]
	{ String id="",className=""; }
	: JO (PUNT factInf=post_variable[null])? 
	{ if (factInf==null)
	  {
	  	factInf=new ParamInfo();
		tractarJo(factInf);
	  }
	  else
		factInf.setText("this." + factInf.getText());
	}
	| SUPER { className=toStringSuperClassName(); }
	(
		PUNT factInf=post_variable[null] { factInf.setText("." + factInf.getText()); }
		|
		factInf=crida_funcio[className,className,CONSTRUCTOR] { checkSuperIsFirstStatement(LT(1).getLine()); factInf.setIdType(ACCIO); }
	)
	{ factInf.setText("super" + factInf.getText()); }
	| factInf=post_variable[null]
	;
post_variable[ParamInfo typeInf] returns[ParamInfo factInf=new ParamInfo()]
		  { LinkedList list=new LinkedList(); ParamInfo factInf1=null,factInf2=null,tableInf=null; String className=null; int tCrida; }
		  :(id:IDENTIFICADOR
{
			tCrida=RES;
			if (typeInf!=null)
			{
				if (typeInf.getDimension()==0)
				{
					if (!symbolTable.getParentScopeName().equalsIgnoreCase(typeInf.getClassName()))
					{	// Si no es tracta de la classe que estem parsejant
						factInf=isDeclaredIdOfClass(id.getText(),typeInf,LT(1).getLine());
						className=typeInf.getClassName();
					}
					else  // Es tracta de la classe q estem parsejant, per tant ha de ser visible en ambit actual
						factInf=isDeclaredId(id.getText(),LT(1).getLine(),list);
					factInf.setText(factInf.getText().toLowerCase());  // Passa el text parsejat a minuscules
				}
				else
				{
					factInf.setType(ID_NOT_DECLARED);  // Per a una taula no hi ha membres declarats
					tCrida=TAULA;
				}
			}
			else
			{
				factInf=isDeclaredId(id.getText(),LT(1).getLine(),list);
				if (factInf.getIdType()!=CLASS_DEF)
					factInf.setText(factInf.getText().toLowerCase());  // Passa el text parsejat a minuscules si no es nom d'una classe
			}
			if ((factInf.getType()!=ID_NOT_DECLARED) && (factInf.getSuperClassName()!=null))
				// Si el ID esta declarat, i definit en una altre superclasse, actualitza el nom de la classe.
				className=factInf.getSuperClassName();
}
		  (
		  PUNT factInf1=post_variable[factInf]
		  | (tableInf=index_taula[id.getText()]) { checkAccesTable(factInf,tableInf,LT(1).getLine()); }
			(PUNT factInf1=post_variable[factInf] | /* res */)
		  | factInf2=crida_funcio[id.getText(),className,tCrida]
					  {
							if (factInf2!=null)
							{
								// Tracta una crida a una funcio
								if (factInf2.getType()!=TAULA)
								{
									factInf.setIdType(factInf2.getIdType());
									factInf.setType(factInf2.getType());
									factInf.setClassName(factInf2.getClassName());
									factInf.appendText(factInf2.getText());
								}
								else
								{
									// Membre LONGITUT d'una taula
									factInf.setIdType(factInf2.getIdType());
									factInf.setType(NUMERO_ENTER);
									factInf.setText("length");
								}
							}
					  }
					  
		  	(PUNT factInf1=post_variable[factInf] | /* res */)
		  | /* res */
		  {
					if (((factInf.getIdType()==FUNCIO) || (factInf.getIdType()==ACCIO)) && (factInf2==null) && (factInf1==null))
					{
						// Actualiza per a que doni error de inst. no valida, ja que no ha
						// parsejat per crida (si hagues parsejat crida, factInf1==null)
						factInf.setIdType(CLASS_TYPE);
					}		  
		  }
		  ))
		 {	
		 	try
			{
				// Reporta missatge d'error si no esta declarat i no es una crida a una func.
				if ((factInf.getType()==ID_NOT_DECLARED) && (factInf2==null))
				{	// Actualitzem el text parsejat
					factInf.appendText(id.getText());
					throw new IdNotDeclaredException(id.getText(),inFilename,LT(1).getLine());
				}
				if (factInf1!=null)
				{
					// Tracta un camp o atribut d'una classe
					if (factInf1.getType()!=ID_NOT_DECLARED) // Si esta declarat, actualitzem el tipus
					{
						factInf.setType(factInf1.getType());
						factInf.setIdType(factInf1.getIdType());
						factInf.setDimension(factInf1.getDimension());
						factInf.setClassName(factInf1.getClassName());
						factInf.appendText("." + factInf1.getText());
					}
					else
						// Nomes actualitzem el text parsejat
						factInf.appendText("." + factInf1.getText());					
					
				}
			}
			catch(PSemanticException sEx)
			{
				reportError(sEx);
			}
		  }
	;
	
crida_funcio[String varName,String className,int cridaType] returns[ParamInfo funcInfo=new ParamInfo()]
			{
				int numParams=0;
				LinkedList list=null;
				ParamInfo[] parParams=new ParamInfo[30];
				ParamInfo parInf=new ParamInfo();
				String text="";

				funcInfo.setText("(");	
			}
			:   PAREN_ESQ 
				numParams=parametres_actuals[parParams,0,parInf]
				PAREN_DRET
			{
				try
				{
					// Afegeix els parametres actuals parsejats
					text=parInf.getText();
					if (text!="")
						text=text.substring(0,text.lastIndexOf(','));
					funcInfo.appendText(text);
					funcInfo.appendText(")");
					parInf=null; // marca per el GC				
					list=getListOfFunctions(varName,className,LT(1).getLine());
					// Comprovacions de tipus
					if (list!=null)
					{
						// El ID de la funcio es visible en aquest ambit, comprovem els parametres
						parInf=checkParametersOfFunction(varName,className,cridaType,list,parParams,numParams,LT(1).getLine());
						if (parInf!=null)
						{
							funcInfo.setType(parInf.getType());
							funcInfo.setIdType(parInf.getIdType());
							funcInfo.setClassName(parInf.getClassName());
						}
						else
						{
							// Identificador declarat, pero no coincideix la signatura de la funcio
							if ((cridaType==CONSTRUCTOR) && (numParams==0))
							{
								// El constructor per defecte sempre existeix, encara q no estigui declarat, d'altra manera: error.
							}
							else
							{
								funcInfo.setIdType(ACCIO);  // Per que no peti i digui que no es una instruccio valida
								throw new MetodeNotExistException(getFuncToString(varName,parParams,numParams),
																  inFilename,LT(1).getLine());
							}
						}
					}
					else // Identificador no declarat
					{
						
						if ((cridaType==CONSTRUCTOR) && (numParams==0))
						{
							// El constructor per defecte sempre existeix, encara q no estigui declarat, d'altra manera: error.
						}
						else if (cridaType==TAULA)
						{
							if ((varName.equals(LONGITUT_OF_TAULA))  && (numParams==0))
							{
								// Acceptem el membre de taula q dona la LONGITUT de la taula
								funcInfo.setIdType(FUNCIO);
								funcInfo.setType(TAULA);
							}
							else
							{
								funcInfo.setIdType(ACCIO);  // Per que no peti i digui que no es una instruccio valida
								throw new MetodeNotExistException(getFuncToString(varName,parParams,numParams),
																  inFilename,LT(1).getLine());
							}							
						}
						else
						{
							funcInfo.setIdType(ACCIO);  // Per que no peti i digui que no es una instruccio valida
							throw new MetodeNotExistException(getFuncToString(varName,parParams,numParams),
															  inFilename,LT(1).getLine());							
						}
					}
				}
				catch(PSemanticException sEx)
				{
					reportError(sEx);
				}
			}
	;
parametres_actuals[ParamInfo []parParams,int index,ParamInfo textInf] returns[int numParametres=0;] 
	{ ParamInfo parInf=null; }
	: (parInf=parametre_actual (COMA numParametres=parametres_actuals[parParams,index+1,textInf] | /*res*/ {numParametres=index+1;}))?
	{
		// Comprovacio dels tipus dels parametres
		if (parParams!=null)
		{
				if (parInf!=null)
				{
					parParams[index]=parInf;
					textInf.setText(parInf.getText() + "," + textInf.getText());
				}
		}
	}
	;
parametre_actual returns[ParamInfo paramInf=null]
	: paramInf=expressio
    ;
index_taula[String varName] returns [ParamInfo indexInf=new ParamInfo()]
	{ int dim=0; }
	: (acces_taula[varName,indexInf])+
	;
acces_taula[String varName,ParamInfo indexInf] 
	{ ParamInfo expInf=null; }
	: (CORXET_ESQ expInf=expressio CORXET_DRET)
	{ 
		// Comprova que l'expressio per indexar la taula sigui de tipus entera
		try
		{
			indexInf.incDimension();
			indexInf.appendText("[" + expInf.getText() + "]");
			if (expInf.getType()!=NUMERO_ENTER)
				throw new RequiredEnterForIndexException(varName,inFilename,LT(1).getLine());
		}
		catch(PSemanticException sEx)
		{
			reportError(sEx);
		}
	}
	;
op_suma_resta returns [String val=null]
	:( OP_SUMA {val="+";}
	| OP_RESTA {val="-";}
	)
	;
op_mult_div returns [String val=null]
	:( OP_MULT {val="*";}
	| OP_DIV {val="/";}
	)
	;
op_mod returns [String val=null]
	:( MOD {val="%";}
	)
	;
op_div returns [String val=null]
	:( DIV {val="/";}
	)
	;	
op_relacional returns [String val=null]
	:(  OP_IGUAL {val="==";} | OP_DIFERENT {val="!=";} 
	| OP_MPETIT {val="<";}| OP_MPETIT_IGUAL {val="<=";}
	| OP_MGRAN {val=">";} | OP_MGRAN_IGUAL {val=">=";}
	| OP_I {val="&&";} | OP_O {val="||";}
	)
	;
factor returns [ParamInfo factInf=new ParamInfo()] { String text=null; int type=0,dim=0; ParamInfo factInf1=null; }
		:
		( id1:NUMERO_ENTER 							{ type=NUMERO_ENTER; text=id1.getText();		}
		| id2:NUMERO_REAL  							{ type=NUMERO_REAL; text=id2.getText();  		}
		| id3:LITERAL_CARACTER 						{ type=LITERAL_CARACTER; text=id3.getText();	}
		| id4:LITERAL_CADENA    					{ type=LITERAL_CADENA; text=id4.getText();		}
		| text=val_boolea	    					{ type=BOOLEA; 									}
		| factInf1=variable			 				{ type=factInf1.getType(); text=factInf1.getText(); dim=factInf1.getDimension(); }
		| PAREN_ESQ factInf1=expressio PAREN_DRET 	{ type=factInf1.getType(); text=factInf1.getText(); dim=factInf1.getDimension(); }
		| OP_NO factInf1=factor 					{ type=factInf1.getType(); text="(!" + factInf1.getText() + ")"; 
																										dim=factInf1.getDimension(); }
		| NULL		{ type=NULL; dim=0; text="null"; }
		)
		{
			factInf.setType(type);
			factInf.setText(text);
			factInf.setDimension(dim);
			if (factInf1!=null)
			{
				factInf.setIdType(factInf1.getIdType());
				factInf.setClassName(factInf1.getClassName());
			}
		}
	;
val_boolea returns [String val=null]
	:( CERT {val="true";}
	 | FALS {val="false";}
	 )
	;
operant returns [ParamInfo factInf=new ParamInfo()] 
	{ int Type=0,Type1=0,Type2=0; ParamInfo factInf1=null,factInf2=null; String op="",op2="";}
	: factInf1=factor ((op=op_mult_div factInf2=operant) | 
					   (op=op_mod factInf2=operant) | (op2=op_div factInf2=operant) | /* res */ )
	{
	  // ANALISIS SEMANTIC: comprovar de tipus
	  try
	  {
		  Type1=factInf1.getType();
		  if (factInf2!=null) Type2=factInf2.getType();
		  if (Type2==0) 
			Type=Type1;
		  else
		  {
			if (op2!="")
			{
				// Tractem DIV (divisio entera): comprova que els dos factors siguin enters
				if ((Type1!=NUMERO_ENTER) || (Type2!=NUMERO_ENTER))
					throw new RequiredEnterException("els operadors del div",inFilename,LT(1).getLine());
				op=op2;
			}
			if (op.equals("%"))
			{
				// Si es MOD, comprova que els dos factors siguin enters
				if ((Type1!=NUMERO_ENTER) || (Type2!=NUMERO_ENTER))
					throw new RequiredEnterException("els operadors del mod",inFilename,LT(1).getLine());
			}
			if (factInf2!=null)
				Type=checkTypes(factInf1,factInf2,true,true,LT(1).getLine());		
		  }
		  // Text parsejat
		  factInf.setType(Type);
		  factInf.setIdType(factInf1.getIdType());
		  factInf.setClassName(factInf1.getClassName());
		  factInf.setDimension(factInf1.getDimension());  
		  if (factInf2!=null) factInf.setText(factInf1.getText() + op + factInf2.getText());
		  else factInf.setText(factInf1.getText());
	   }
	   catch (PSemanticException sEx)
	   {
	   	  reportError(sEx);
	   }
	}
    ;
expressio_simple returns [ParamInfo factInf=new ParamInfo()]
	{ int Type=0,Type1=0,Type2=0; ParamInfo factInf1=null,factInf2=null; String op="",signe=""; }
	: 
	(signe=op_suma_resta | ) factInf1=operant (op=op_suma_resta factInf2=expressio_simple | /* res */)
	{
	  // ANALISIS SEMANTIC: comprovar de tipus
	  Type1=factInf1.getType();
	  if (factInf2!=null) Type2=factInf2.getType();
      if (Type2==0) 
		Type=Type1;
	  else
	  {
		if (factInf2!=null)
			Type=checkTypes(factInf1,factInf2,true,true,LT(1).getLine());
	  }
	  factInf.setType(Type);
	  factInf.setIdType(factInf1.getIdType());
	  factInf.setClassName(factInf1.getClassName());
	  factInf.setDimension(factInf1.getDimension());
	  if (factInf2!=null) factInf.setText(signe + factInf1.getText() + op + factInf2.getText());
	  else factInf.setText(signe + factInf1.getText());	  
	}
	;
expressio returns[ParamInfo factInf=new ParamInfo()] { int Type=0,Type1=0,Type2=0; String op=""; ParamInfo factInf1=null,factInf2=null; }
			:
			(
			factInf1=expressio_simple (op=op_relacional factInf2=expressio_simple | /* res */)
			| factInf=inst_crear
			| factInf=inst_llegir
			)
			{
	  		  // ANALISIS SEMANTIC: comprovar de tipus
			  if (factInf1!=null)
			  {
			  	  // Parseja expressio
				  Type1=factInf1.getType();
				  if (factInf2!=null) Type2=factInf2.getType();
				  if (Type2==0) 
					Type=Type1;
				  else
				  {
					if (factInf2!=null)
						Type=checkTypes(factInf1,factInf2,true,true,LT(1).getLine());
					Type=BOOLEA;				
				  }
				  if (factInf2!=null) factInf.setText(factInf1.getText() + op + factInf2.getText());
				  else factInf.setText(factInf1.getText());
				  factInf.setDimension(factInf1.getDimension());
				  factInf.setType(Type);
				  factInf.setIdType(factInf1.getIdType());
				  factInf.setClassName(factInf1.getClassName());
			  }
			}
	;
inst_crear returns[ParamInfo crearInf=new ParamInfo()]
			{ ParamInfo typeInf=null,tableInf=null,crida=null; String className=null; }
			:( CREAR typeInf=tipus {
								if (!symbolTable.getParentScopeName().equalsIgnoreCase(typeInf.getClassName()))
									// Si no es tracta de la classe que estem parsejant
									className=typeInf.getClassName();
							}
				 (
				 (tableInf=index_taula["crear"])
				 |
				 crida=crida_funcio[typeInf.getClassName(),className,CONSTRUCTOR]
				 )
			 )
			{
				if (typeInf!=null)
				{
					crearInf.setType(typeInf.getType());
					crearInf.setIdType(CLASS_TYPE);
					crearInf.setClassName(typeInf.getClassName());
					crearInf.setDimension(0);
					crearInf.setText(out.getExpressionCrear(typeInf.getType(),typeInf.getClassName()));
				}
				if (tableInf!=null)
				{
					crearInf.setDimension(tableInf.getDimension());
					crearInf.setText(crearInf.getText() +  tableInf.getText());
				}
				if (crida!=null)
					crearInf.setText(crearInf.getText() + crida.getText());
			}
	;
inst_llegir returns[ParamInfo llegirInf=new ParamInfo()]
	: LLEGIR PAREN_ESQ PAREN_DRET
	{ 
		llegirInf.setType(LLEGIR);
		llegirInf.setDimension(0);
	}
	;

/* **** Instruccions **** */
instruccions: (instruccio_ratlla (sepa_inst) )*
	;
instruccio_ratlla: instruccio_simple (PUNTCOMA instruccio_ratlla | /* res */ )
	;
instruccio_simple: 	(
					 inst_assignacio_crida_funcio
					 | inst_si_llavors
					 | inst_cas
					 | inst_mentre
					 | inst_per
					 | inst_res
					 | inst_escriure
					 | inst_escriureln
					 | inst_destruir
					 | inst_retorna { /* Dintre algoritme no, dintre de l'ambit una propietat per saber kin tipus es */}
			  		)
					{ isFirstInstruction=false; /* S'ha parsejat una inst., ja no es la primera */ }
	;
inst_escriure { ParamInfo expInf=null; }
		: ESCRIURE PAREN_ESQ (expInf=expressio)? PAREN_DRET
		{ 
			if (expInf==null)
				out.writeInstEscriure("");
			else
				out.writeInstEscriure(expInf.getText());
		}
	;
inst_escriureln { ParamInfo expInf=null; }
		: ESCRIURELN PAREN_ESQ (expInf=expressio)? PAREN_DRET
		{ 
			if (expInf==null)
				out.writeInstEscriureln("");
			else
				out.writeInstEscriureln(expInf.getText());		
		}
	;	
inst_assignacio_crida_funcio { ParamInfo sourceInf=null,destInf=null; }
				:
				destInf=variable { auxExpAssig=destInf; }
				(OP_ASSIGNACIO sourceInf=expressio
						{ tractarInstAssig(destInf,sourceInf,LT(1).getLine()); }
				| { /* En aquest cas es una crida */ tractarInstCrida(destInf,LT(1).getLine()); }
				)
				{ auxExpAssig=null; }
	;
inst_si_llavors  { ParamInfo condInf=null; }
				 : SI condInf=expressio LLAVORS
				 { tractarInstSi(condInf,LT(1).getLine()); }
				 (
				  instruccio_ratlla
				 (ALTRAMENT
				 { out.writeEndBlock(); out.writeInstAltrament(); }
				 instruccio_ratlla | )
				 |
				 (sepa_inst) instruccions
				 (ALTRAMENT
				 { out.writeEndBlock(); out.writeInstAltrament(); }
				 (sepa_inst) instruccions | )
				 )
				 FSI
				 {
			 		out.writeEndBlock();
				 }
	;
cond_cas[boolean firstCond] returns[boolean fCond=false;]
		 //fCond=false Ja que sempre ja s'haura tractat la primera condicio del CAS
		 { ParamInfo condInf=null; }
		 :(CORXET_ESQ CORXET_DRET condInf=expressio OP_CAS 
		 { tractarInstCas(condInf,firstCond,LT(1).getLine()); }
		   (sepa_inst)? instruccions)
		 {
			out.writeEndBlock();
		 }
	;
inst_cas { boolean firstCond=true; }
		: CAS (sepa_inst)
		  (firstCond=cond_cas[firstCond])+
		  FCAS
	;
capzalera_mentre { ParamInfo condInf=null; }
			: MENTRE condInf=expressio FER
			{
				try
				{
					if (condInf.getType()!=BOOLEA) 
						throw new RequiredBooleaException("la condicio del mentre",inFilename,LT(1).getLine());
				  	out.writeInstHeadMentre(condInf.getText());
				}
				catch(PSemanticException sEx)
				{
					reportError(sEx);
				}
			}
	;
inst_mentre:  capzalera_mentre
			  (instruccio_ratlla | (sepa_inst) instruccions)
			  FMENTRE
			 {
			 	out.writeEndBlock();
			 }
	;
capzalera_per { ParamInfo factInf=null,iniExp=null,endExp=null,pasExp=null; }
	: PER (factInf=variable OP_ASSIGNACIO iniExp=expressio) FINS endExp=expressio (PAS pasExp=expressio)? FER
	{
		tractarInstPer(factInf,iniExp,endExp,pasExp,LT(1).getLine());
	}
	;
inst_per:	capzalera_per
			( instruccio_ratlla | (sepa_inst) instruccions)
			FPER
			{
			 	out.writeEndBlock();
			}
	;
inst_res:	RES { out.writeInstRes(); }
	;
inst_retorna:  RETORNA id:IDENTIFICADOR { out.writeInstRetorna(id.getText()); }
	;
inst_destruir: DESTRUIR id:IDENTIFICADOR { out.writeInstDestruir(id.getText()); }
	;
