/*  CPC (Compilador de PseudoCodi)
 *  Autor: Carles Royan Salvatella.
 *  Ultima modificacio: 18/06/2003
 *  Versio: 1.0
 */
 
 import java.io.*;
 import antlr.TokenStreamRecognitionException;
/**
 * <p>Classe que conte la funcio main. Es l'encarregada de parsejar
 *	la linia de comandes per trobar les opcions que introdueix l'usuari
 *	i els fitxers d'entrada. Despres crida al CpcParser per a que tradueixi
 *	els fitxers.
 * </p>
 * @author Carles Royan Salvatella
 * @version 1.0 - Data: 18/06/2003
 */
 class cpc
 {
 	/** Versio del traductor */
 	private final static String VERSION="1.0";
	/** Programa que s'executa per compilar els fitxers (opcio: -j) */
	private static String compileComand= "javac";
	/** Especifica si un cop traduits, s'han de compilar els fitxers */
	private static boolean finalCompile=false;
	/** Especifica si el mode verbose esta activat */	
	private static boolean verboseMode=false;
	/** Classe que implementa l'analisi lexica */
	private static CpcLexer lexer;
	/** Classe que implementa l'analisi sintactica */	
	private static CpcParser parser;
	
	/** Funcio principal que tracta la linia de comandes. */
 	public static void main(String[] args)
	{
		// Inici del traductor
		int numParam=0;
		
		if (args.length > 0)
		{
			numParam=checkParams(args);
			if (numParam!=-1)
				checkFiles(args,numParam);
		}
		else
			printCommandHelp();
	}
	/** Tracta la llista de fitxers, tot passant-los al CpcParser */
	private static void checkFiles(String[] args,int numParam)
	{
		while (numParam < args.length)
		{
			parseFile(args[numParam]);
			numParam++;
		}
	}
	/** Parseja un fitxer de pseudocodi */	
	private static void parseFile(String filename)
	{
		try
		{
			String fileOut="";
			int numError=0;
			
			lexer=new CpcLexer(new DataInputStream(new FileInputStream(filename)));
			parser=new CpcParser(lexer);
			parser.setVersion(VERSION);
			parser.setVerboseMode(verboseMode);
			parser.setInputFilename(filename);
			numError=parser.programa(false);
			// Si s'ha activat el flag de compilar i no hi ha errors, compila el fitxer java resultant
			if ((finalCompile) && (numError==0))
			{
				fileOut=filename.substring(0,filename.lastIndexOf('.'))+".java";			
				if (verboseMode)
					System.err.println("[compilant el fitxer " + fileOut + "]");
				compileJavaFile(fileOut);
			}
		}
		catch (FileNotFoundException e)
		{
			System.err.println("Error: No existeix el fitxer o el directori: " + filename);
		}
		catch (TokenStreamRecognitionException e)
		{
			// Error lexic (actua en mode panic)
			System.err.println("Error lexic: ");
			System.err.println(e);
			// Tanca i esborra el fitxer de sortida generat fins el moment
			parser.closeOutFile();
			(new File(parser.getOutFile())).delete();
			System.exit(-1);
		}
		catch (Exception e)
		{
			// Imprimeix la pila
			System.err.println("Error inesperat:");
			e.printStackTrace();
			System.exit(-1);
		}
	}
	/** Comprova si s'han passat opcions en la linia de comandes, i en aquest
		cas, les tracta.  Retorna l'index on hi ha els fitxers a tractar */
	private static int checkParams(String[] args)
	{
		int numParam=0;
		boolean parametre=true;
		
		while (parametre)
		{
			if (args[numParam].charAt(0)=='-')
			{
				switch (args[numParam].charAt(1))
				{
					case 'j':
						// Tracta el parametre -j: Compila el fitxer traduit amb el compilador javac
						finalCompile=true;					
						break;
					case 'v':
						// Tracta el parametre -v: Imprimeix missatges sobre el que fa el traductor
						verboseMode=true;						
						break;
					default:
						printCommandHelp();
						numParam=-2;
						parametre=false;
						break;
				}
				numParam++;
			}
			else
				parametre=false;
		}
		return numParam;
	}
	/** Compila el fitxer passat per paramatre */	
	private static void compileJavaFile(String filename)
	{
		Runtime rt = Runtime.getRuntime();
		Process p;
		String jComand="";

		try 
		{
			jComand=compileComand + " " + filename;
			p = rt.exec(jComand);
			p.waitFor();
		}
		catch (Exception e) 
		{
			System.err.println(e);		
		}				
	}
	/** Imprimeix per la sortida estandard una petita ajuda de les comandes
		disponibles per l'usuari. */
	private static void printCommandHelp()
	{
		System.out.println("Us: java cpc [opcions] fitxers");
		System.out.println("Opcions disponibles:");
		System.out.println("-j \t\t Compila el fitxer traduit amb el compilador javac");
		System.out.println("-v \t\t Imprimeix missatges sobre el que fa el traductor");
		System.out.println();
	}
 }
